class R {
  /**
  Get the line description around the given position.
  */
  lineAt(t) {
    if (t < 0 || t > this.length)
      throw new RangeError(`Invalid position ${t} in document of length ${this.length}`);
    return this.lineInner(t, !1, 1, 0);
  }
  /**
  Get the description for the given (1-based) line number.
  */
  line(t) {
    if (t < 1 || t > this.lines)
      throw new RangeError(`Invalid line number ${t} in ${this.lines}-line document`);
    return this.lineInner(t, !0, 1, 0);
  }
  /**
  Replace a range of the text with the given content.
  */
  replace(t, e, i) {
    let n = [];
    return this.decompose(
      0,
      t,
      n,
      2
      /* Open.To */
    ), i.length && i.decompose(
      0,
      i.length,
      n,
      3
      /* Open.To */
    ), this.decompose(
      e,
      this.length,
      n,
      1
      /* Open.From */
    ), mt.from(n, this.length - (e - t) + i.length);
  }
  /**
  Append another document to this one.
  */
  append(t) {
    return this.replace(this.length, this.length, t);
  }
  /**
  Retrieve the text between the given points.
  */
  slice(t, e = this.length) {
    let i = [];
    return this.decompose(t, e, i, 0), mt.from(i, e - t);
  }
  /**
  Test whether this text is equal to another instance.
  */
  eq(t) {
    if (t == this)
      return !0;
    if (t.length != this.length || t.lines != this.lines)
      return !1;
    let e = this.scanIdentical(t, 1), i = this.length - this.scanIdentical(t, -1), n = new we(this), s = new we(t);
    for (let l = e, o = e; ; ) {
      if (n.next(l), s.next(l), l = 0, n.lineBreak != s.lineBreak || n.done != s.done || n.value != s.value)
        return !1;
      if (o += n.value.length, n.done || o >= i)
        return !0;
    }
  }
  /**
  Iterate over the text. When `dir` is `-1`, iteration happens
  from end to start. This will return lines and the breaks between
  them as separate strings.
  */
  iter(t = 1) {
    return new we(this, t);
  }
  /**
  Iterate over a range of the text. When `from` > `to`, the
  iterator will run in reverse.
  */
  iterRange(t, e = this.length) {
    return new Os(this, t, e);
  }
  /**
  Return a cursor that iterates over the given range of lines,
  _without_ returning the line breaks between, and yielding empty
  strings for empty lines.
  
  When `from` and `to` are given, they should be 1-based line numbers.
  */
  iterLines(t, e) {
    let i;
    if (t == null)
      i = this.iter();
    else {
      e == null && (e = this.lines + 1);
      let n = this.line(t).from;
      i = this.iterRange(n, Math.max(n, e == this.lines + 1 ? this.length : e <= 1 ? 0 : this.line(e - 1).to));
    }
    return new Ds(i);
  }
  /**
  Return the document as a string, using newline characters to
  separate lines.
  */
  toString() {
    return this.sliceString(0);
  }
  /**
  Convert the document to an array of lines (which can be
  deserialized again via [`Text.of`](https://codemirror.net/6/docs/ref/#state.Text^of)).
  */
  toJSON() {
    let t = [];
    return this.flatten(t), t;
  }
  /**
  @internal
  */
  constructor() {
  }
  /**
  Create a `Text` instance for the given array of lines.
  */
  static of(t) {
    if (t.length == 0)
      throw new RangeError("A document must have at least one line");
    return t.length == 1 && !t[0] ? R.empty : t.length <= 32 ? new z(t) : mt.from(z.split(t, []));
  }
}
class z extends R {
  constructor(t, e = $r(t)) {
    super(), this.text = t, this.length = e;
  }
  get lines() {
    return this.text.length;
  }
  get children() {
    return null;
  }
  lineInner(t, e, i, n) {
    for (let s = 0; ; s++) {
      let l = this.text[s], o = n + l.length;
      if ((e ? i : o) >= t)
        return new Yr(n, o, i, l);
      n = o + 1, i++;
    }
  }
  decompose(t, e, i, n) {
    let s = t <= 0 && e >= this.length ? this : new z(xn(this.text, t, e), Math.min(e, this.length) - Math.max(0, t));
    if (n & 1) {
      let l = i.pop(), o = Ye(s.text, l.text.slice(), 0, s.length);
      if (o.length <= 32)
        i.push(new z(o, l.length + s.length));
      else {
        let h = o.length >> 1;
        i.push(new z(o.slice(0, h)), new z(o.slice(h)));
      }
    } else
      i.push(s);
  }
  replace(t, e, i) {
    if (!(i instanceof z))
      return super.replace(t, e, i);
    let n = Ye(this.text, Ye(i.text, xn(this.text, 0, t)), e), s = this.length + i.length - (e - t);
    return n.length <= 32 ? new z(n, s) : mt.from(z.split(n, []), s);
  }
  sliceString(t, e = this.length, i = `
`) {
    let n = "";
    for (let s = 0, l = 0; s <= e && l < this.text.length; l++) {
      let o = this.text[l], h = s + o.length;
      s > t && l && (n += i), t < h && e > s && (n += o.slice(Math.max(0, t - s), e - s)), s = h + 1;
    }
    return n;
  }
  flatten(t) {
    for (let e of this.text)
      t.push(e);
  }
  scanIdentical() {
    return 0;
  }
  static split(t, e) {
    let i = [], n = -1;
    for (let s of t)
      i.push(s), n += s.length + 1, i.length == 32 && (e.push(new z(i, n)), i = [], n = -1);
    return n > -1 && e.push(new z(i, n)), e;
  }
}
class mt extends R {
  constructor(t, e) {
    super(), this.children = t, this.length = e, this.lines = 0;
    for (let i of t)
      this.lines += i.lines;
  }
  lineInner(t, e, i, n) {
    for (let s = 0; ; s++) {
      let l = this.children[s], o = n + l.length, h = i + l.lines - 1;
      if ((e ? h : o) >= t)
        return l.lineInner(t, e, i, n);
      n = o + 1, i = h + 1;
    }
  }
  decompose(t, e, i, n) {
    for (let s = 0, l = 0; l <= e && s < this.children.length; s++) {
      let o = this.children[s], h = l + o.length;
      if (t <= h && e >= l) {
        let a = n & ((l <= t ? 1 : 0) | (h >= e ? 2 : 0));
        l >= t && h <= e && !a ? i.push(o) : o.decompose(t - l, e - l, i, a);
      }
      l = h + 1;
    }
  }
  replace(t, e, i) {
    if (i.lines < this.lines)
      for (let n = 0, s = 0; n < this.children.length; n++) {
        let l = this.children[n], o = s + l.length;
        if (t >= s && e <= o) {
          let h = l.replace(t - s, e - s, i), a = this.lines - l.lines + h.lines;
          if (h.lines < a >> 5 - 1 && h.lines > a >> 5 + 1) {
            let f = this.children.slice();
            return f[n] = h, new mt(f, this.length - (e - t) + i.length);
          }
          return super.replace(s, o, h);
        }
        s = o + 1;
      }
    return super.replace(t, e, i);
  }
  sliceString(t, e = this.length, i = `
`) {
    let n = "";
    for (let s = 0, l = 0; s < this.children.length && l <= e; s++) {
      let o = this.children[s], h = l + o.length;
      l > t && s && (n += i), t < h && e > l && (n += o.sliceString(t - l, e - l, i)), l = h + 1;
    }
    return n;
  }
  flatten(t) {
    for (let e of this.children)
      e.flatten(t);
  }
  scanIdentical(t, e) {
    if (!(t instanceof mt))
      return 0;
    let i = 0, [n, s, l, o] = e > 0 ? [0, 0, this.children.length, t.children.length] : [this.children.length - 1, t.children.length - 1, -1, -1];
    for (; ; n += e, s += e) {
      if (n == l || s == o)
        return i;
      let h = this.children[n], a = t.children[s];
      if (h != a)
        return i + h.scanIdentical(a, e);
      i += h.length + 1;
    }
  }
  static from(t, e = t.reduce((i, n) => i + n.length + 1, -1)) {
    let i = 0;
    for (let d of t)
      i += d.lines;
    if (i < 32) {
      let d = [];
      for (let g of t)
        g.flatten(d);
      return new z(d, e);
    }
    let n = Math.max(
      32,
      i >> 5
      /* Tree.BranchShift */
    ), s = n << 1, l = n >> 1, o = [], h = 0, a = -1, f = [];
    function c(d) {
      let g;
      if (d.lines > s && d instanceof mt)
        for (let m of d.children)
          c(m);
      else
        d.lines > l && (h > l || !h) ? (u(), o.push(d)) : d instanceof z && h && (g = f[f.length - 1]) instanceof z && d.lines + g.lines <= 32 ? (h += d.lines, a += d.length + 1, f[f.length - 1] = new z(g.text.concat(d.text), g.length + 1 + d.length)) : (h + d.lines > n && u(), h += d.lines, a += d.length + 1, f.push(d));
    }
    function u() {
      h != 0 && (o.push(f.length == 1 ? f[0] : mt.from(f, a)), a = -1, h = f.length = 0);
    }
    for (let d of t)
      c(d);
    return u(), o.length == 1 ? o[0] : new mt(o, e);
  }
}
R.empty = /* @__PURE__ */ new z([""], 0);
function $r(r) {
  let t = -1;
  for (let e of r)
    t += e.length + 1;
  return t;
}
function Ye(r, t, e = 0, i = 1e9) {
  for (let n = 0, s = 0, l = !0; s < r.length && n <= i; s++) {
    let o = r[s], h = n + o.length;
    h >= e && (h > i && (o = o.slice(0, i - n)), n < e && (o = o.slice(e - n)), l ? (t[t.length - 1] += o, l = !1) : t.push(o)), n = h + 1;
  }
  return t;
}
function xn(r, t, e) {
  return Ye(r, [""], t, e);
}
class we {
  constructor(t, e = 1) {
    this.dir = e, this.done = !1, this.lineBreak = !1, this.value = "", this.nodes = [t], this.offsets = [e > 0 ? 1 : (t instanceof z ? t.text.length : t.children.length) << 1];
  }
  nextInner(t, e) {
    for (this.done = this.lineBreak = !1; ; ) {
      let i = this.nodes.length - 1, n = this.nodes[i], s = this.offsets[i], l = s >> 1, o = n instanceof z ? n.text.length : n.children.length;
      if (l == (e > 0 ? o : 0)) {
        if (i == 0)
          return this.done = !0, this.value = "", this;
        e > 0 && this.offsets[i - 1]++, this.nodes.pop(), this.offsets.pop();
      } else if ((s & 1) == (e > 0 ? 0 : 1)) {
        if (this.offsets[i] += e, t == 0)
          return this.lineBreak = !0, this.value = `
`, this;
        t--;
      } else if (n instanceof z) {
        let h = n.text[l + (e < 0 ? -1 : 0)];
        if (this.offsets[i] += e, h.length > Math.max(0, t))
          return this.value = t == 0 ? h : e > 0 ? h.slice(t) : h.slice(0, h.length - t), this;
        t -= h.length;
      } else {
        let h = n.children[l + (e < 0 ? -1 : 0)];
        t > h.length ? (t -= h.length, this.offsets[i] += e) : (e < 0 && this.offsets[i]--, this.nodes.push(h), this.offsets.push(e > 0 ? 1 : (h instanceof z ? h.text.length : h.children.length) << 1));
      }
    }
  }
  next(t = 0) {
    return t < 0 && (this.nextInner(-t, -this.dir), t = this.value.length), this.nextInner(t, this.dir);
  }
}
class Os {
  constructor(t, e, i) {
    this.value = "", this.done = !1, this.cursor = new we(t, e > i ? -1 : 1), this.pos = e > i ? t.length : 0, this.from = Math.min(e, i), this.to = Math.max(e, i);
  }
  nextInner(t, e) {
    if (e < 0 ? this.pos <= this.from : this.pos >= this.to)
      return this.value = "", this.done = !0, this;
    t += Math.max(0, e < 0 ? this.pos - this.to : this.from - this.pos);
    let i = e < 0 ? this.pos - this.from : this.to - this.pos;
    t > i && (t = i), i -= t;
    let { value: n } = this.cursor.next(t);
    return this.pos += (n.length + t) * e, this.value = n.length <= i ? n : e < 0 ? n.slice(n.length - i) : n.slice(0, i), this.done = !this.value, this;
  }
  next(t = 0) {
    return t < 0 ? t = Math.max(t, this.from - this.pos) : t > 0 && (t = Math.min(t, this.to - this.pos)), this.nextInner(t, this.cursor.dir);
  }
  get lineBreak() {
    return this.cursor.lineBreak && this.value != "";
  }
}
class Ds {
  constructor(t) {
    this.inner = t, this.afterBreak = !0, this.value = "", this.done = !1;
  }
  next(t = 0) {
    let { done: e, lineBreak: i, value: n } = this.inner.next(t);
    return e ? (this.done = !0, this.value = "") : i ? this.afterBreak ? this.value = "" : (this.afterBreak = !0, this.next()) : (this.value = n, this.afterBreak = !1), this;
  }
  get lineBreak() {
    return !1;
  }
}
typeof Symbol < "u" && (R.prototype[Symbol.iterator] = function() {
  return this.iter();
}, we.prototype[Symbol.iterator] = Os.prototype[Symbol.iterator] = Ds.prototype[Symbol.iterator] = function() {
  return this;
});
class Yr {
  /**
  @internal
  */
  constructor(t, e, i, n) {
    this.from = t, this.to = e, this.number = i, this.text = n;
  }
  /**
  The length of the line (not including any line break after it).
  */
  get length() {
    return this.to - this.from;
  }
}
let Zt = /* @__PURE__ */ "lc,34,7n,7,7b,19,,,,2,,2,,,20,b,1c,l,g,,2t,7,2,6,2,2,,4,z,,u,r,2j,b,1m,9,9,,o,4,,9,,3,,5,17,3,3b,f,,w,1j,,,,4,8,4,,3,7,a,2,t,,1m,,,,2,4,8,,9,,a,2,q,,2,2,1l,,4,2,4,2,2,3,3,,u,2,3,,b,2,1l,,4,5,,2,4,,k,2,m,6,,,1m,,,2,,4,8,,7,3,a,2,u,,1n,,,,c,,9,,14,,3,,1l,3,5,3,,4,7,2,b,2,t,,1m,,2,,2,,3,,5,2,7,2,b,2,s,2,1l,2,,,2,4,8,,9,,a,2,t,,20,,4,,2,3,,,8,,29,,2,7,c,8,2q,,2,9,b,6,22,2,r,,,,,,1j,e,,5,,2,5,b,,10,9,,2u,4,,6,,2,2,2,p,2,4,3,g,4,d,,2,2,6,,f,,jj,3,qa,3,t,3,t,2,u,2,1s,2,,7,8,,2,b,9,,19,3,3b,2,y,,3a,3,4,2,9,,6,3,63,2,2,,1m,,,7,,,,,2,8,6,a,2,,1c,h,1r,4,1c,7,,,5,,14,9,c,2,w,4,2,2,,3,1k,,,2,3,,,3,1m,8,2,2,48,3,,d,,7,4,,6,,3,2,5i,1m,,5,ek,,5f,x,2da,3,3x,,2o,w,fe,6,2x,2,n9w,4,,a,w,2,28,2,7k,,3,,4,,p,2,5,,47,2,q,i,d,,12,8,p,b,1a,3,1c,,2,4,2,2,13,,1v,6,2,2,2,2,c,,8,,1b,,1f,,,3,2,2,5,2,,,16,2,8,,6m,,2,,4,,fn4,,kh,g,g,g,a6,2,gt,,6a,,45,5,1ae,3,,2,5,4,14,3,4,,4l,2,fx,4,ar,2,49,b,4w,,1i,f,1k,3,1d,4,2,2,1x,3,10,5,,8,1q,,c,2,1g,9,a,4,2,,2n,3,2,,,2,6,,4g,,3,8,l,2,1l,2,,,,,m,,e,7,3,5,5f,8,2,3,,,n,,29,,2,6,,,2,,,2,,2,6j,,2,4,6,2,,2,r,2,2d,8,2,,,2,2y,,,,2,6,,,2t,3,2,4,,5,77,9,,2,6t,,a,2,,,4,,40,4,2,2,4,,w,a,14,6,2,4,8,,9,6,2,3,1a,d,,2,ba,7,,6,,,2a,m,2,7,,2,,2,3e,6,3,,,2,,7,,,20,2,3,,,,9n,2,f0b,5,1n,7,t4,,1r,4,29,,f5k,2,43q,,,3,4,5,8,8,2,7,u,4,44,3,1iz,1j,4,1e,8,,e,,m,5,,f,11s,7,,h,2,7,,2,,5,79,7,c5,4,15s,7,31,7,240,5,gx7k,2o,3k,6o".split(",").map((r) => r ? parseInt(r, 36) : 1);
for (let r = 1; r < Zt.length; r++)
  Zt[r] += Zt[r - 1];
function Gr(r) {
  for (let t = 1; t < Zt.length; t += 2)
    if (Zt[t] > r)
      return Zt[t - 1] <= r;
  return !1;
}
function kn(r) {
  return r >= 127462 && r <= 127487;
}
const vn = 8205;
function kt(r, t, e = !0, i = !0) {
  return (e ? Ts : Jr)(r, t, i);
}
function Ts(r, t, e) {
  if (t == r.length)
    return t;
  t && Ps(r.charCodeAt(t)) && Bs(r.charCodeAt(t - 1)) && t--;
  let i = ui(r, t);
  for (t += Sn(i); t < r.length; ) {
    let n = ui(r, t);
    if (i == vn || n == vn || e && Gr(n))
      t += Sn(n), i = n;
    else if (kn(n)) {
      let s = 0, l = t - 2;
      for (; l >= 0 && kn(ui(r, l)); )
        s++, l -= 2;
      if (s % 2 == 0)
        break;
      t += 2;
    } else
      break;
  }
  return t;
}
function Jr(r, t, e) {
  for (; t > 0; ) {
    let i = Ts(r, t - 2, e);
    if (i < t)
      return i;
    t--;
  }
  return 0;
}
function Ps(r) {
  return r >= 56320 && r < 57344;
}
function Bs(r) {
  return r >= 55296 && r < 56320;
}
function ui(r, t) {
  let e = r.charCodeAt(t);
  if (!Bs(e) || t + 1 == r.length)
    return e;
  let i = r.charCodeAt(t + 1);
  return Ps(i) ? (e - 55296 << 10) + (i - 56320) + 65536 : e;
}
function Sn(r) {
  return r < 65536 ? 1 : 2;
}
const Mi = /\r\n?|\n/;
var nt = /* @__PURE__ */ function(r) {
  return r[r.Simple = 0] = "Simple", r[r.TrackDel = 1] = "TrackDel", r[r.TrackBefore = 2] = "TrackBefore", r[r.TrackAfter = 3] = "TrackAfter", r;
}(nt || (nt = {}));
class At {
  // Sections are encoded as pairs of integers. The first is the
  // length in the current document, and the second is -1 for
  // unaffected sections, and the length of the replacement content
  // otherwise. So an insertion would be (0, n>0), a deletion (n>0,
  // 0), and a replacement two positive numbers.
  /**
  @internal
  */
  constructor(t) {
    this.sections = t;
  }
  /**
  The length of the document before the change.
  */
  get length() {
    let t = 0;
    for (let e = 0; e < this.sections.length; e += 2)
      t += this.sections[e];
    return t;
  }
  /**
  The length of the document after the change.
  */
  get newLength() {
    let t = 0;
    for (let e = 0; e < this.sections.length; e += 2) {
      let i = this.sections[e + 1];
      t += i < 0 ? this.sections[e] : i;
    }
    return t;
  }
  /**
  False when there are actual changes in this set.
  */
  get empty() {
    return this.sections.length == 0 || this.sections.length == 2 && this.sections[1] < 0;
  }
  /**
  Iterate over the unchanged parts left by these changes. `posA`
  provides the position of the range in the old document, `posB`
  the new position in the changed document.
  */
  iterGaps(t) {
    for (let e = 0, i = 0, n = 0; e < this.sections.length; ) {
      let s = this.sections[e++], l = this.sections[e++];
      l < 0 ? (t(i, n, s), n += s) : n += l, i += s;
    }
  }
  /**
  Iterate over the ranges changed by these changes. (See
  [`ChangeSet.iterChanges`](https://codemirror.net/6/docs/ref/#state.ChangeSet.iterChanges) for a
  variant that also provides you with the inserted text.)
  `fromA`/`toA` provides the extent of the change in the starting
  document, `fromB`/`toB` the extent of the replacement in the
  changed document.
  
  When `individual` is true, adjacent changes (which are kept
  separate for [position mapping](https://codemirror.net/6/docs/ref/#state.ChangeDesc.mapPos)) are
  reported separately.
  */
  iterChangedRanges(t, e = !1) {
    Oi(this, t, e);
  }
  /**
  Get a description of the inverted form of these changes.
  */
  get invertedDesc() {
    let t = [];
    for (let e = 0; e < this.sections.length; ) {
      let i = this.sections[e++], n = this.sections[e++];
      n < 0 ? t.push(i, n) : t.push(n, i);
    }
    return new At(t);
  }
  /**
  Compute the combined effect of applying another set of changes
  after this one. The length of the document after this set should
  match the length before `other`.
  */
  composeDesc(t) {
    return this.empty ? t : t.empty ? this : Rs(this, t);
  }
  /**
  Map this description, which should start with the same document
  as `other`, over another set of changes, so that it can be
  applied after it. When `before` is true, map as if the changes
  in `other` happened before the ones in `this`.
  */
  mapDesc(t, e = !1) {
    return t.empty ? this : Di(this, t, e);
  }
  mapPos(t, e = -1, i = nt.Simple) {
    let n = 0, s = 0;
    for (let l = 0; l < this.sections.length; ) {
      let o = this.sections[l++], h = this.sections[l++], a = n + o;
      if (h < 0) {
        if (a > t)
          return s + (t - n);
        s += o;
      } else {
        if (i != nt.Simple && a >= t && (i == nt.TrackDel && n < t && a > t || i == nt.TrackBefore && n < t || i == nt.TrackAfter && a > t))
          return null;
        if (a > t || a == t && e < 0 && !o)
          return t == n || e < 0 ? s : s + h;
        s += h;
      }
      n = a;
    }
    if (t > n)
      throw new RangeError(`Position ${t} is out of range for changeset of length ${n}`);
    return s;
  }
  /**
  Check whether these changes touch a given range. When one of the
  changes entirely covers the range, the string `"cover"` is
  returned.
  */
  touchesRange(t, e = t) {
    for (let i = 0, n = 0; i < this.sections.length && n <= e; ) {
      let s = this.sections[i++], l = this.sections[i++], o = n + s;
      if (l >= 0 && n <= e && o >= t)
        return n < t && o > e ? "cover" : !0;
      n = o;
    }
    return !1;
  }
  /**
  @internal
  */
  toString() {
    let t = "";
    for (let e = 0; e < this.sections.length; ) {
      let i = this.sections[e++], n = this.sections[e++];
      t += (t ? " " : "") + i + (n >= 0 ? ":" + n : "");
    }
    return t;
  }
  /**
  Serialize this change desc to a JSON-representable value.
  */
  toJSON() {
    return this.sections;
  }
  /**
  Create a change desc from its JSON representation (as produced
  by [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeDesc.toJSON).
  */
  static fromJSON(t) {
    if (!Array.isArray(t) || t.length % 2 || t.some((e) => typeof e != "number"))
      throw new RangeError("Invalid JSON representation of ChangeDesc");
    return new At(t);
  }
  /**
  @internal
  */
  static create(t) {
    return new At(t);
  }
}
class j extends At {
  constructor(t, e) {
    super(t), this.inserted = e;
  }
  /**
  Apply the changes to a document, returning the modified
  document.
  */
  apply(t) {
    if (this.length != t.length)
      throw new RangeError("Applying change set to a document with the wrong length");
    return Oi(this, (e, i, n, s, l) => t = t.replace(n, n + (i - e), l), !1), t;
  }
  mapDesc(t, e = !1) {
    return Di(this, t, e, !0);
  }
  /**
  Given the document as it existed _before_ the changes, return a
  change set that represents the inverse of this set, which could
  be used to go from the document created by the changes back to
  the document as it existed before the changes.
  */
  invert(t) {
    let e = this.sections.slice(), i = [];
    for (let n = 0, s = 0; n < e.length; n += 2) {
      let l = e[n], o = e[n + 1];
      if (o >= 0) {
        e[n] = o, e[n + 1] = l;
        let h = n >> 1;
        for (; i.length < h; )
          i.push(R.empty);
        i.push(l ? t.slice(s, s + l) : R.empty);
      }
      s += l;
    }
    return new j(e, i);
  }
  /**
  Combine two subsequent change sets into a single set. `other`
  must start in the document produced by `this`. If `this` goes
  `docA` → `docB` and `other` represents `docB` → `docC`, the
  returned value will represent the change `docA` → `docC`.
  */
  compose(t) {
    return this.empty ? t : t.empty ? this : Rs(this, t, !0);
  }
  /**
  Given another change set starting in the same document, maps this
  change set over the other, producing a new change set that can be
  applied to the document produced by applying `other`. When
  `before` is `true`, order changes as if `this` comes before
  `other`, otherwise (the default) treat `other` as coming first.
  
  Given two changes `A` and `B`, `A.compose(B.map(A))` and
  `B.compose(A.map(B, true))` will produce the same document. This
  provides a basic form of [operational
  transformation](https://en.wikipedia.org/wiki/Operational_transformation),
  and can be used for collaborative editing.
  */
  map(t, e = !1) {
    return t.empty ? this : Di(this, t, e, !0);
  }
  /**
  Iterate over the changed ranges in the document, calling `f` for
  each, with the range in the original document (`fromA`-`toA`)
  and the range that replaces it in the new document
  (`fromB`-`toB`).
  
  When `individual` is true, adjacent changes are reported
  separately.
  */
  iterChanges(t, e = !1) {
    Oi(this, t, e);
  }
  /**
  Get a [change description](https://codemirror.net/6/docs/ref/#state.ChangeDesc) for this change
  set.
  */
  get desc() {
    return At.create(this.sections);
  }
  /**
  @internal
  */
  filter(t) {
    let e = [], i = [], n = [], s = new ke(this);
    t:
      for (let l = 0, o = 0; ; ) {
        let h = l == t.length ? 1e9 : t[l++];
        for (; o < h || o == h && s.len == 0; ) {
          if (s.done)
            break t;
          let f = Math.min(s.len, h - o);
          U(n, f, -1);
          let c = s.ins == -1 ? -1 : s.off == 0 ? s.ins : 0;
          U(e, f, c), c > 0 && Lt(i, e, s.text), s.forward(f), o += f;
        }
        let a = t[l++];
        for (; o < a; ) {
          if (s.done)
            break t;
          let f = Math.min(s.len, a - o);
          U(e, f, -1), U(n, f, s.ins == -1 ? -1 : s.off == 0 ? s.ins : 0), s.forward(f), o += f;
        }
      }
    return {
      changes: new j(e, i),
      filtered: At.create(n)
    };
  }
  /**
  Serialize this change set to a JSON-representable value.
  */
  toJSON() {
    let t = [];
    for (let e = 0; e < this.sections.length; e += 2) {
      let i = this.sections[e], n = this.sections[e + 1];
      n < 0 ? t.push(i) : n == 0 ? t.push([i]) : t.push([i].concat(this.inserted[e >> 1].toJSON()));
    }
    return t;
  }
  /**
  Create a change set for the given changes, for a document of the
  given length, using `lineSep` as line separator.
  */
  static of(t, e, i) {
    let n = [], s = [], l = 0, o = null;
    function h(f = !1) {
      if (!f && !n.length)
        return;
      l < e && U(n, e - l, -1);
      let c = new j(n, s);
      o = o ? o.compose(c.map(o)) : c, n = [], s = [], l = 0;
    }
    function a(f) {
      if (Array.isArray(f))
        for (let c of f)
          a(c);
      else if (f instanceof j) {
        if (f.length != e)
          throw new RangeError(`Mismatched change set length (got ${f.length}, expected ${e})`);
        h(), o = o ? o.compose(f.map(o)) : f;
      } else {
        let { from: c, to: u = c, insert: d } = f;
        if (c > u || c < 0 || u > e)
          throw new RangeError(`Invalid change range ${c} to ${u} (in doc of length ${e})`);
        let g = d ? typeof d == "string" ? R.of(d.split(i || Mi)) : d : R.empty, m = g.length;
        if (c == u && m == 0)
          return;
        c < l && h(), c > l && U(n, c - l, -1), U(n, u - c, m), Lt(s, n, g), l = u;
      }
    }
    return a(t), h(!o), o;
  }
  /**
  Create an empty changeset of the given length.
  */
  static empty(t) {
    return new j(t ? [t, -1] : [], []);
  }
  /**
  Create a changeset from its JSON representation (as produced by
  [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeSet.toJSON).
  */
  static fromJSON(t) {
    if (!Array.isArray(t))
      throw new RangeError("Invalid JSON representation of ChangeSet");
    let e = [], i = [];
    for (let n = 0; n < t.length; n++) {
      let s = t[n];
      if (typeof s == "number")
        e.push(s, -1);
      else {
        if (!Array.isArray(s) || typeof s[0] != "number" || s.some((l, o) => o && typeof l != "string"))
          throw new RangeError("Invalid JSON representation of ChangeSet");
        if (s.length == 1)
          e.push(s[0], 0);
        else {
          for (; i.length < n; )
            i.push(R.empty);
          i[n] = R.of(s.slice(1)), e.push(s[0], i[n].length);
        }
      }
    }
    return new j(e, i);
  }
  /**
  @internal
  */
  static createSet(t, e) {
    return new j(t, e);
  }
}
function U(r, t, e, i = !1) {
  if (t == 0 && e <= 0)
    return;
  let n = r.length - 2;
  n >= 0 && e <= 0 && e == r[n + 1] ? r[n] += t : t == 0 && r[n] == 0 ? r[n + 1] += e : i ? (r[n] += t, r[n + 1] += e) : r.push(t, e);
}
function Lt(r, t, e) {
  if (e.length == 0)
    return;
  let i = t.length - 2 >> 1;
  if (i < r.length)
    r[r.length - 1] = r[r.length - 1].append(e);
  else {
    for (; r.length < i; )
      r.push(R.empty);
    r.push(e);
  }
}
function Oi(r, t, e) {
  let i = r.inserted;
  for (let n = 0, s = 0, l = 0; l < r.sections.length; ) {
    let o = r.sections[l++], h = r.sections[l++];
    if (h < 0)
      n += o, s += o;
    else {
      let a = n, f = s, c = R.empty;
      for (; a += o, f += h, h && i && (c = c.append(i[l - 2 >> 1])), !(e || l == r.sections.length || r.sections[l + 1] < 0); )
        o = r.sections[l++], h = r.sections[l++];
      t(n, a, s, f, c), n = a, s = f;
    }
  }
}
function Di(r, t, e, i = !1) {
  let n = [], s = i ? [] : null, l = new ke(r), o = new ke(t);
  for (let h = -1; ; )
    if (l.ins == -1 && o.ins == -1) {
      let a = Math.min(l.len, o.len);
      U(n, a, -1), l.forward(a), o.forward(a);
    } else if (o.ins >= 0 && (l.ins < 0 || h == l.i || l.off == 0 && (o.len < l.len || o.len == l.len && !e))) {
      let a = o.len;
      for (U(n, o.ins, -1); a; ) {
        let f = Math.min(l.len, a);
        l.ins >= 0 && h < l.i && l.len <= f && (U(n, 0, l.ins), s && Lt(s, n, l.text), h = l.i), l.forward(f), a -= f;
      }
      o.next();
    } else if (l.ins >= 0) {
      let a = 0, f = l.len;
      for (; f; )
        if (o.ins == -1) {
          let c = Math.min(f, o.len);
          a += c, f -= c, o.forward(c);
        } else if (o.ins == 0 && o.len < f)
          f -= o.len, o.next();
        else
          break;
      U(n, a, h < l.i ? l.ins : 0), s && h < l.i && Lt(s, n, l.text), h = l.i, l.forward(l.len - f);
    } else {
      if (l.done && o.done)
        return s ? j.createSet(n, s) : At.create(n);
      throw new Error("Mismatched change set lengths");
    }
}
function Rs(r, t, e = !1) {
  let i = [], n = e ? [] : null, s = new ke(r), l = new ke(t);
  for (let o = !1; ; ) {
    if (s.done && l.done)
      return n ? j.createSet(i, n) : At.create(i);
    if (s.ins == 0)
      U(i, s.len, 0, o), s.next();
    else if (l.len == 0 && !l.done)
      U(i, 0, l.ins, o), n && Lt(n, i, l.text), l.next();
    else {
      if (s.done || l.done)
        throw new Error("Mismatched change set lengths");
      {
        let h = Math.min(s.len2, l.len), a = i.length;
        if (s.ins == -1) {
          let f = l.ins == -1 ? -1 : l.off ? 0 : l.ins;
          U(i, h, f, o), n && f && Lt(n, i, l.text);
        } else
          l.ins == -1 ? (U(i, s.off ? 0 : s.len, h, o), n && Lt(n, i, s.textBit(h))) : (U(i, s.off ? 0 : s.len, l.off ? 0 : l.ins, o), n && !l.off && Lt(n, i, l.text));
        o = (s.ins > h || l.ins >= 0 && l.len > h) && (o || i.length > a), s.forward2(h), l.forward(h);
      }
    }
  }
}
class ke {
  constructor(t) {
    this.set = t, this.i = 0, this.next();
  }
  next() {
    let { sections: t } = this.set;
    this.i < t.length ? (this.len = t[this.i++], this.ins = t[this.i++]) : (this.len = 0, this.ins = -2), this.off = 0;
  }
  get done() {
    return this.ins == -2;
  }
  get len2() {
    return this.ins < 0 ? this.len : this.ins;
  }
  get text() {
    let { inserted: t } = this.set, e = this.i - 2 >> 1;
    return e >= t.length ? R.empty : t[e];
  }
  textBit(t) {
    let { inserted: e } = this.set, i = this.i - 2 >> 1;
    return i >= e.length && !t ? R.empty : e[i].slice(this.off, t == null ? void 0 : this.off + t);
  }
  forward(t) {
    t == this.len ? this.next() : (this.len -= t, this.off += t);
  }
  forward2(t) {
    this.ins == -1 ? this.forward(t) : t == this.ins ? this.next() : (this.ins -= t, this.off += t);
  }
}
class qt {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.flags = i;
  }
  /**
  The anchor of the range—the side that doesn't move when you
  extend it.
  */
  get anchor() {
    return this.flags & 16 ? this.to : this.from;
  }
  /**
  The head of the range, which is moved when the range is
  [extended](https://codemirror.net/6/docs/ref/#state.SelectionRange.extend).
  */
  get head() {
    return this.flags & 16 ? this.from : this.to;
  }
  /**
  True when `anchor` and `head` are at the same position.
  */
  get empty() {
    return this.from == this.to;
  }
  /**
  If this is a cursor that is explicitly associated with the
  character on one of its sides, this returns the side. -1 means
  the character before its position, 1 the character after, and 0
  means no association.
  */
  get assoc() {
    return this.flags & 4 ? -1 : this.flags & 8 ? 1 : 0;
  }
  /**
  The bidirectional text level associated with this cursor, if
  any.
  */
  get bidiLevel() {
    let t = this.flags & 3;
    return t == 3 ? null : t;
  }
  /**
  The goal column (stored vertical offset) associated with a
  cursor. This is used to preserve the vertical position when
  [moving](https://codemirror.net/6/docs/ref/#view.EditorView.moveVertically) across
  lines of different length.
  */
  get goalColumn() {
    let t = this.flags >> 5;
    return t == 33554431 ? void 0 : t;
  }
  /**
  Map this range through a change, producing a valid range in the
  updated document.
  */
  map(t, e = -1) {
    let i, n;
    return this.empty ? i = n = t.mapPos(this.from, e) : (i = t.mapPos(this.from, 1), n = t.mapPos(this.to, -1)), i == this.from && n == this.to ? this : new qt(i, n, this.flags);
  }
  /**
  Extend this range to cover at least `from` to `to`.
  */
  extend(t, e = t) {
    if (t <= this.anchor && e >= this.anchor)
      return x.range(t, e);
    let i = Math.abs(t - this.anchor) > Math.abs(e - this.anchor) ? t : e;
    return x.range(this.anchor, i);
  }
  /**
  Compare this range to another range.
  */
  eq(t) {
    return this.anchor == t.anchor && this.head == t.head;
  }
  /**
  Return a JSON-serializable object representing the range.
  */
  toJSON() {
    return { anchor: this.anchor, head: this.head };
  }
  /**
  Convert a JSON representation of a range to a `SelectionRange`
  instance.
  */
  static fromJSON(t) {
    if (!t || typeof t.anchor != "number" || typeof t.head != "number")
      throw new RangeError("Invalid JSON representation for SelectionRange");
    return x.range(t.anchor, t.head);
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new qt(t, e, i);
  }
}
class x {
  constructor(t, e) {
    this.ranges = t, this.mainIndex = e;
  }
  /**
  Map a selection through a change. Used to adjust the selection
  position for changes.
  */
  map(t, e = -1) {
    return t.empty ? this : x.create(this.ranges.map((i) => i.map(t, e)), this.mainIndex);
  }
  /**
  Compare this selection to another selection.
  */
  eq(t) {
    if (this.ranges.length != t.ranges.length || this.mainIndex != t.mainIndex)
      return !1;
    for (let e = 0; e < this.ranges.length; e++)
      if (!this.ranges[e].eq(t.ranges[e]))
        return !1;
    return !0;
  }
  /**
  Get the primary selection range. Usually, you should make sure
  your code applies to _all_ ranges, by using methods like
  [`changeByRange`](https://codemirror.net/6/docs/ref/#state.EditorState.changeByRange).
  */
  get main() {
    return this.ranges[this.mainIndex];
  }
  /**
  Make sure the selection only has one range. Returns a selection
  holding only the main range from this selection.
  */
  asSingle() {
    return this.ranges.length == 1 ? this : new x([this.main], 0);
  }
  /**
  Extend this selection with an extra range.
  */
  addRange(t, e = !0) {
    return x.create([t].concat(this.ranges), e ? 0 : this.mainIndex + 1);
  }
  /**
  Replace a given range with another range, and then normalize the
  selection to merge and sort ranges if necessary.
  */
  replaceRange(t, e = this.mainIndex) {
    let i = this.ranges.slice();
    return i[e] = t, x.create(i, this.mainIndex);
  }
  /**
  Convert this selection to an object that can be serialized to
  JSON.
  */
  toJSON() {
    return { ranges: this.ranges.map((t) => t.toJSON()), main: this.mainIndex };
  }
  /**
  Create a selection from a JSON representation.
  */
  static fromJSON(t) {
    if (!t || !Array.isArray(t.ranges) || typeof t.main != "number" || t.main >= t.ranges.length)
      throw new RangeError("Invalid JSON representation for EditorSelection");
    return new x(t.ranges.map((e) => qt.fromJSON(e)), t.main);
  }
  /**
  Create a selection holding a single range.
  */
  static single(t, e = t) {
    return new x([x.range(t, e)], 0);
  }
  /**
  Sort and merge the given set of ranges, creating a valid
  selection.
  */
  static create(t, e = 0) {
    if (t.length == 0)
      throw new RangeError("A selection needs at least one range");
    for (let i = 0, n = 0; n < t.length; n++) {
      let s = t[n];
      if (s.empty ? s.from <= i : s.from < i)
        return x.normalized(t.slice(), e);
      i = s.to;
    }
    return new x(t, e);
  }
  /**
  Create a cursor selection range at the given position. You can
  safely ignore the optional arguments in most situations.
  */
  static cursor(t, e = 0, i, n) {
    return qt.create(t, t, (e == 0 ? 0 : e < 0 ? 4 : 8) | (i == null ? 3 : Math.min(2, i)) | (n ?? 33554431) << 5);
  }
  /**
  Create a selection range.
  */
  static range(t, e, i, n) {
    let s = (i ?? 33554431) << 5 | (n == null ? 3 : Math.min(2, n));
    return e < t ? qt.create(e, t, 24 | s) : qt.create(t, e, (e > t ? 4 : 0) | s);
  }
  /**
  @internal
  */
  static normalized(t, e = 0) {
    let i = t[e];
    t.sort((n, s) => n.from - s.from), e = t.indexOf(i);
    for (let n = 1; n < t.length; n++) {
      let s = t[n], l = t[n - 1];
      if (s.empty ? s.from <= l.to : s.from < l.to) {
        let o = l.from, h = Math.max(s.to, l.to);
        n <= e && e--, t.splice(--n, 2, s.anchor > s.head ? x.range(h, o) : x.range(o, h));
      }
    }
    return new x(t, e);
  }
}
function Es(r, t) {
  for (let e of r.ranges)
    if (e.to > t)
      throw new RangeError("Selection points outside of document");
}
let ln = 0;
class O {
  constructor(t, e, i, n, s) {
    this.combine = t, this.compareInput = e, this.compare = i, this.isStatic = n, this.id = ln++, this.default = t([]), this.extensions = typeof s == "function" ? s(this) : s;
  }
  /**
  Define a new facet.
  */
  static define(t = {}) {
    return new O(t.combine || ((e) => e), t.compareInput || ((e, i) => e === i), t.compare || (t.combine ? (e, i) => e === i : on), !!t.static, t.enables);
  }
  /**
  Returns an extension that adds the given value to this facet.
  */
  of(t) {
    return new Ge([], this, 0, t);
  }
  /**
  Create an extension that computes a value for the facet from a
  state. You must take care to declare the parts of the state that
  this value depends on, since your function is only called again
  for a new state when one of those parts changed.
  
  In cases where your value depends only on a single field, you'll
  want to use the [`from`](https://codemirror.net/6/docs/ref/#state.Facet.from) method instead.
  */
  compute(t, e) {
    if (this.isStatic)
      throw new Error("Can't compute a static facet");
    return new Ge(t, this, 1, e);
  }
  /**
  Create an extension that computes zero or more values for this
  facet from a state.
  */
  computeN(t, e) {
    if (this.isStatic)
      throw new Error("Can't compute a static facet");
    return new Ge(t, this, 2, e);
  }
  from(t, e) {
    return e || (e = (i) => i), this.compute([t], (i) => e(i.field(t)));
  }
}
function on(r, t) {
  return r == t || r.length == t.length && r.every((e, i) => e === t[i]);
}
class Ge {
  constructor(t, e, i, n) {
    this.dependencies = t, this.facet = e, this.type = i, this.value = n, this.id = ln++;
  }
  dynamicSlot(t) {
    var e;
    let i = this.value, n = this.facet.compareInput, s = this.id, l = t[s] >> 1, o = this.type == 2, h = !1, a = !1, f = [];
    for (let c of this.dependencies)
      c == "doc" ? h = !0 : c == "selection" ? a = !0 : ((e = t[c.id]) !== null && e !== void 0 ? e : 1) & 1 || f.push(t[c.id]);
    return {
      create(c) {
        return c.values[l] = i(c), 1;
      },
      update(c, u) {
        if (h && u.docChanged || a && (u.docChanged || u.selection) || Ti(c, f)) {
          let d = i(c);
          if (o ? !An(d, c.values[l], n) : !n(d, c.values[l]))
            return c.values[l] = d, 1;
        }
        return 0;
      },
      reconfigure: (c, u) => {
        let d, g = u.config.address[s];
        if (g != null) {
          let m = Qe(u, g);
          if (this.dependencies.every((p) => p instanceof O ? u.facet(p) === c.facet(p) : p instanceof Ft ? u.field(p, !1) == c.field(p, !1) : !0) || (o ? An(d = i(c), m, n) : n(d = i(c), m)))
            return c.values[l] = m, 0;
        } else
          d = i(c);
        return c.values[l] = d, 1;
      }
    };
  }
}
function An(r, t, e) {
  if (r.length != t.length)
    return !1;
  for (let i = 0; i < r.length; i++)
    if (!e(r[i], t[i]))
      return !1;
  return !0;
}
function Ti(r, t) {
  let e = !1;
  for (let i of t)
    be(r, i) & 1 && (e = !0);
  return e;
}
function Ur(r, t, e) {
  let i = e.map((h) => r[h.id]), n = e.map((h) => h.type), s = i.filter((h) => !(h & 1)), l = r[t.id] >> 1;
  function o(h) {
    let a = [];
    for (let f = 0; f < i.length; f++) {
      let c = Qe(h, i[f]);
      if (n[f] == 2)
        for (let u of c)
          a.push(u);
      else
        a.push(c);
    }
    return t.combine(a);
  }
  return {
    create(h) {
      for (let a of i)
        be(h, a);
      return h.values[l] = o(h), 1;
    },
    update(h, a) {
      if (!Ti(h, s))
        return 0;
      let f = o(h);
      return t.compare(f, h.values[l]) ? 0 : (h.values[l] = f, 1);
    },
    reconfigure(h, a) {
      let f = Ti(h, i), c = a.config.facets[t.id], u = a.facet(t);
      if (c && !f && on(e, c))
        return h.values[l] = u, 0;
      let d = o(h);
      return t.compare(d, u) ? (h.values[l] = u, 0) : (h.values[l] = d, 1);
    }
  };
}
const Cn = /* @__PURE__ */ O.define({ static: !0 });
class Ft {
  constructor(t, e, i, n, s) {
    this.id = t, this.createF = e, this.updateF = i, this.compareF = n, this.spec = s, this.provides = void 0;
  }
  /**
  Define a state field.
  */
  static define(t) {
    let e = new Ft(ln++, t.create, t.update, t.compare || ((i, n) => i === n), t);
    return t.provide && (e.provides = t.provide(e)), e;
  }
  create(t) {
    let e = t.facet(Cn).find((i) => i.field == this);
    return ((e == null ? void 0 : e.create) || this.createF)(t);
  }
  /**
  @internal
  */
  slot(t) {
    let e = t[this.id] >> 1;
    return {
      create: (i) => (i.values[e] = this.create(i), 1),
      update: (i, n) => {
        let s = i.values[e], l = this.updateF(s, n);
        return this.compareF(s, l) ? 0 : (i.values[e] = l, 1);
      },
      reconfigure: (i, n) => n.config.address[this.id] != null ? (i.values[e] = n.field(this), 0) : (i.values[e] = this.create(i), 1)
    };
  }
  /**
  Returns an extension that enables this field and overrides the
  way it is initialized. Can be useful when you need to provide a
  non-default starting value for the field.
  */
  init(t) {
    return [this, Cn.of({ field: this, create: t })];
  }
  /**
  State field instances can be used as
  [`Extension`](https://codemirror.net/6/docs/ref/#state.Extension) values to enable the field in a
  given state.
  */
  get extension() {
    return this;
  }
}
const zt = { lowest: 4, low: 3, default: 2, high: 1, highest: 0 };
function fe(r) {
  return (t) => new Ls(t, r);
}
const Xr = {
  /**
  The highest precedence level, for extensions that should end up
  near the start of the precedence ordering.
  */
  highest: /* @__PURE__ */ fe(zt.highest),
  /**
  A higher-than-default precedence, for extensions that should
  come before those with default precedence.
  */
  high: /* @__PURE__ */ fe(zt.high),
  /**
  The default precedence, which is also used for extensions
  without an explicit precedence.
  */
  default: /* @__PURE__ */ fe(zt.default),
  /**
  A lower-than-default precedence.
  */
  low: /* @__PURE__ */ fe(zt.low),
  /**
  The lowest precedence level. Meant for things that should end up
  near the end of the extension order.
  */
  lowest: /* @__PURE__ */ fe(zt.lowest)
};
class Ls {
  constructor(t, e) {
    this.inner = t, this.prec = e;
  }
}
class ai {
  /**
  Create an instance of this compartment to add to your [state
  configuration](https://codemirror.net/6/docs/ref/#state.EditorStateConfig.extensions).
  */
  of(t) {
    return new Pi(this, t);
  }
  /**
  Create an [effect](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) that
  reconfigures this compartment.
  */
  reconfigure(t) {
    return ai.reconfigure.of({ compartment: this, extension: t });
  }
  /**
  Get the current content of the compartment in the state, or
  `undefined` if it isn't present.
  */
  get(t) {
    return t.config.compartments.get(this);
  }
}
class Pi {
  constructor(t, e) {
    this.compartment = t, this.inner = e;
  }
}
class Ze {
  constructor(t, e, i, n, s, l) {
    for (this.base = t, this.compartments = e, this.dynamicSlots = i, this.address = n, this.staticValues = s, this.facets = l, this.statusTemplate = []; this.statusTemplate.length < i.length; )
      this.statusTemplate.push(
        0
        /* SlotStatus.Unresolved */
      );
  }
  staticFacet(t) {
    let e = this.address[t.id];
    return e == null ? t.default : this.staticValues[e >> 1];
  }
  static resolve(t, e, i) {
    let n = [], s = /* @__PURE__ */ Object.create(null), l = /* @__PURE__ */ new Map();
    for (let u of _r(t, e, l))
      u instanceof Ft ? n.push(u) : (s[u.facet.id] || (s[u.facet.id] = [])).push(u);
    let o = /* @__PURE__ */ Object.create(null), h = [], a = [];
    for (let u of n)
      o[u.id] = a.length << 1, a.push((d) => u.slot(d));
    let f = i == null ? void 0 : i.config.facets;
    for (let u in s) {
      let d = s[u], g = d[0].facet, m = f && f[u] || [];
      if (d.every(
        (p) => p.type == 0
        /* Provider.Static */
      ))
        if (o[g.id] = h.length << 1 | 1, on(m, d))
          h.push(i.facet(g));
        else {
          let p = g.combine(d.map((y) => y.value));
          h.push(i && g.compare(p, i.facet(g)) ? i.facet(g) : p);
        }
      else {
        for (let p of d)
          p.type == 0 ? (o[p.id] = h.length << 1 | 1, h.push(p.value)) : (o[p.id] = a.length << 1, a.push((y) => p.dynamicSlot(y)));
        o[g.id] = a.length << 1, a.push((p) => Ur(p, g, d));
      }
    }
    let c = a.map((u) => u(o));
    return new Ze(t, l, c, o, h, s);
  }
}
function _r(r, t, e) {
  let i = [[], [], [], [], []], n = /* @__PURE__ */ new Map();
  function s(l, o) {
    let h = n.get(l);
    if (h != null) {
      if (h <= o)
        return;
      let a = i[h].indexOf(l);
      a > -1 && i[h].splice(a, 1), l instanceof Pi && e.delete(l.compartment);
    }
    if (n.set(l, o), Array.isArray(l))
      for (let a of l)
        s(a, o);
    else if (l instanceof Pi) {
      if (e.has(l.compartment))
        throw new RangeError("Duplicate use of compartment in extensions");
      let a = t.get(l.compartment) || l.inner;
      e.set(l.compartment, a), s(a, o);
    } else if (l instanceof Ls)
      s(l.inner, l.prec);
    else if (l instanceof Ft)
      i[o].push(l), l.provides && s(l.provides, o);
    else if (l instanceof Ge)
      i[o].push(l), l.facet.extensions && s(l.facet.extensions, zt.default);
    else {
      let a = l.extension;
      if (!a)
        throw new Error(`Unrecognized extension value in extension set (${l}). This sometimes happens because multiple instances of @codemirror/state are loaded, breaking instanceof checks.`);
      s(a, o);
    }
  }
  return s(r, zt.default), i.reduce((l, o) => l.concat(o));
}
function be(r, t) {
  if (t & 1)
    return 2;
  let e = t >> 1, i = r.status[e];
  if (i == 4)
    throw new Error("Cyclic dependency between fields and/or facets");
  if (i & 2)
    return i;
  r.status[e] = 4;
  let n = r.computeSlot(r, r.config.dynamicSlots[e]);
  return r.status[e] = 2 | n;
}
function Qe(r, t) {
  return t & 1 ? r.config.staticValues[t >> 1] : r.values[t >> 1];
}
const Ns = /* @__PURE__ */ O.define(), Is = /* @__PURE__ */ O.define({
  combine: (r) => r.some((t) => t),
  static: !0
}), Hs = /* @__PURE__ */ O.define({
  combine: (r) => r.length ? r[0] : void 0,
  static: !0
}), Fs = /* @__PURE__ */ O.define(), Vs = /* @__PURE__ */ O.define(), Ws = /* @__PURE__ */ O.define(), zs = /* @__PURE__ */ O.define({
  combine: (r) => r.length ? r[0] : !1
});
class ae {
  /**
  @internal
  */
  constructor(t, e) {
    this.type = t, this.value = e;
  }
  /**
  Define a new type of annotation.
  */
  static define() {
    return new Zr();
  }
}
class Zr {
  /**
  Create an instance of this annotation.
  */
  of(t) {
    return new ae(this, t);
  }
}
class Qr {
  /**
  @internal
  */
  constructor(t) {
    this.map = t;
  }
  /**
  Create a [state effect](https://codemirror.net/6/docs/ref/#state.StateEffect) instance of this
  type.
  */
  of(t) {
    return new q(this, t);
  }
}
class q {
  /**
  @internal
  */
  constructor(t, e) {
    this.type = t, this.value = e;
  }
  /**
  Map this effect through a position mapping. Will return
  `undefined` when that ends up deleting the effect.
  */
  map(t) {
    let e = this.type.map(this.value, t);
    return e === void 0 ? void 0 : e == this.value ? this : new q(this.type, e);
  }
  /**
  Tells you whether this effect object is of a given
  [type](https://codemirror.net/6/docs/ref/#state.StateEffectType).
  */
  is(t) {
    return this.type == t;
  }
  /**
  Define a new effect type. The type parameter indicates the type
  of values that his effect holds. It should be a type that
  doesn't include `undefined`, since that is used in
  [mapping](https://codemirror.net/6/docs/ref/#state.StateEffect.map) to indicate that an effect is
  removed.
  */
  static define(t = {}) {
    return new Qr(t.map || ((e) => e));
  }
  /**
  Map an array of effects through a change set.
  */
  static mapEffects(t, e) {
    if (!t.length)
      return t;
    let i = [];
    for (let n of t) {
      let s = n.map(e);
      s && i.push(s);
    }
    return i;
  }
}
q.reconfigure = /* @__PURE__ */ q.define();
q.appendConfig = /* @__PURE__ */ q.define();
class _ {
  constructor(t, e, i, n, s, l) {
    this.startState = t, this.changes = e, this.selection = i, this.effects = n, this.annotations = s, this.scrollIntoView = l, this._doc = null, this._state = null, i && Es(i, e.newLength), s.some((o) => o.type == _.time) || (this.annotations = s.concat(_.time.of(Date.now())));
  }
  /**
  @internal
  */
  static create(t, e, i, n, s, l) {
    return new _(t, e, i, n, s, l);
  }
  /**
  The new document produced by the transaction. Contrary to
  [`.state`](https://codemirror.net/6/docs/ref/#state.Transaction.state)`.doc`, accessing this won't
  force the entire new state to be computed right away, so it is
  recommended that [transaction
  filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) use this getter
  when they need to look at the new document.
  */
  get newDoc() {
    return this._doc || (this._doc = this.changes.apply(this.startState.doc));
  }
  /**
  The new selection produced by the transaction. If
  [`this.selection`](https://codemirror.net/6/docs/ref/#state.Transaction.selection) is undefined,
  this will [map](https://codemirror.net/6/docs/ref/#state.EditorSelection.map) the start state's
  current selection through the changes made by the transaction.
  */
  get newSelection() {
    return this.selection || this.startState.selection.map(this.changes);
  }
  /**
  The new state created by the transaction. Computed on demand
  (but retained for subsequent access), so it is recommended not to
  access it in [transaction
  filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) when possible.
  */
  get state() {
    return this._state || this.startState.applyTransaction(this), this._state;
  }
  /**
  Get the value of the given annotation type, if any.
  */
  annotation(t) {
    for (let e of this.annotations)
      if (e.type == t)
        return e.value;
  }
  /**
  Indicates whether the transaction changed the document.
  */
  get docChanged() {
    return !this.changes.empty;
  }
  /**
  Indicates whether this transaction reconfigures the state
  (through a [configuration compartment](https://codemirror.net/6/docs/ref/#state.Compartment) or
  with a top-level configuration
  [effect](https://codemirror.net/6/docs/ref/#state.StateEffect^reconfigure).
  */
  get reconfigured() {
    return this.startState.config != this.state.config;
  }
  /**
  Returns true if the transaction has a [user
  event](https://codemirror.net/6/docs/ref/#state.Transaction^userEvent) annotation that is equal to
  or more specific than `event`. For example, if the transaction
  has `"select.pointer"` as user event, `"select"` and
  `"select.pointer"` will match it.
  */
  isUserEvent(t) {
    let e = this.annotation(_.userEvent);
    return !!(e && (e == t || e.length > t.length && e.slice(0, t.length) == t && e[t.length] == "."));
  }
}
_.time = /* @__PURE__ */ ae.define();
_.userEvent = /* @__PURE__ */ ae.define();
_.addToHistory = /* @__PURE__ */ ae.define();
_.remote = /* @__PURE__ */ ae.define();
function tl(r, t) {
  let e = [];
  for (let i = 0, n = 0; ; ) {
    let s, l;
    if (i < r.length && (n == t.length || t[n] >= r[i]))
      s = r[i++], l = r[i++];
    else if (n < t.length)
      s = t[n++], l = t[n++];
    else
      return e;
    !e.length || e[e.length - 1] < s ? e.push(s, l) : e[e.length - 1] < l && (e[e.length - 1] = l);
  }
}
function qs(r, t, e) {
  var i;
  let n, s, l;
  return e ? (n = t.changes, s = j.empty(t.changes.length), l = r.changes.compose(t.changes)) : (n = t.changes.map(r.changes), s = r.changes.mapDesc(t.changes, !0), l = r.changes.compose(n)), {
    changes: l,
    selection: t.selection ? t.selection.map(s) : (i = r.selection) === null || i === void 0 ? void 0 : i.map(n),
    effects: q.mapEffects(r.effects, n).concat(q.mapEffects(t.effects, s)),
    annotations: r.annotations.length ? r.annotations.concat(t.annotations) : t.annotations,
    scrollIntoView: r.scrollIntoView || t.scrollIntoView
  };
}
function Bi(r, t, e) {
  let i = t.selection, n = Qt(t.annotations);
  return t.userEvent && (n = n.concat(_.userEvent.of(t.userEvent))), {
    changes: t.changes instanceof j ? t.changes : j.of(t.changes || [], e, r.facet(Hs)),
    selection: i && (i instanceof x ? i : x.single(i.anchor, i.head)),
    effects: Qt(t.effects),
    annotations: n,
    scrollIntoView: !!t.scrollIntoView
  };
}
function Ks(r, t, e) {
  let i = Bi(r, t.length ? t[0] : {}, r.doc.length);
  t.length && t[0].filter === !1 && (e = !1);
  for (let s = 1; s < t.length; s++) {
    t[s].filter === !1 && (e = !1);
    let l = !!t[s].sequential;
    i = qs(i, Bi(r, t[s], l ? i.changes.newLength : r.doc.length), l);
  }
  let n = _.create(r, i.changes, i.selection, i.effects, i.annotations, i.scrollIntoView);
  return il(e ? el(n) : n);
}
function el(r) {
  let t = r.startState, e = !0;
  for (let n of t.facet(Fs)) {
    let s = n(r);
    if (s === !1) {
      e = !1;
      break;
    }
    Array.isArray(s) && (e = e === !0 ? s : tl(e, s));
  }
  if (e !== !0) {
    let n, s;
    if (e === !1)
      s = r.changes.invertedDesc, n = j.empty(t.doc.length);
    else {
      let l = r.changes.filter(e);
      n = l.changes, s = l.filtered.mapDesc(l.changes).invertedDesc;
    }
    r = _.create(t, n, r.selection && r.selection.map(s), q.mapEffects(r.effects, s), r.annotations, r.scrollIntoView);
  }
  let i = t.facet(Vs);
  for (let n = i.length - 1; n >= 0; n--) {
    let s = i[n](r);
    s instanceof _ ? r = s : Array.isArray(s) && s.length == 1 && s[0] instanceof _ ? r = s[0] : r = Ks(t, Qt(s), !1);
  }
  return r;
}
function il(r) {
  let t = r.startState, e = t.facet(Ws), i = r;
  for (let n = e.length - 1; n >= 0; n--) {
    let s = e[n](r);
    s && Object.keys(s).length && (i = qs(i, Bi(t, s, r.changes.newLength), !0));
  }
  return i == r ? r : _.create(t, r.changes, r.selection, i.effects, i.annotations, i.scrollIntoView);
}
const nl = [];
function Qt(r) {
  return r == null ? nl : Array.isArray(r) ? r : [r];
}
var vt = /* @__PURE__ */ function(r) {
  return r[r.Word = 0] = "Word", r[r.Space = 1] = "Space", r[r.Other = 2] = "Other", r;
}(vt || (vt = {}));
const sl = /[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;
let Ri;
try {
  Ri = /* @__PURE__ */ new RegExp("[\\p{Alphabetic}\\p{Number}_]", "u");
} catch {
}
function rl(r) {
  if (Ri)
    return Ri.test(r);
  for (let t = 0; t < r.length; t++) {
    let e = r[t];
    if (/\w/.test(e) || e > "" && (e.toUpperCase() != e.toLowerCase() || sl.test(e)))
      return !0;
  }
  return !1;
}
function ll(r) {
  return (t) => {
    if (!/\S/.test(t))
      return vt.Space;
    if (rl(t))
      return vt.Word;
    for (let e = 0; e < r.length; e++)
      if (t.indexOf(r[e]) > -1)
        return vt.Word;
    return vt.Other;
  };
}
class E {
  constructor(t, e, i, n, s, l) {
    this.config = t, this.doc = e, this.selection = i, this.values = n, this.status = t.statusTemplate.slice(), this.computeSlot = s, l && (l._state = this);
    for (let o = 0; o < this.config.dynamicSlots.length; o++)
      be(this, o << 1);
    this.computeSlot = null;
  }
  field(t, e = !0) {
    let i = this.config.address[t.id];
    if (i == null) {
      if (e)
        throw new RangeError("Field is not present in this state");
      return;
    }
    return be(this, i), Qe(this, i);
  }
  /**
  Create a [transaction](https://codemirror.net/6/docs/ref/#state.Transaction) that updates this
  state. Any number of [transaction specs](https://codemirror.net/6/docs/ref/#state.TransactionSpec)
  can be passed. Unless
  [`sequential`](https://codemirror.net/6/docs/ref/#state.TransactionSpec.sequential) is set, the
  [changes](https://codemirror.net/6/docs/ref/#state.TransactionSpec.changes) (if any) of each spec
  are assumed to start in the _current_ document (not the document
  produced by previous specs), and its
  [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection) and
  [effects](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) are assumed to refer
  to the document created by its _own_ changes. The resulting
  transaction contains the combined effect of all the different
  specs. For [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection), later
  specs take precedence over earlier ones.
  */
  update(...t) {
    return Ks(this, t, !0);
  }
  /**
  @internal
  */
  applyTransaction(t) {
    let e = this.config, { base: i, compartments: n } = e;
    for (let l of t.effects)
      l.is(ai.reconfigure) ? (e && (n = /* @__PURE__ */ new Map(), e.compartments.forEach((o, h) => n.set(h, o)), e = null), n.set(l.value.compartment, l.value.extension)) : l.is(q.reconfigure) ? (e = null, i = l.value) : l.is(q.appendConfig) && (e = null, i = Qt(i).concat(l.value));
    let s;
    e ? s = t.startState.values.slice() : (e = Ze.resolve(i, n, this), s = new E(e, this.doc, this.selection, e.dynamicSlots.map(() => null), (o, h) => h.reconfigure(o, this), null).values), new E(e, t.newDoc, t.newSelection, s, (l, o) => o.update(l, t), t);
  }
  /**
  Create a [transaction spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec) that
  replaces every selection range with the given content.
  */
  replaceSelection(t) {
    return typeof t == "string" && (t = this.toText(t)), this.changeByRange((e) => ({
      changes: { from: e.from, to: e.to, insert: t },
      range: x.cursor(e.from + t.length)
    }));
  }
  /**
  Create a set of changes and a new selection by running the given
  function for each range in the active selection. The function
  can return an optional set of changes (in the coordinate space
  of the start document), plus an updated range (in the coordinate
  space of the document produced by the call's own changes). This
  method will merge all the changes and ranges into a single
  changeset and selection, and return it as a [transaction
  spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec), which can be passed to
  [`update`](https://codemirror.net/6/docs/ref/#state.EditorState.update).
  */
  changeByRange(t) {
    let e = this.selection, i = t(e.ranges[0]), n = this.changes(i.changes), s = [i.range], l = Qt(i.effects);
    for (let o = 1; o < e.ranges.length; o++) {
      let h = t(e.ranges[o]), a = this.changes(h.changes), f = a.map(n);
      for (let u = 0; u < o; u++)
        s[u] = s[u].map(f);
      let c = n.mapDesc(a, !0);
      s.push(h.range.map(c)), n = n.compose(f), l = q.mapEffects(l, f).concat(q.mapEffects(Qt(h.effects), c));
    }
    return {
      changes: n,
      selection: x.create(s, e.mainIndex),
      effects: l
    };
  }
  /**
  Create a [change set](https://codemirror.net/6/docs/ref/#state.ChangeSet) from the given change
  description, taking the state's document length and line
  separator into account.
  */
  changes(t = []) {
    return t instanceof j ? t : j.of(t, this.doc.length, this.facet(E.lineSeparator));
  }
  /**
  Using the state's [line
  separator](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator), create a
  [`Text`](https://codemirror.net/6/docs/ref/#state.Text) instance from the given string.
  */
  toText(t) {
    return R.of(t.split(this.facet(E.lineSeparator) || Mi));
  }
  /**
  Return the given range of the document as a string.
  */
  sliceDoc(t = 0, e = this.doc.length) {
    return this.doc.sliceString(t, e, this.lineBreak);
  }
  /**
  Get the value of a state [facet](https://codemirror.net/6/docs/ref/#state.Facet).
  */
  facet(t) {
    let e = this.config.address[t.id];
    return e == null ? t.default : (be(this, e), Qe(this, e));
  }
  /**
  Convert this state to a JSON-serializable object. When custom
  fields should be serialized, you can pass them in as an object
  mapping property names (in the resulting object, which should
  not use `doc` or `selection`) to fields.
  */
  toJSON(t) {
    let e = {
      doc: this.sliceDoc(),
      selection: this.selection.toJSON()
    };
    if (t)
      for (let i in t) {
        let n = t[i];
        n instanceof Ft && this.config.address[n.id] != null && (e[i] = n.spec.toJSON(this.field(t[i]), this));
      }
    return e;
  }
  /**
  Deserialize a state from its JSON representation. When custom
  fields should be deserialized, pass the same object you passed
  to [`toJSON`](https://codemirror.net/6/docs/ref/#state.EditorState.toJSON) when serializing as
  third argument.
  */
  static fromJSON(t, e = {}, i) {
    if (!t || typeof t.doc != "string")
      throw new RangeError("Invalid JSON representation for EditorState");
    let n = [];
    if (i) {
      for (let s in i)
        if (Object.prototype.hasOwnProperty.call(t, s)) {
          let l = i[s], o = t[s];
          n.push(l.init((h) => l.spec.fromJSON(o, h)));
        }
    }
    return E.create({
      doc: t.doc,
      selection: x.fromJSON(t.selection),
      extensions: e.extensions ? n.concat([e.extensions]) : n
    });
  }
  /**
  Create a new state. You'll usually only need this when
  initializing an editor—updated states are created by applying
  transactions.
  */
  static create(t = {}) {
    let e = Ze.resolve(t.extensions || [], /* @__PURE__ */ new Map()), i = t.doc instanceof R ? t.doc : R.of((t.doc || "").split(e.staticFacet(E.lineSeparator) || Mi)), n = t.selection ? t.selection instanceof x ? t.selection : x.single(t.selection.anchor, t.selection.head) : x.single(0);
    return Es(n, i.length), e.staticFacet(Is) || (n = n.asSingle()), new E(e, i, n, e.dynamicSlots.map(() => null), (s, l) => l.create(s), null);
  }
  /**
  The size (in columns) of a tab in the document, determined by
  the [`tabSize`](https://codemirror.net/6/docs/ref/#state.EditorState^tabSize) facet.
  */
  get tabSize() {
    return this.facet(E.tabSize);
  }
  /**
  Get the proper [line-break](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator)
  string for this state.
  */
  get lineBreak() {
    return this.facet(E.lineSeparator) || `
`;
  }
  /**
  Returns true when the editor is
  [configured](https://codemirror.net/6/docs/ref/#state.EditorState^readOnly) to be read-only.
  */
  get readOnly() {
    return this.facet(zs);
  }
  /**
  Look up a translation for the given phrase (via the
  [`phrases`](https://codemirror.net/6/docs/ref/#state.EditorState^phrases) facet), or return the
  original string if no translation is found.
  
  If additional arguments are passed, they will be inserted in
  place of markers like `$1` (for the first value) and `$2`, etc.
  A single `$` is equivalent to `$1`, and `$$` will produce a
  literal dollar sign.
  */
  phrase(t, ...e) {
    for (let i of this.facet(E.phrases))
      if (Object.prototype.hasOwnProperty.call(i, t)) {
        t = i[t];
        break;
      }
    return e.length && (t = t.replace(/\$(\$|\d*)/g, (i, n) => {
      if (n == "$")
        return "$";
      let s = +(n || 1);
      return !s || s > e.length ? i : e[s - 1];
    })), t;
  }
  /**
  Find the values for a given language data field, provided by the
  the [`languageData`](https://codemirror.net/6/docs/ref/#state.EditorState^languageData) facet.
  
  Examples of language data fields are...
  
  - [`"commentTokens"`](https://codemirror.net/6/docs/ref/#commands.CommentTokens) for specifying
    comment syntax.
  - [`"autocomplete"`](https://codemirror.net/6/docs/ref/#autocomplete.autocompletion^config.override)
    for providing language-specific completion sources.
  - [`"wordChars"`](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) for adding
    characters that should be considered part of words in this
    language.
  - [`"closeBrackets"`](https://codemirror.net/6/docs/ref/#autocomplete.CloseBracketConfig) controls
    bracket closing behavior.
  */
  languageDataAt(t, e, i = -1) {
    let n = [];
    for (let s of this.facet(Ns))
      for (let l of s(this, e, i))
        Object.prototype.hasOwnProperty.call(l, t) && n.push(l[t]);
    return n;
  }
  /**
  Return a function that can categorize strings (expected to
  represent a single [grapheme cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak))
  into one of:
  
   - Word (contains an alphanumeric character or a character
     explicitly listed in the local language's `"wordChars"`
     language data, which should be a string)
   - Space (contains only whitespace)
   - Other (anything else)
  */
  charCategorizer(t) {
    return ll(this.languageDataAt("wordChars", t).join(""));
  }
  /**
  Find the word at the given position, meaning the range
  containing all [word](https://codemirror.net/6/docs/ref/#state.CharCategory.Word) characters
  around it. If no word characters are adjacent to the position,
  this returns null.
  */
  wordAt(t) {
    let { text: e, from: i, length: n } = this.doc.lineAt(t), s = this.charCategorizer(t), l = t - i, o = t - i;
    for (; l > 0; ) {
      let h = kt(e, l, !1);
      if (s(e.slice(h, l)) != vt.Word)
        break;
      l = h;
    }
    for (; o < n; ) {
      let h = kt(e, o);
      if (s(e.slice(o, h)) != vt.Word)
        break;
      o = h;
    }
    return l == o ? null : x.range(l + i, o + i);
  }
}
E.allowMultipleSelections = Is;
E.tabSize = /* @__PURE__ */ O.define({
  combine: (r) => r.length ? r[0] : 4
});
E.lineSeparator = Hs;
E.readOnly = zs;
E.phrases = /* @__PURE__ */ O.define({
  compare(r, t) {
    let e = Object.keys(r), i = Object.keys(t);
    return e.length == i.length && e.every((n) => r[n] == t[n]);
  }
});
E.languageData = Ns;
E.changeFilter = Fs;
E.transactionFilter = Vs;
E.transactionExtender = Ws;
ai.reconfigure = /* @__PURE__ */ q.define();
function ol(r, t, e = {}) {
  let i = {};
  for (let n of r)
    for (let s of Object.keys(n)) {
      let l = n[s], o = i[s];
      if (o === void 0)
        i[s] = l;
      else if (!(o === l || l === void 0))
        if (Object.hasOwnProperty.call(e, s))
          i[s] = e[s](o, l);
        else
          throw new Error("Config merge conflict for field " + s);
    }
  for (let n in t)
    i[n] === void 0 && (i[n] = t[n]);
  return i;
}
class ee {
  /**
  Compare this value with another value. Used when comparing
  rangesets. The default implementation compares by identity.
  Unless you are only creating a fixed number of unique instances
  of your value type, it is a good idea to implement this
  properly.
  */
  eq(t) {
    return this == t;
  }
  /**
  Create a [range](https://codemirror.net/6/docs/ref/#state.Range) with this value.
  */
  range(t, e = t) {
    return Ei.create(t, e, this);
  }
}
ee.prototype.startSide = ee.prototype.endSide = 0;
ee.prototype.point = !1;
ee.prototype.mapMode = nt.TrackDel;
let Ei = class js {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.value = i;
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new js(t, e, i);
  }
};
function Li(r, t) {
  return r.from - t.from || r.value.startSide - t.value.startSide;
}
class hn {
  constructor(t, e, i, n) {
    this.from = t, this.to = e, this.value = i, this.maxPoint = n;
  }
  get length() {
    return this.to[this.to.length - 1];
  }
  // Find the index of the given position and side. Use the ranges'
  // `from` pos when `end == false`, `to` when `end == true`.
  findIndex(t, e, i, n = 0) {
    let s = i ? this.to : this.from;
    for (let l = n, o = s.length; ; ) {
      if (l == o)
        return l;
      let h = l + o >> 1, a = s[h] - t || (i ? this.value[h].endSide : this.value[h].startSide) - e;
      if (h == l)
        return a >= 0 ? l : o;
      a >= 0 ? o = h : l = h + 1;
    }
  }
  between(t, e, i, n) {
    for (let s = this.findIndex(e, -1e9, !0), l = this.findIndex(i, 1e9, !1, s); s < l; s++)
      if (n(this.from[s] + t, this.to[s] + t, this.value[s]) === !1)
        return !1;
  }
  map(t, e) {
    let i = [], n = [], s = [], l = -1, o = -1;
    for (let h = 0; h < this.value.length; h++) {
      let a = this.value[h], f = this.from[h] + t, c = this.to[h] + t, u, d;
      if (f == c) {
        let g = e.mapPos(f, a.startSide, a.mapMode);
        if (g == null || (u = d = g, a.startSide != a.endSide && (d = e.mapPos(f, a.endSide), d < u)))
          continue;
      } else if (u = e.mapPos(f, a.startSide), d = e.mapPos(c, a.endSide), u > d || u == d && a.startSide > 0 && a.endSide <= 0)
        continue;
      (d - u || a.endSide - a.startSide) < 0 || (l < 0 && (l = u), a.point && (o = Math.max(o, d - u)), i.push(a), n.push(u - l), s.push(d - l));
    }
    return { mapped: i.length ? new hn(n, s, i, o) : null, pos: l };
  }
}
class W {
  constructor(t, e, i, n) {
    this.chunkPos = t, this.chunk = e, this.nextLayer = i, this.maxPoint = n;
  }
  /**
  @internal
  */
  static create(t, e, i, n) {
    return new W(t, e, i, n);
  }
  /**
  @internal
  */
  get length() {
    let t = this.chunk.length - 1;
    return t < 0 ? 0 : Math.max(this.chunkEnd(t), this.nextLayer.length);
  }
  /**
  The number of ranges in the set.
  */
  get size() {
    if (this.isEmpty)
      return 0;
    let t = this.nextLayer.size;
    for (let e of this.chunk)
      t += e.value.length;
    return t;
  }
  /**
  @internal
  */
  chunkEnd(t) {
    return this.chunkPos[t] + this.chunk[t].length;
  }
  /**
  Update the range set, optionally adding new ranges or filtering
  out existing ones.
  
  (Note: The type parameter is just there as a kludge to work
  around TypeScript variance issues that prevented `RangeSet<X>`
  from being a subtype of `RangeSet<Y>` when `X` is a subtype of
  `Y`.)
  */
  update(t) {
    let { add: e = [], sort: i = !1, filterFrom: n = 0, filterTo: s = this.length } = t, l = t.filter;
    if (e.length == 0 && !l)
      return this;
    if (i && (e = e.slice().sort(Li)), this.isEmpty)
      return e.length ? W.of(e) : this;
    let o = new $s(this, null, -1).goto(0), h = 0, a = [], f = new ti();
    for (; o.value || h < e.length; )
      if (h < e.length && (o.from - e[h].from || o.startSide - e[h].value.startSide) >= 0) {
        let c = e[h++];
        f.addInner(c.from, c.to, c.value) || a.push(c);
      } else
        o.rangeIndex == 1 && o.chunkIndex < this.chunk.length && (h == e.length || this.chunkEnd(o.chunkIndex) < e[h].from) && (!l || n > this.chunkEnd(o.chunkIndex) || s < this.chunkPos[o.chunkIndex]) && f.addChunk(this.chunkPos[o.chunkIndex], this.chunk[o.chunkIndex]) ? o.nextChunk() : ((!l || n > o.to || s < o.from || l(o.from, o.to, o.value)) && (f.addInner(o.from, o.to, o.value) || a.push(Ei.create(o.from, o.to, o.value))), o.next());
    return f.finishInner(this.nextLayer.isEmpty && !a.length ? W.empty : this.nextLayer.update({ add: a, filter: l, filterFrom: n, filterTo: s }));
  }
  /**
  Map this range set through a set of changes, return the new set.
  */
  map(t) {
    if (t.empty || this.isEmpty)
      return this;
    let e = [], i = [], n = -1;
    for (let l = 0; l < this.chunk.length; l++) {
      let o = this.chunkPos[l], h = this.chunk[l], a = t.touchesRange(o, o + h.length);
      if (a === !1)
        n = Math.max(n, h.maxPoint), e.push(h), i.push(t.mapPos(o));
      else if (a === !0) {
        let { mapped: f, pos: c } = h.map(o, t);
        f && (n = Math.max(n, f.maxPoint), e.push(f), i.push(c));
      }
    }
    let s = this.nextLayer.map(t);
    return e.length == 0 ? s : new W(i, e, s || W.empty, n);
  }
  /**
  Iterate over the ranges that touch the region `from` to `to`,
  calling `f` for each. There is no guarantee that the ranges will
  be reported in any specific order. When the callback returns
  `false`, iteration stops.
  */
  between(t, e, i) {
    if (!this.isEmpty) {
      for (let n = 0; n < this.chunk.length; n++) {
        let s = this.chunkPos[n], l = this.chunk[n];
        if (e >= s && t <= s + l.length && l.between(s, t - s, e - s, i) === !1)
          return;
      }
      this.nextLayer.between(t, e, i);
    }
  }
  /**
  Iterate over the ranges in this set, in order, including all
  ranges that end at or after `from`.
  */
  iter(t = 0) {
    return ve.from([this]).goto(t);
  }
  /**
  @internal
  */
  get isEmpty() {
    return this.nextLayer == this;
  }
  /**
  Iterate over the ranges in a collection of sets, in order,
  starting from `from`.
  */
  static iter(t, e = 0) {
    return ve.from(t).goto(e);
  }
  /**
  Iterate over two groups of sets, calling methods on `comparator`
  to notify it of possible differences.
  */
  static compare(t, e, i, n, s = -1) {
    let l = t.filter((c) => c.maxPoint > 0 || !c.isEmpty && c.maxPoint >= s), o = e.filter((c) => c.maxPoint > 0 || !c.isEmpty && c.maxPoint >= s), h = Mn(l, o, i), a = new ce(l, h, s), f = new ce(o, h, s);
    i.iterGaps((c, u, d) => On(a, c, f, u, d, n)), i.empty && i.length == 0 && On(a, 0, f, 0, 0, n);
  }
  /**
  Compare the contents of two groups of range sets, returning true
  if they are equivalent in the given range.
  */
  static eq(t, e, i = 0, n) {
    n == null && (n = 1e9 - 1);
    let s = t.filter((f) => !f.isEmpty && e.indexOf(f) < 0), l = e.filter((f) => !f.isEmpty && t.indexOf(f) < 0);
    if (s.length != l.length)
      return !1;
    if (!s.length)
      return !0;
    let o = Mn(s, l), h = new ce(s, o, 0).goto(i), a = new ce(l, o, 0).goto(i);
    for (; ; ) {
      if (h.to != a.to || !Ni(h.active, a.active) || h.point && (!a.point || !h.point.eq(a.point)))
        return !1;
      if (h.to > n)
        return !0;
      h.next(), a.next();
    }
  }
  /**
  Iterate over a group of range sets at the same time, notifying
  the iterator about the ranges covering every given piece of
  content. Returns the open count (see
  [`SpanIterator.span`](https://codemirror.net/6/docs/ref/#state.SpanIterator.span)) at the end
  of the iteration.
  */
  static spans(t, e, i, n, s = -1) {
    let l = new ce(t, null, s).goto(e), o = e, h = l.openStart;
    for (; ; ) {
      let a = Math.min(l.to, i);
      if (l.point) {
        let f = l.activeForPoint(l.to), c = l.pointFrom < e ? f.length + 1 : Math.min(f.length, h);
        n.point(o, a, l.point, f, c, l.pointRank), h = Math.min(l.openEnd(a), f.length);
      } else
        a > o && (n.span(o, a, l.active, h), h = l.openEnd(a));
      if (l.to > i)
        return h + (l.point && l.to > i ? 1 : 0);
      o = l.to, l.next();
    }
  }
  /**
  Create a range set for the given range or array of ranges. By
  default, this expects the ranges to be _sorted_ (by start
  position and, if two start at the same position,
  `value.startSide`). You can pass `true` as second argument to
  cause the method to sort them.
  */
  static of(t, e = !1) {
    let i = new ti();
    for (let n of t instanceof Ei ? [t] : e ? hl(t) : t)
      i.add(n.from, n.to, n.value);
    return i.finish();
  }
}
W.empty = /* @__PURE__ */ new W([], [], null, -1);
function hl(r) {
  if (r.length > 1)
    for (let t = r[0], e = 1; e < r.length; e++) {
      let i = r[e];
      if (Li(t, i) > 0)
        return r.slice().sort(Li);
      t = i;
    }
  return r;
}
W.empty.nextLayer = W.empty;
class ti {
  finishChunk(t) {
    this.chunks.push(new hn(this.from, this.to, this.value, this.maxPoint)), this.chunkPos.push(this.chunkStart), this.chunkStart = -1, this.setMaxPoint = Math.max(this.setMaxPoint, this.maxPoint), this.maxPoint = -1, t && (this.from = [], this.to = [], this.value = []);
  }
  /**
  Create an empty builder.
  */
  constructor() {
    this.chunks = [], this.chunkPos = [], this.chunkStart = -1, this.last = null, this.lastFrom = -1e9, this.lastTo = -1e9, this.from = [], this.to = [], this.value = [], this.maxPoint = -1, this.setMaxPoint = -1, this.nextLayer = null;
  }
  /**
  Add a range. Ranges should be added in sorted (by `from` and
  `value.startSide`) order.
  */
  add(t, e, i) {
    this.addInner(t, e, i) || (this.nextLayer || (this.nextLayer = new ti())).add(t, e, i);
  }
  /**
  @internal
  */
  addInner(t, e, i) {
    let n = t - this.lastTo || i.startSide - this.last.endSide;
    if (n <= 0 && (t - this.lastFrom || i.startSide - this.last.startSide) < 0)
      throw new Error("Ranges must be added sorted by `from` position and `startSide`");
    return n < 0 ? !1 : (this.from.length == 250 && this.finishChunk(!0), this.chunkStart < 0 && (this.chunkStart = t), this.from.push(t - this.chunkStart), this.to.push(e - this.chunkStart), this.last = i, this.lastFrom = t, this.lastTo = e, this.value.push(i), i.point && (this.maxPoint = Math.max(this.maxPoint, e - t)), !0);
  }
  /**
  @internal
  */
  addChunk(t, e) {
    if ((t - this.lastTo || e.value[0].startSide - this.last.endSide) < 0)
      return !1;
    this.from.length && this.finishChunk(!0), this.setMaxPoint = Math.max(this.setMaxPoint, e.maxPoint), this.chunks.push(e), this.chunkPos.push(t);
    let i = e.value.length - 1;
    return this.last = e.value[i], this.lastFrom = e.from[i] + t, this.lastTo = e.to[i] + t, !0;
  }
  /**
  Finish the range set. Returns the new set. The builder can't be
  used anymore after this has been called.
  */
  finish() {
    return this.finishInner(W.empty);
  }
  /**
  @internal
  */
  finishInner(t) {
    if (this.from.length && this.finishChunk(!1), this.chunks.length == 0)
      return t;
    let e = W.create(this.chunkPos, this.chunks, this.nextLayer ? this.nextLayer.finishInner(t) : t, this.setMaxPoint);
    return this.from = null, e;
  }
}
function Mn(r, t, e) {
  let i = /* @__PURE__ */ new Map();
  for (let s of r)
    for (let l = 0; l < s.chunk.length; l++)
      s.chunk[l].maxPoint <= 0 && i.set(s.chunk[l], s.chunkPos[l]);
  let n = /* @__PURE__ */ new Set();
  for (let s of t)
    for (let l = 0; l < s.chunk.length; l++) {
      let o = i.get(s.chunk[l]);
      o != null && (e ? e.mapPos(o) : o) == s.chunkPos[l] && !(e != null && e.touchesRange(o, o + s.chunk[l].length)) && n.add(s.chunk[l]);
    }
  return n;
}
class $s {
  constructor(t, e, i, n = 0) {
    this.layer = t, this.skip = e, this.minPoint = i, this.rank = n;
  }
  get startSide() {
    return this.value ? this.value.startSide : 0;
  }
  get endSide() {
    return this.value ? this.value.endSide : 0;
  }
  goto(t, e = -1e9) {
    return this.chunkIndex = this.rangeIndex = 0, this.gotoInner(t, e, !1), this;
  }
  gotoInner(t, e, i) {
    for (; this.chunkIndex < this.layer.chunk.length; ) {
      let n = this.layer.chunk[this.chunkIndex];
      if (!(this.skip && this.skip.has(n) || this.layer.chunkEnd(this.chunkIndex) < t || n.maxPoint < this.minPoint))
        break;
      this.chunkIndex++, i = !1;
    }
    if (this.chunkIndex < this.layer.chunk.length) {
      let n = this.layer.chunk[this.chunkIndex].findIndex(t - this.layer.chunkPos[this.chunkIndex], e, !0);
      (!i || this.rangeIndex < n) && this.setRangeIndex(n);
    }
    this.next();
  }
  forward(t, e) {
    (this.to - t || this.endSide - e) < 0 && this.gotoInner(t, e, !0);
  }
  next() {
    for (; ; )
      if (this.chunkIndex == this.layer.chunk.length) {
        this.from = this.to = 1e9, this.value = null;
        break;
      } else {
        let t = this.layer.chunkPos[this.chunkIndex], e = this.layer.chunk[this.chunkIndex], i = t + e.from[this.rangeIndex];
        if (this.from = i, this.to = t + e.to[this.rangeIndex], this.value = e.value[this.rangeIndex], this.setRangeIndex(this.rangeIndex + 1), this.minPoint < 0 || this.value.point && this.to - this.from >= this.minPoint)
          break;
      }
  }
  setRangeIndex(t) {
    if (t == this.layer.chunk[this.chunkIndex].value.length) {
      if (this.chunkIndex++, this.skip)
        for (; this.chunkIndex < this.layer.chunk.length && this.skip.has(this.layer.chunk[this.chunkIndex]); )
          this.chunkIndex++;
      this.rangeIndex = 0;
    } else
      this.rangeIndex = t;
  }
  nextChunk() {
    this.chunkIndex++, this.rangeIndex = 0, this.next();
  }
  compare(t) {
    return this.from - t.from || this.startSide - t.startSide || this.rank - t.rank || this.to - t.to || this.endSide - t.endSide;
  }
}
class ve {
  constructor(t) {
    this.heap = t;
  }
  static from(t, e = null, i = -1) {
    let n = [];
    for (let s = 0; s < t.length; s++)
      for (let l = t[s]; !l.isEmpty; l = l.nextLayer)
        l.maxPoint >= i && n.push(new $s(l, e, i, s));
    return n.length == 1 ? n[0] : new ve(n);
  }
  get startSide() {
    return this.value ? this.value.startSide : 0;
  }
  goto(t, e = -1e9) {
    for (let i of this.heap)
      i.goto(t, e);
    for (let i = this.heap.length >> 1; i >= 0; i--)
      di(this.heap, i);
    return this.next(), this;
  }
  forward(t, e) {
    for (let i of this.heap)
      i.forward(t, e);
    for (let i = this.heap.length >> 1; i >= 0; i--)
      di(this.heap, i);
    (this.to - t || this.value.endSide - e) < 0 && this.next();
  }
  next() {
    if (this.heap.length == 0)
      this.from = this.to = 1e9, this.value = null, this.rank = -1;
    else {
      let t = this.heap[0];
      this.from = t.from, this.to = t.to, this.value = t.value, this.rank = t.rank, t.value && t.next(), di(this.heap, 0);
    }
  }
}
function di(r, t) {
  for (let e = r[t]; ; ) {
    let i = (t << 1) + 1;
    if (i >= r.length)
      break;
    let n = r[i];
    if (i + 1 < r.length && n.compare(r[i + 1]) >= 0 && (n = r[i + 1], i++), e.compare(n) < 0)
      break;
    r[i] = e, r[t] = n, t = i;
  }
}
class ce {
  constructor(t, e, i) {
    this.minPoint = i, this.active = [], this.activeTo = [], this.activeRank = [], this.minActive = -1, this.point = null, this.pointFrom = 0, this.pointRank = 0, this.to = -1e9, this.endSide = 0, this.openStart = -1, this.cursor = ve.from(t, e, i);
  }
  goto(t, e = -1e9) {
    return this.cursor.goto(t, e), this.active.length = this.activeTo.length = this.activeRank.length = 0, this.minActive = -1, this.to = t, this.endSide = e, this.openStart = -1, this.next(), this;
  }
  forward(t, e) {
    for (; this.minActive > -1 && (this.activeTo[this.minActive] - t || this.active[this.minActive].endSide - e) < 0; )
      this.removeActive(this.minActive);
    this.cursor.forward(t, e);
  }
  removeActive(t) {
    Re(this.active, t), Re(this.activeTo, t), Re(this.activeRank, t), this.minActive = Dn(this.active, this.activeTo);
  }
  addActive(t) {
    let e = 0, { value: i, to: n, rank: s } = this.cursor;
    for (; e < this.activeRank.length && this.activeRank[e] <= s; )
      e++;
    Ee(this.active, e, i), Ee(this.activeTo, e, n), Ee(this.activeRank, e, s), t && Ee(t, e, this.cursor.from), this.minActive = Dn(this.active, this.activeTo);
  }
  // After calling this, if `this.point` != null, the next range is a
  // point. Otherwise, it's a regular range, covered by `this.active`.
  next() {
    let t = this.to, e = this.point;
    this.point = null;
    let i = this.openStart < 0 ? [] : null;
    for (; ; ) {
      let n = this.minActive;
      if (n > -1 && (this.activeTo[n] - this.cursor.from || this.active[n].endSide - this.cursor.startSide) < 0) {
        if (this.activeTo[n] > t) {
          this.to = this.activeTo[n], this.endSide = this.active[n].endSide;
          break;
        }
        this.removeActive(n), i && Re(i, n);
      } else if (this.cursor.value)
        if (this.cursor.from > t) {
          this.to = this.cursor.from, this.endSide = this.cursor.startSide;
          break;
        } else {
          let s = this.cursor.value;
          if (!s.point)
            this.addActive(i), this.cursor.next();
          else if (e && this.cursor.to == this.to && this.cursor.from < this.cursor.to)
            this.cursor.next();
          else {
            this.point = s, this.pointFrom = this.cursor.from, this.pointRank = this.cursor.rank, this.to = this.cursor.to, this.endSide = s.endSide, this.cursor.next(), this.forward(this.to, this.endSide);
            break;
          }
        }
      else {
        this.to = this.endSide = 1e9;
        break;
      }
    }
    if (i) {
      this.openStart = 0;
      for (let n = i.length - 1; n >= 0 && i[n] < t; n--)
        this.openStart++;
    }
  }
  activeForPoint(t) {
    if (!this.active.length)
      return this.active;
    let e = [];
    for (let i = this.active.length - 1; i >= 0 && !(this.activeRank[i] < this.pointRank); i--)
      (this.activeTo[i] > t || this.activeTo[i] == t && this.active[i].endSide >= this.point.endSide) && e.push(this.active[i]);
    return e.reverse();
  }
  openEnd(t) {
    let e = 0;
    for (let i = this.activeTo.length - 1; i >= 0 && this.activeTo[i] > t; i--)
      e++;
    return e;
  }
}
function On(r, t, e, i, n, s) {
  r.goto(t), e.goto(i);
  let l = i + n, o = i, h = i - t;
  for (; ; ) {
    let a = r.to + h - e.to || r.endSide - e.endSide, f = a < 0 ? r.to + h : e.to, c = Math.min(f, l);
    if (r.point || e.point ? r.point && e.point && (r.point == e.point || r.point.eq(e.point)) && Ni(r.activeForPoint(r.to), e.activeForPoint(e.to)) || s.comparePoint(o, c, r.point, e.point) : c > o && !Ni(r.active, e.active) && s.compareRange(o, c, r.active, e.active), f > l)
      break;
    o = f, a <= 0 && r.next(), a >= 0 && e.next();
  }
}
function Ni(r, t) {
  if (r.length != t.length)
    return !1;
  for (let e = 0; e < r.length; e++)
    if (r[e] != t[e] && !r[e].eq(t[e]))
      return !1;
  return !0;
}
function Re(r, t) {
  for (let e = t, i = r.length - 1; e < i; e++)
    r[e] = r[e + 1];
  r.pop();
}
function Ee(r, t, e) {
  for (let i = r.length - 1; i >= t; i--)
    r[i + 1] = r[i];
  r[t] = e;
}
function Dn(r, t) {
  let e = -1, i = 1e9;
  for (let n = 0; n < t.length; n++)
    (t[n] - i || r[n].endSide - r[e].endSide) < 0 && (e = n, i = t[n]);
  return e;
}
function al(r, t, e, i) {
  for (let n = 0, s = 0; ; ) {
    if (s >= t)
      return n;
    if (n == r.length)
      break;
    s += r.charCodeAt(n) == 9 ? e - s % e : 1, n = kt(r, n);
  }
  return i === !0 ? -1 : r.length;
}
const Ii = "ͼ", Tn = typeof Symbol > "u" ? "__" + Ii : Symbol.for(Ii), Hi = typeof Symbol > "u" ? "__styleSet" + Math.floor(Math.random() * 1e8) : Symbol("styleSet"), Pn = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : {};
class ie {
  // :: (Object<Style>, ?{finish: ?(string) → string})
  // Create a style module from the given spec.
  //
  // When `finish` is given, it is called on regular (non-`@`)
  // selectors (after `&` expansion) to compute the final selector.
  constructor(t, e) {
    this.rules = [];
    let { finish: i } = e || {};
    function n(l) {
      return /^@/.test(l) ? [l] : l.split(/,\s*/);
    }
    function s(l, o, h, a) {
      let f = [], c = /^@(\w+)\b/.exec(l[0]), u = c && c[1] == "keyframes";
      if (c && o == null)
        return h.push(l[0] + ";");
      for (let d in o) {
        let g = o[d];
        if (/&/.test(d))
          s(
            d.split(/,\s*/).map((m) => l.map((p) => m.replace(/&/, p))).reduce((m, p) => m.concat(p)),
            g,
            h
          );
        else if (g && typeof g == "object") {
          if (!c)
            throw new RangeError("The value of a property (" + d + ") should be a primitive value.");
          s(n(d), g, f, u);
        } else
          g != null && f.push(d.replace(/_.*/, "").replace(/[A-Z]/g, (m) => "-" + m.toLowerCase()) + ": " + g + ";");
      }
      (f.length || u) && h.push((i && !c && !a ? l.map(i) : l).join(", ") + " {" + f.join(" ") + "}");
    }
    for (let l in t)
      s(n(l), t[l], this.rules);
  }
  // :: () → string
  // Returns a string containing the module's CSS rules.
  getRules() {
    return this.rules.join(`
`);
  }
  // :: () → string
  // Generate a new unique CSS class name.
  static newName() {
    let t = Pn[Tn] || 1;
    return Pn[Tn] = t + 1, Ii + t.toString(36);
  }
  // :: (union<Document, ShadowRoot>, union<[StyleModule], StyleModule>, ?{nonce: ?string})
  //
  // Mount the given set of modules in the given DOM root, which ensures
  // that the CSS rules defined by the module are available in that
  // context.
  //
  // Rules are only added to the document once per root.
  //
  // Rule order will follow the order of the modules, so that rules from
  // modules later in the array take precedence of those from earlier
  // modules. If you call this function multiple times for the same root
  // in a way that changes the order of already mounted modules, the old
  // order will be changed.
  //
  // If a Content Security Policy nonce is provided, it is added to
  // the `<style>` tag generated by the library.
  static mount(t, e, i) {
    let n = t[Hi], s = i && i.nonce;
    n ? s && n.setNonce(s) : n = new fl(t, s), n.mount(Array.isArray(e) ? e : [e]);
  }
}
let Bn = /* @__PURE__ */ new Map();
class fl {
  constructor(t, e) {
    let i = t.ownerDocument || t, n = i.defaultView;
    if (!t.head && t.adoptedStyleSheets && n.CSSStyleSheet) {
      let s = Bn.get(i);
      if (s)
        return t.adoptedStyleSheets = [s.sheet, ...t.adoptedStyleSheets], t[Hi] = s;
      this.sheet = new n.CSSStyleSheet(), t.adoptedStyleSheets = [this.sheet, ...t.adoptedStyleSheets], Bn.set(i, this);
    } else {
      this.styleTag = i.createElement("style"), e && this.styleTag.setAttribute("nonce", e);
      let s = t.head || t;
      s.insertBefore(this.styleTag, s.firstChild);
    }
    this.modules = [], t[Hi] = this;
  }
  mount(t) {
    let e = this.sheet, i = 0, n = 0;
    for (let s = 0; s < t.length; s++) {
      let l = t[s], o = this.modules.indexOf(l);
      if (o < n && o > -1 && (this.modules.splice(o, 1), n--, o = -1), o == -1) {
        if (this.modules.splice(n++, 0, l), e)
          for (let h = 0; h < l.rules.length; h++)
            e.insertRule(l.rules[h], i++);
      } else {
        for (; n < o; )
          i += this.modules[n++].rules.length;
        i += l.rules.length, n++;
      }
    }
    if (!e) {
      let s = "";
      for (let l = 0; l < this.modules.length; l++)
        s += this.modules[l].getRules() + `
`;
      this.styleTag.textContent = s;
    }
  }
  setNonce(t) {
    this.styleTag && this.styleTag.getAttribute("nonce") != t && this.styleTag.setAttribute("nonce", t);
  }
}
var ne = {
  8: "Backspace",
  9: "Tab",
  10: "Enter",
  12: "NumLock",
  13: "Enter",
  16: "Shift",
  17: "Control",
  18: "Alt",
  20: "CapsLock",
  27: "Escape",
  32: " ",
  33: "PageUp",
  34: "PageDown",
  35: "End",
  36: "Home",
  37: "ArrowLeft",
  38: "ArrowUp",
  39: "ArrowRight",
  40: "ArrowDown",
  44: "PrintScreen",
  45: "Insert",
  46: "Delete",
  59: ";",
  61: "=",
  91: "Meta",
  92: "Meta",
  106: "*",
  107: "+",
  108: ",",
  109: "-",
  110: ".",
  111: "/",
  144: "NumLock",
  145: "ScrollLock",
  160: "Shift",
  161: "Shift",
  162: "Control",
  163: "Control",
  164: "Alt",
  165: "Alt",
  173: "-",
  186: ";",
  187: "=",
  188: ",",
  189: "-",
  190: ".",
  191: "/",
  192: "`",
  219: "[",
  220: "\\",
  221: "]",
  222: "'"
}, Fi = {
  48: ")",
  49: "!",
  50: "@",
  51: "#",
  52: "$",
  53: "%",
  54: "^",
  55: "&",
  56: "*",
  57: "(",
  59: ":",
  61: "+",
  173: "_",
  186: ":",
  187: "+",
  188: "<",
  189: "_",
  190: ">",
  191: "?",
  192: "~",
  219: "{",
  220: "|",
  221: "}",
  222: '"'
};
for (var G = 0; G < 10; G++)
  ne[48 + G] = ne[96 + G] = String(G);
for (var G = 1; G <= 24; G++)
  ne[G + 111] = "F" + G;
for (var G = 65; G <= 90; G++)
  ne[G] = String.fromCharCode(G + 32), Fi[G] = String.fromCharCode(G);
for (var gi in ne)
  Fi.hasOwnProperty(gi) || (Fi[gi] = ne[gi]);
function ei(r) {
  let t;
  return r.nodeType == 11 ? t = r.getSelection ? r : r.ownerDocument : t = r, t.getSelection();
}
function Vi(r, t) {
  return t ? r == t || r.contains(t.nodeType != 1 ? t.parentNode : t) : !1;
}
function cl(r) {
  let t = r.activeElement;
  for (; t && t.shadowRoot; )
    t = t.shadowRoot.activeElement;
  return t;
}
function Je(r, t) {
  if (!t.anchorNode)
    return !1;
  try {
    return Vi(r, t.anchorNode);
  } catch {
    return !1;
  }
}
function Se(r) {
  return r.nodeType == 3 ? Gt(r, 0, r.nodeValue.length).getClientRects() : r.nodeType == 1 ? r.getClientRects() : [];
}
function ii(r, t, e, i) {
  return e ? Rn(r, t, e, i, -1) || Rn(r, t, e, i, 1) : !1;
}
function Ae(r) {
  for (var t = 0; ; t++)
    if (r = r.previousSibling, !r)
      return t;
}
function Rn(r, t, e, i, n) {
  for (; ; ) {
    if (r == e && t == i)
      return !0;
    if (t == (n < 0 ? 0 : Mt(r))) {
      if (r.nodeName == "DIV")
        return !1;
      let s = r.parentNode;
      if (!s || s.nodeType != 1)
        return !1;
      t = Ae(r) + (n < 0 ? 0 : 1), r = s;
    } else if (r.nodeType == 1) {
      if (r = r.childNodes[t + (n < 0 ? -1 : 0)], r.nodeType == 1 && r.contentEditable == "false")
        return !1;
      t = n < 0 ? Mt(r) : 0;
    } else
      return !1;
  }
}
function Mt(r) {
  return r.nodeType == 3 ? r.nodeValue.length : r.childNodes.length;
}
function an(r, t) {
  let e = t ? r.left : r.right;
  return { left: e, right: e, top: r.top, bottom: r.bottom };
}
function ul(r) {
  return {
    left: 0,
    right: r.innerWidth,
    top: 0,
    bottom: r.innerHeight
  };
}
function dl(r, t, e, i, n, s, l, o) {
  let h = r.ownerDocument, a = h.defaultView || window;
  for (let f = r, c = !1; f && !c; )
    if (f.nodeType == 1) {
      let u, d = f == h.body, g = 1, m = 1;
      if (d)
        u = ul(a);
      else {
        if (/^(fixed|sticky)$/.test(getComputedStyle(f).position) && (c = !0), f.scrollHeight <= f.clientHeight && f.scrollWidth <= f.clientWidth) {
          f = f.assignedSlot || f.parentNode;
          continue;
        }
        let S = f.getBoundingClientRect();
        g = S.width / f.offsetWidth, m = S.height / f.offsetHeight, u = {
          left: S.left,
          right: S.left + f.clientWidth * g,
          top: S.top,
          bottom: S.top + f.clientHeight * m
        };
      }
      let p = 0, y = 0;
      if (n == "nearest")
        t.top < u.top ? (y = -(u.top - t.top + l), e > 0 && t.bottom > u.bottom + y && (y = t.bottom - u.bottom + y + l)) : t.bottom > u.bottom && (y = t.bottom - u.bottom + l, e < 0 && t.top - y < u.top && (y = -(u.top + y - t.top + l)));
      else {
        let S = t.bottom - t.top, C = u.bottom - u.top;
        y = (n == "center" && S <= C ? t.top + S / 2 - C / 2 : n == "start" || n == "center" && e < 0 ? t.top - l : t.bottom - C + l) - u.top;
      }
      if (i == "nearest" ? t.left < u.left ? (p = -(u.left - t.left + s), e > 0 && t.right > u.right + p && (p = t.right - u.right + p + s)) : t.right > u.right && (p = t.right - u.right + s, e < 0 && t.left < u.left + p && (p = -(u.left + p - t.left + s))) : p = (i == "center" ? t.left + (t.right - t.left) / 2 - (u.right - u.left) / 2 : i == "start" == o ? t.left - s : t.right - (u.right - u.left) + s) - u.left, p || y)
        if (d)
          a.scrollBy(p, y);
        else {
          let S = 0, C = 0;
          if (y) {
            let b = f.scrollTop;
            f.scrollTop += y / m, C = (f.scrollTop - b) * m;
          }
          if (p) {
            let b = f.scrollLeft;
            f.scrollLeft += p / g, S = (f.scrollLeft - b) * g;
          }
          t = {
            left: t.left - S,
            top: t.top - C,
            right: t.right - S,
            bottom: t.bottom - C
          }, S && Math.abs(S - p) < 1 && (i = "nearest"), C && Math.abs(C - y) < 1 && (n = "nearest");
        }
      if (d)
        break;
      f = f.assignedSlot || f.parentNode;
    } else if (f.nodeType == 11)
      f = f.host;
    else
      break;
}
function gl(r) {
  let t = r.ownerDocument;
  for (let e = r.parentNode; e && e != t.body; )
    if (e.nodeType == 1) {
      if (e.scrollHeight > e.clientHeight || e.scrollWidth > e.clientWidth)
        return e;
      e = e.assignedSlot || e.parentNode;
    } else if (e.nodeType == 11)
      e = e.host;
    else
      break;
  return null;
}
class pl {
  constructor() {
    this.anchorNode = null, this.anchorOffset = 0, this.focusNode = null, this.focusOffset = 0;
  }
  eq(t) {
    return this.anchorNode == t.anchorNode && this.anchorOffset == t.anchorOffset && this.focusNode == t.focusNode && this.focusOffset == t.focusOffset;
  }
  setRange(t) {
    let { anchorNode: e, focusNode: i } = t;
    this.set(e, Math.min(t.anchorOffset, e ? Mt(e) : 0), i, Math.min(t.focusOffset, i ? Mt(i) : 0));
  }
  set(t, e, i, n) {
    this.anchorNode = t, this.anchorOffset = e, this.focusNode = i, this.focusOffset = n;
  }
}
let _t = null;
function Ys(r) {
  if (r.setActive)
    return r.setActive();
  if (_t)
    return r.focus(_t);
  let t = [];
  for (let e = r; e && (t.push(e, e.scrollTop, e.scrollLeft), e != e.ownerDocument); e = e.parentNode)
    ;
  if (r.focus(_t == null ? {
    get preventScroll() {
      return _t = { preventScroll: !0 }, !0;
    }
  } : void 0), !_t) {
    _t = !1;
    for (let e = 0; e < t.length; ) {
      let i = t[e++], n = t[e++], s = t[e++];
      i.scrollTop != n && (i.scrollTop = n), i.scrollLeft != s && (i.scrollLeft = s);
    }
  }
}
let En;
function Gt(r, t, e = t) {
  let i = En || (En = document.createRange());
  return i.setEnd(r, e), i.setStart(r, t), i;
}
function te(r, t, e) {
  let i = { key: t, code: t, keyCode: e, which: e, cancelable: !0 }, n = new KeyboardEvent("keydown", i);
  n.synthetic = !0, r.dispatchEvent(n);
  let s = new KeyboardEvent("keyup", i);
  return s.synthetic = !0, r.dispatchEvent(s), n.defaultPrevented || s.defaultPrevented;
}
function ml(r) {
  for (; r; ) {
    if (r && (r.nodeType == 9 || r.nodeType == 11 && r.host))
      return r;
    r = r.assignedSlot || r.parentNode;
  }
  return null;
}
function Gs(r) {
  for (; r.attributes.length; )
    r.removeAttributeNode(r.attributes[0]);
}
function yl(r, t) {
  let e = t.focusNode, i = t.focusOffset;
  if (!e || t.anchorNode != e || t.anchorOffset != i)
    return !1;
  for (i = Math.min(i, Mt(e)); ; )
    if (i) {
      if (e.nodeType != 1)
        return !1;
      let n = e.childNodes[i - 1];
      n.contentEditable == "false" ? i-- : (e = n, i = Mt(e));
    } else {
      if (e == r)
        return !0;
      i = Ae(e), e = e.parentNode;
    }
}
function Js(r) {
  return r.scrollTop > Math.max(1, r.scrollHeight - r.clientHeight - 4);
}
class X {
  constructor(t, e, i = !0) {
    this.node = t, this.offset = e, this.precise = i;
  }
  static before(t, e) {
    return new X(t.parentNode, Ae(t), e);
  }
  static after(t, e) {
    return new X(t.parentNode, Ae(t) + 1, e);
  }
}
const fn = [];
class I {
  constructor() {
    this.parent = null, this.dom = null, this.flags = 2;
  }
  get overrideDOMText() {
    return null;
  }
  get posAtStart() {
    return this.parent ? this.parent.posBefore(this) : 0;
  }
  get posAtEnd() {
    return this.posAtStart + this.length;
  }
  posBefore(t) {
    let e = this.posAtStart;
    for (let i of this.children) {
      if (i == t)
        return e;
      e += i.length + i.breakAfter;
    }
    throw new RangeError("Invalid child in posBefore");
  }
  posAfter(t) {
    return this.posBefore(t) + t.length;
  }
  sync(t, e) {
    if (this.flags & 2) {
      let i = this.dom, n = null, s;
      for (let l of this.children) {
        if (l.flags & 7) {
          if (!l.dom && (s = n ? n.nextSibling : i.firstChild)) {
            let o = I.get(s);
            (!o || !o.parent && o.canReuseDOM(l)) && l.reuseDOM(s);
          }
          l.sync(t, e), l.flags &= -8;
        }
        if (s = n ? n.nextSibling : i.firstChild, e && !e.written && e.node == i && s != l.dom && (e.written = !0), l.dom.parentNode == i)
          for (; s && s != l.dom; )
            s = Ln(s);
        else
          i.insertBefore(l.dom, s);
        n = l.dom;
      }
      for (s = n ? n.nextSibling : i.firstChild, s && e && e.node == i && (e.written = !0); s; )
        s = Ln(s);
    } else if (this.flags & 1)
      for (let i of this.children)
        i.flags & 7 && (i.sync(t, e), i.flags &= -8);
  }
  reuseDOM(t) {
  }
  localPosFromDOM(t, e) {
    let i;
    if (t == this.dom)
      i = this.dom.childNodes[e];
    else {
      let n = Mt(t) == 0 ? 0 : e == 0 ? -1 : 1;
      for (; ; ) {
        let s = t.parentNode;
        if (s == this.dom)
          break;
        n == 0 && s.firstChild != s.lastChild && (t == s.firstChild ? n = -1 : n = 1), t = s;
      }
      n < 0 ? i = t : i = t.nextSibling;
    }
    if (i == this.dom.firstChild)
      return 0;
    for (; i && !I.get(i); )
      i = i.nextSibling;
    if (!i)
      return this.length;
    for (let n = 0, s = 0; ; n++) {
      let l = this.children[n];
      if (l.dom == i)
        return s;
      s += l.length + l.breakAfter;
    }
  }
  domBoundsAround(t, e, i = 0) {
    let n = -1, s = -1, l = -1, o = -1;
    for (let h = 0, a = i, f = i; h < this.children.length; h++) {
      let c = this.children[h], u = a + c.length;
      if (a < t && u > e)
        return c.domBoundsAround(t, e, a);
      if (u >= t && n == -1 && (n = h, s = a), a > e && c.dom.parentNode == this.dom) {
        l = h, o = f;
        break;
      }
      f = u, a = u + c.breakAfter;
    }
    return {
      from: s,
      to: o < 0 ? i + this.length : o,
      startDOM: (n ? this.children[n - 1].dom.nextSibling : null) || this.dom.firstChild,
      endDOM: l < this.children.length && l >= 0 ? this.children[l].dom : null
    };
  }
  markDirty(t = !1) {
    this.flags |= 2, this.markParentsDirty(t);
  }
  markParentsDirty(t) {
    for (let e = this.parent; e; e = e.parent) {
      if (t && (e.flags |= 2), e.flags & 1)
        return;
      e.flags |= 1, t = !1;
    }
  }
  setParent(t) {
    this.parent != t && (this.parent = t, this.flags & 7 && this.markParentsDirty(!0));
  }
  setDOM(t) {
    this.dom != t && (this.dom && (this.dom.cmView = null), this.dom = t, t.cmView = this);
  }
  get rootView() {
    for (let t = this; ; ) {
      let e = t.parent;
      if (!e)
        return t;
      t = e;
    }
  }
  replaceChildren(t, e, i = fn) {
    this.markDirty();
    for (let n = t; n < e; n++) {
      let s = this.children[n];
      s.parent == this && s.destroy();
    }
    this.children.splice(t, e - t, ...i);
    for (let n = 0; n < i.length; n++)
      i[n].setParent(this);
  }
  ignoreMutation(t) {
    return !1;
  }
  ignoreEvent(t) {
    return !1;
  }
  childCursor(t = this.length) {
    return new Us(this.children, t, this.children.length);
  }
  childPos(t, e = 1) {
    return this.childCursor().findPos(t, e);
  }
  toString() {
    let t = this.constructor.name.replace("View", "");
    return t + (this.children.length ? "(" + this.children.join() + ")" : this.length ? "[" + (t == "Text" ? this.text : this.length) + "]" : "") + (this.breakAfter ? "#" : "");
  }
  static get(t) {
    return t.cmView;
  }
  get isEditable() {
    return !0;
  }
  get isWidget() {
    return !1;
  }
  get isHidden() {
    return !1;
  }
  merge(t, e, i, n, s, l) {
    return !1;
  }
  become(t) {
    return !1;
  }
  canReuseDOM(t) {
    return t.constructor == this.constructor && !((this.flags | t.flags) & 8);
  }
  // When this is a zero-length view with a side, this should return a
  // number <= 0 to indicate it is before its position, or a
  // number > 0 when after its position.
  getSide() {
    return 0;
  }
  destroy() {
    this.parent = null;
  }
}
I.prototype.breakAfter = 0;
function Ln(r) {
  let t = r.nextSibling;
  return r.parentNode.removeChild(r), t;
}
class Us {
  constructor(t, e, i) {
    this.children = t, this.pos = e, this.i = i, this.off = 0;
  }
  findPos(t, e = 1) {
    for (; ; ) {
      if (t > this.pos || t == this.pos && (e > 0 || this.i == 0 || this.children[this.i - 1].breakAfter))
        return this.off = t - this.pos, this;
      let i = this.children[--this.i];
      this.pos -= i.length + i.breakAfter;
    }
  }
}
function Xs(r, t, e, i, n, s, l, o, h) {
  let { children: a } = r, f = a.length ? a[t] : null, c = s.length ? s[s.length - 1] : null, u = c ? c.breakAfter : l;
  if (!(t == i && f && !l && !u && s.length < 2 && f.merge(e, n, s.length ? c : null, e == 0, o, h))) {
    if (i < a.length) {
      let d = a[i];
      d && (n < d.length || d.breakAfter && (c != null && c.breakAfter)) ? (t == i && (d = d.split(n), n = 0), !u && c && d.merge(0, n, c, !0, 0, h) ? s[s.length - 1] = d : (n && d.merge(0, n, null, !1, 0, h), s.push(d))) : d != null && d.breakAfter && (c ? c.breakAfter = 1 : l = 1), i++;
    }
    for (f && (f.breakAfter = l, e > 0 && (!l && s.length && f.merge(e, f.length, s[0], !1, o, 0) ? f.breakAfter = s.shift().breakAfter : (e < f.length || f.children.length && f.children[f.children.length - 1].length == 0) && f.merge(e, f.length, null, !1, o, 0), t++)); t < i && s.length; )
      if (a[i - 1].become(s[s.length - 1]))
        i--, s.pop(), h = s.length ? 0 : o;
      else if (a[t].become(s[0]))
        t++, s.shift(), o = s.length ? 0 : h;
      else
        break;
    !s.length && t && i < a.length && !a[t - 1].breakAfter && a[i].merge(0, 0, a[t - 1], !1, o, h) && t--, (t < i || s.length) && r.replaceChildren(t, i, s);
  }
}
function _s(r, t, e, i, n, s) {
  let l = r.childCursor(), { i: o, off: h } = l.findPos(e, 1), { i: a, off: f } = l.findPos(t, -1), c = t - e;
  for (let u of i)
    c += u.length;
  r.length += c, Xs(r, a, f, o, h, i, 0, n, s);
}
let st = typeof navigator < "u" ? navigator : { userAgent: "", vendor: "", platform: "" }, Wi = typeof document < "u" ? document : { documentElement: { style: {} } };
const zi = /* @__PURE__ */ /Edge\/(\d+)/.exec(st.userAgent), Zs = /* @__PURE__ */ /MSIE \d/.test(st.userAgent), qi = /* @__PURE__ */ /Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(st.userAgent), fi = !!(Zs || qi || zi), Nn = !fi && /* @__PURE__ */ /gecko\/(\d+)/i.test(st.userAgent), pi = !fi && /* @__PURE__ */ /Chrome\/(\d+)/.exec(st.userAgent), In = "webkitFontSmoothing" in Wi.documentElement.style, Qs = !fi && /* @__PURE__ */ /Apple Computer/.test(st.vendor), Hn = Qs && (/* @__PURE__ */ /Mobile\/\w+/.test(st.userAgent) || st.maxTouchPoints > 2);
var k = {
  mac: Hn || /* @__PURE__ */ /Mac/.test(st.platform),
  windows: /* @__PURE__ */ /Win/.test(st.platform),
  linux: /* @__PURE__ */ /Linux|X11/.test(st.platform),
  ie: fi,
  ie_version: Zs ? Wi.documentMode || 6 : qi ? +qi[1] : zi ? +zi[1] : 0,
  gecko: Nn,
  gecko_version: Nn ? +(/* @__PURE__ */ /Firefox\/(\d+)/.exec(st.userAgent) || [0, 0])[1] : 0,
  chrome: !!pi,
  chrome_version: pi ? +pi[1] : 0,
  ios: Hn,
  android: /* @__PURE__ */ /Android\b/.test(st.userAgent),
  webkit: In,
  safari: Qs,
  webkit_version: In ? +(/* @__PURE__ */ /\bAppleWebKit\/(\d+)/.exec(navigator.userAgent) || [0, 0])[1] : 0,
  tabSize: Wi.documentElement.style.tabSize != null ? "tab-size" : "-moz-tab-size"
};
const wl = 256;
class Ot extends I {
  constructor(t) {
    super(), this.text = t;
  }
  get length() {
    return this.text.length;
  }
  createDOM(t) {
    this.setDOM(t || document.createTextNode(this.text));
  }
  sync(t, e) {
    this.dom || this.createDOM(), this.dom.nodeValue != this.text && (e && e.node == this.dom && (e.written = !0), this.dom.nodeValue = this.text);
  }
  reuseDOM(t) {
    t.nodeType == 3 && this.createDOM(t);
  }
  merge(t, e, i) {
    return this.flags & 8 || i && (!(i instanceof Ot) || this.length - (e - t) + i.length > wl || i.flags & 8) ? !1 : (this.text = this.text.slice(0, t) + (i ? i.text : "") + this.text.slice(e), this.markDirty(), !0);
  }
  split(t) {
    let e = new Ot(this.text.slice(t));
    return this.text = this.text.slice(0, t), this.markDirty(), e.flags |= this.flags & 8, e;
  }
  localPosFromDOM(t, e) {
    return t == this.dom ? e : e ? this.text.length : 0;
  }
  domAtPos(t) {
    return new X(this.dom, t);
  }
  domBoundsAround(t, e, i) {
    return { from: i, to: i + this.length, startDOM: this.dom, endDOM: this.dom.nextSibling };
  }
  coordsAt(t, e) {
    return bl(this.dom, t, e);
  }
}
class Dt extends I {
  constructor(t, e = [], i = 0) {
    super(), this.mark = t, this.children = e, this.length = i;
    for (let n of e)
      n.setParent(this);
  }
  setAttrs(t) {
    if (Gs(t), this.mark.class && (t.className = this.mark.class), this.mark.attrs)
      for (let e in this.mark.attrs)
        t.setAttribute(e, this.mark.attrs[e]);
    return t;
  }
  canReuseDOM(t) {
    return super.canReuseDOM(t) && !((this.flags | t.flags) & 8);
  }
  reuseDOM(t) {
    t.nodeName == this.mark.tagName.toUpperCase() && (this.setDOM(t), this.flags |= 6);
  }
  sync(t, e) {
    this.dom ? this.flags & 4 && this.setAttrs(this.dom) : this.setDOM(this.setAttrs(document.createElement(this.mark.tagName))), super.sync(t, e);
  }
  merge(t, e, i, n, s, l) {
    return i && (!(i instanceof Dt && i.mark.eq(this.mark)) || t && s <= 0 || e < this.length && l <= 0) ? !1 : (_s(this, t, e, i ? i.children : [], s - 1, l - 1), this.markDirty(), !0);
  }
  split(t) {
    let e = [], i = 0, n = -1, s = 0;
    for (let o of this.children) {
      let h = i + o.length;
      h > t && e.push(i < t ? o.split(t - i) : o), n < 0 && i >= t && (n = s), i = h, s++;
    }
    let l = this.length - t;
    return this.length = t, n > -1 && (this.children.length = n, this.markDirty()), new Dt(this.mark, e, l);
  }
  domAtPos(t) {
    return tr(this, t);
  }
  coordsAt(t, e) {
    return ir(this, t, e);
  }
}
function bl(r, t, e) {
  let i = r.nodeValue.length;
  t > i && (t = i);
  let n = t, s = t, l = 0;
  t == 0 && e < 0 || t == i && e >= 0 ? k.chrome || k.gecko || (t ? (n--, l = 1) : s < i && (s++, l = -1)) : e < 0 ? n-- : s < i && s++;
  let o = Gt(r, n, s).getClientRects();
  if (!o.length)
    return null;
  let h = o[(l ? l < 0 : e >= 0) ? 0 : o.length - 1];
  return k.safari && !l && h.width == 0 && (h = Array.prototype.find.call(o, (a) => a.width) || h), l ? an(h, l < 0) : h || null;
}
class Kt extends I {
  static create(t, e, i) {
    return new Kt(t, e, i);
  }
  constructor(t, e, i) {
    super(), this.widget = t, this.length = e, this.side = i, this.prevWidget = null;
  }
  split(t) {
    let e = Kt.create(this.widget, this.length - t, this.side);
    return this.length -= t, e;
  }
  sync(t) {
    (!this.dom || !this.widget.updateDOM(this.dom, t)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(t)), this.dom.contentEditable = "false");
  }
  getSide() {
    return this.side;
  }
  merge(t, e, i, n, s, l) {
    return i && (!(i instanceof Kt) || !this.widget.compare(i.widget) || t > 0 && s <= 0 || e < this.length && l <= 0) ? !1 : (this.length = t + (i ? i.length : 0) + (this.length - e), !0);
  }
  become(t) {
    return t instanceof Kt && t.side == this.side && this.widget.constructor == t.widget.constructor ? (this.widget.compare(t.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = t.widget, this.length = t.length, !0) : !1;
  }
  ignoreMutation() {
    return !0;
  }
  ignoreEvent(t) {
    return this.widget.ignoreEvent(t);
  }
  get overrideDOMText() {
    if (this.length == 0)
      return R.empty;
    let t = this;
    for (; t.parent; )
      t = t.parent;
    let { view: e } = t, i = e && e.state.doc, n = this.posAtStart;
    return i ? i.slice(n, n + this.length) : R.empty;
  }
  domAtPos(t) {
    return (this.length ? t == 0 : this.side > 0) ? X.before(this.dom) : X.after(this.dom, t == this.length);
  }
  domBoundsAround() {
    return null;
  }
  coordsAt(t, e) {
    let i = this.widget.coordsAt(this.dom, t, e);
    if (i)
      return i;
    let n = this.dom.getClientRects(), s = null;
    if (!n.length)
      return null;
    let l = this.side ? this.side < 0 : t > 0;
    for (let o = l ? n.length - 1 : 0; s = n[o], !(t > 0 ? o == 0 : o == n.length - 1 || s.top < s.bottom); o += l ? -1 : 1)
      ;
    return an(s, !l);
  }
  get isEditable() {
    return !1;
  }
  get isWidget() {
    return !0;
  }
  get isHidden() {
    return this.widget.isHidden;
  }
  destroy() {
    super.destroy(), this.dom && this.widget.destroy(this.dom);
  }
}
class se extends I {
  constructor(t) {
    super(), this.side = t;
  }
  get length() {
    return 0;
  }
  merge() {
    return !1;
  }
  become(t) {
    return t instanceof se && t.side == this.side;
  }
  split() {
    return new se(this.side);
  }
  sync() {
    if (!this.dom) {
      let t = document.createElement("img");
      t.className = "cm-widgetBuffer", t.setAttribute("aria-hidden", "true"), this.setDOM(t);
    }
  }
  getSide() {
    return this.side;
  }
  domAtPos(t) {
    return this.side > 0 ? X.before(this.dom) : X.after(this.dom);
  }
  localPosFromDOM() {
    return 0;
  }
  domBoundsAround() {
    return null;
  }
  coordsAt(t) {
    return this.dom.getBoundingClientRect();
  }
  get overrideDOMText() {
    return R.empty;
  }
  get isHidden() {
    return !0;
  }
}
Ot.prototype.children = Kt.prototype.children = se.prototype.children = fn;
function tr(r, t) {
  let e = r.dom, { children: i } = r, n = 0;
  for (let s = 0; n < i.length; n++) {
    let l = i[n], o = s + l.length;
    if (!(o == s && l.getSide() <= 0)) {
      if (t > s && t < o && l.dom.parentNode == e)
        return l.domAtPos(t - s);
      if (t <= s)
        break;
      s = o;
    }
  }
  for (let s = n; s > 0; s--) {
    let l = i[s - 1];
    if (l.dom.parentNode == e)
      return l.domAtPos(l.length);
  }
  for (let s = n; s < i.length; s++) {
    let l = i[s];
    if (l.dom.parentNode == e)
      return l.domAtPos(0);
  }
  return new X(e, 0);
}
function er(r, t, e) {
  let i, { children: n } = r;
  e > 0 && t instanceof Dt && n.length && (i = n[n.length - 1]) instanceof Dt && i.mark.eq(t.mark) ? er(i, t.children[0], e - 1) : (n.push(t), t.setParent(r)), r.length += t.length;
}
function ir(r, t, e) {
  let i = null, n = -1, s = null, l = -1;
  function o(a, f) {
    for (let c = 0, u = 0; c < a.children.length && u <= f; c++) {
      let d = a.children[c], g = u + d.length;
      g >= f && (d.children.length ? o(d, f - u) : (!s || s.isHidden && e > 0) && (g > f || u == g && d.getSide() > 0) ? (s = d, l = f - u) : (u < f || u == g && d.getSide() < 0 && !d.isHidden) && (i = d, n = f - u)), u = g;
    }
  }
  o(r, t);
  let h = (e < 0 ? i : s) || i || s;
  return h ? h.coordsAt(Math.max(0, h == i ? n : l), e) : xl(r);
}
function xl(r) {
  let t = r.dom.lastChild;
  if (!t)
    return r.dom.getBoundingClientRect();
  let e = Se(t);
  return e[e.length - 1] || null;
}
function Ki(r, t) {
  for (let e in r)
    e == "class" && t.class ? t.class += " " + r.class : e == "style" && t.style ? t.style += ";" + r.style : t[e] = r[e];
  return t;
}
const Fn = /* @__PURE__ */ Object.create(null);
function cn(r, t, e) {
  if (r == t)
    return !0;
  r || (r = Fn), t || (t = Fn);
  let i = Object.keys(r), n = Object.keys(t);
  if (i.length - (e && i.indexOf(e) > -1 ? 1 : 0) != n.length - (e && n.indexOf(e) > -1 ? 1 : 0))
    return !1;
  for (let s of i)
    if (s != e && (n.indexOf(s) == -1 || r[s] !== t[s]))
      return !1;
  return !0;
}
function ji(r, t, e) {
  let i = !1;
  if (t)
    for (let n in t)
      e && n in e || (i = !0, n == "style" ? r.style.cssText = "" : r.removeAttribute(n));
  if (e)
    for (let n in e)
      t && t[n] == e[n] || (i = !0, n == "style" ? r.style.cssText = e[n] : r.setAttribute(n, e[n]));
  return i;
}
function kl(r) {
  let t = /* @__PURE__ */ Object.create(null);
  for (let e = 0; e < r.attributes.length; e++) {
    let i = r.attributes[e];
    t[i.name] = i.value;
  }
  return t;
}
class K extends I {
  constructor() {
    super(...arguments), this.children = [], this.length = 0, this.prevAttrs = void 0, this.attrs = null, this.breakAfter = 0;
  }
  // Consumes source
  merge(t, e, i, n, s, l) {
    if (i) {
      if (!(i instanceof K))
        return !1;
      this.dom || i.transferDOM(this);
    }
    return n && this.setDeco(i ? i.attrs : null), _s(this, t, e, i ? i.children : [], s, l), !0;
  }
  split(t) {
    let e = new K();
    if (e.breakAfter = this.breakAfter, this.length == 0)
      return e;
    let { i, off: n } = this.childPos(t);
    n && (e.append(this.children[i].split(n), 0), this.children[i].merge(n, this.children[i].length, null, !1, 0, 0), i++);
    for (let s = i; s < this.children.length; s++)
      e.append(this.children[s], 0);
    for (; i > 0 && this.children[i - 1].length == 0; )
      this.children[--i].destroy();
    return this.children.length = i, this.markDirty(), this.length = t, e;
  }
  transferDOM(t) {
    this.dom && (this.markDirty(), t.setDOM(this.dom), t.prevAttrs = this.prevAttrs === void 0 ? this.attrs : this.prevAttrs, this.prevAttrs = void 0, this.dom = null);
  }
  setDeco(t) {
    cn(this.attrs, t) || (this.dom && (this.prevAttrs = this.attrs, this.markDirty()), this.attrs = t);
  }
  append(t, e) {
    er(this, t, e);
  }
  // Only called when building a line view in ContentBuilder
  addLineDeco(t) {
    let e = t.spec.attributes, i = t.spec.class;
    e && (this.attrs = Ki(e, this.attrs || {})), i && (this.attrs = Ki({ class: i }, this.attrs || {}));
  }
  domAtPos(t) {
    return tr(this, t);
  }
  reuseDOM(t) {
    t.nodeName == "DIV" && (this.setDOM(t), this.flags |= 6);
  }
  sync(t, e) {
    var i;
    this.dom ? this.flags & 4 && (Gs(this.dom), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0) : (this.setDOM(document.createElement("div")), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0), this.prevAttrs !== void 0 && (ji(this.dom, this.prevAttrs, this.attrs), this.dom.classList.add("cm-line"), this.prevAttrs = void 0), super.sync(t, e);
    let n = this.dom.lastChild;
    for (; n && I.get(n) instanceof Dt; )
      n = n.lastChild;
    if (!n || !this.length || n.nodeName != "BR" && ((i = I.get(n)) === null || i === void 0 ? void 0 : i.isEditable) == !1 && (!k.ios || !this.children.some((s) => s instanceof Ot))) {
      let s = document.createElement("BR");
      s.cmIgnore = !0, this.dom.appendChild(s);
    }
  }
  measureTextSize() {
    if (this.children.length == 0 || this.length > 20)
      return null;
    let t = 0, e;
    for (let i of this.children) {
      if (!(i instanceof Ot) || /[^ -~]/.test(i.text))
        return null;
      let n = Se(i.dom);
      if (n.length != 1)
        return null;
      t += n[0].width, e = n[0].height;
    }
    return t ? {
      lineHeight: this.dom.getBoundingClientRect().height,
      charWidth: t / this.length,
      textHeight: e
    } : null;
  }
  coordsAt(t, e) {
    let i = ir(this, t, e);
    if (!this.children.length && i && this.parent) {
      let { heightOracle: n } = this.parent.view.viewState, s = i.bottom - i.top;
      if (Math.abs(s - n.lineHeight) < 2 && n.textHeight < s) {
        let l = (s - n.textHeight) / 2;
        return { top: i.top + l, bottom: i.bottom - l, left: i.left, right: i.left };
      }
    }
    return i;
  }
  become(t) {
    return !1;
  }
  covers() {
    return !0;
  }
  static find(t, e) {
    for (let i = 0, n = 0; i < t.children.length; i++) {
      let s = t.children[i], l = n + s.length;
      if (l >= e) {
        if (s instanceof K)
          return s;
        if (l > e)
          break;
      }
      n = l + s.breakAfter;
    }
    return null;
  }
}
class jt extends I {
  constructor(t, e, i) {
    super(), this.widget = t, this.length = e, this.deco = i, this.breakAfter = 0, this.prevWidget = null;
  }
  merge(t, e, i, n, s, l) {
    return i && (!(i instanceof jt) || !this.widget.compare(i.widget) || t > 0 && s <= 0 || e < this.length && l <= 0) ? !1 : (this.length = t + (i ? i.length : 0) + (this.length - e), !0);
  }
  domAtPos(t) {
    return t == 0 ? X.before(this.dom) : X.after(this.dom, t == this.length);
  }
  split(t) {
    let e = this.length - t;
    this.length = t;
    let i = new jt(this.widget, e, this.deco);
    return i.breakAfter = this.breakAfter, i;
  }
  get children() {
    return fn;
  }
  sync(t) {
    (!this.dom || !this.widget.updateDOM(this.dom, t)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(t)), this.dom.contentEditable = "false");
  }
  get overrideDOMText() {
    return this.parent ? this.parent.view.state.doc.slice(this.posAtStart, this.posAtEnd) : R.empty;
  }
  domBoundsAround() {
    return null;
  }
  become(t) {
    return t instanceof jt && t.widget.constructor == this.widget.constructor ? (t.widget.compare(this.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = t.widget, this.length = t.length, this.deco = t.deco, this.breakAfter = t.breakAfter, !0) : !1;
  }
  ignoreMutation() {
    return !0;
  }
  ignoreEvent(t) {
    return this.widget.ignoreEvent(t);
  }
  get isEditable() {
    return !1;
  }
  get isWidget() {
    return !0;
  }
  coordsAt(t, e) {
    return this.widget.coordsAt(this.dom, t, e);
  }
  destroy() {
    super.destroy(), this.dom && this.widget.destroy(this.dom);
  }
  covers(t) {
    let { startSide: e, endSide: i } = this.deco;
    return e == i ? !1 : t < 0 ? e < 0 : i > 0;
  }
}
class De {
  /**
  Compare this instance to another instance of the same type.
  (TypeScript can't express this, but only instances of the same
  specific class will be passed to this method.) This is used to
  avoid redrawing widgets when they are replaced by a new
  decoration of the same type. The default implementation just
  returns `false`, which will cause new instances of the widget to
  always be redrawn.
  */
  eq(t) {
    return !1;
  }
  /**
  Update a DOM element created by a widget of the same type (but
  different, non-`eq` content) to reflect this widget. May return
  true to indicate that it could update, false to indicate it
  couldn't (in which case the widget will be redrawn). The default
  implementation just returns false.
  */
  updateDOM(t, e) {
    return !1;
  }
  /**
  @internal
  */
  compare(t) {
    return this == t || this.constructor == t.constructor && this.eq(t);
  }
  /**
  The estimated height this widget will have, to be used when
  estimating the height of content that hasn't been drawn. May
  return -1 to indicate you don't know. The default implementation
  returns -1.
  */
  get estimatedHeight() {
    return -1;
  }
  /**
  For inline widgets that are displayed inline (as opposed to
  `inline-block`) and introduce line breaks (through `<br>` tags
  or textual newlines), this must indicate the amount of line
  breaks they introduce. Defaults to 0.
  */
  get lineBreaks() {
    return 0;
  }
  /**
  Can be used to configure which kinds of events inside the widget
  should be ignored by the editor. The default is to ignore all
  events.
  */
  ignoreEvent(t) {
    return !0;
  }
  /**
  Override the way screen coordinates for positions at/in the
  widget are found. `pos` will be the offset into the widget, and
  `side` the side of the position that is being queried—less than
  zero for before, greater than zero for after, and zero for
  directly at that position.
  */
  coordsAt(t, e, i) {
    return null;
  }
  /**
  @internal
  */
  get isHidden() {
    return !1;
  }
  /**
  This is called when the an instance of the widget is removed
  from the editor view.
  */
  destroy(t) {
  }
}
var Ct = /* @__PURE__ */ function(r) {
  return r[r.Text = 0] = "Text", r[r.WidgetBefore = 1] = "WidgetBefore", r[r.WidgetAfter = 2] = "WidgetAfter", r[r.WidgetRange = 3] = "WidgetRange", r;
}(Ct || (Ct = {}));
class H extends ee {
  constructor(t, e, i, n) {
    super(), this.startSide = t, this.endSide = e, this.widget = i, this.spec = n;
  }
  /**
  @internal
  */
  get heightRelevant() {
    return !1;
  }
  /**
  Create a mark decoration, which influences the styling of the
  content in its range. Nested mark decorations will cause nested
  DOM elements to be created. Nesting order is determined by
  precedence of the [facet](https://codemirror.net/6/docs/ref/#view.EditorView^decorations), with
  the higher-precedence decorations creating the inner DOM nodes.
  Such elements are split on line boundaries and on the boundaries
  of lower-precedence decorations.
  */
  static mark(t) {
    return new Te(t);
  }
  /**
  Create a widget decoration, which displays a DOM element at the
  given position.
  */
  static widget(t) {
    let e = Math.max(-1e4, Math.min(1e4, t.side || 0)), i = !!t.block;
    return e += i && !t.inlineOrder ? e > 0 ? 3e8 : -4e8 : e > 0 ? 1e8 : -1e8, new Ht(t, e, e, i, t.widget || null, !1);
  }
  /**
  Create a replace decoration which replaces the given range with
  a widget, or simply hides it.
  */
  static replace(t) {
    let e = !!t.block, i, n;
    if (t.isBlockGap)
      i = -5e8, n = 4e8;
    else {
      let { start: s, end: l } = nr(t, e);
      i = (s ? e ? -3e8 : -1 : 5e8) - 1, n = (l ? e ? 2e8 : 1 : -6e8) + 1;
    }
    return new Ht(t, i, n, e, t.widget || null, !0);
  }
  /**
  Create a line decoration, which can add DOM attributes to the
  line starting at the given position.
  */
  static line(t) {
    return new Pe(t);
  }
  /**
  Build a [`DecorationSet`](https://codemirror.net/6/docs/ref/#view.DecorationSet) from the given
  decorated range or ranges. If the ranges aren't already sorted,
  pass `true` for `sort` to make the library sort them for you.
  */
  static set(t, e = !1) {
    return W.of(t, e);
  }
  /**
  @internal
  */
  hasHeight() {
    return this.widget ? this.widget.estimatedHeight > -1 : !1;
  }
}
H.none = W.empty;
class Te extends H {
  constructor(t) {
    let { start: e, end: i } = nr(t);
    super(e ? -1 : 5e8, i ? 1 : -6e8, null, t), this.tagName = t.tagName || "span", this.class = t.class || "", this.attrs = t.attributes || null;
  }
  eq(t) {
    var e, i;
    return this == t || t instanceof Te && this.tagName == t.tagName && (this.class || ((e = this.attrs) === null || e === void 0 ? void 0 : e.class)) == (t.class || ((i = t.attrs) === null || i === void 0 ? void 0 : i.class)) && cn(this.attrs, t.attrs, "class");
  }
  range(t, e = t) {
    if (t >= e)
      throw new RangeError("Mark decorations may not be empty");
    return super.range(t, e);
  }
}
Te.prototype.point = !1;
class Pe extends H {
  constructor(t) {
    super(-2e8, -2e8, null, t);
  }
  eq(t) {
    return t instanceof Pe && this.spec.class == t.spec.class && cn(this.spec.attributes, t.spec.attributes);
  }
  range(t, e = t) {
    if (e != t)
      throw new RangeError("Line decoration ranges must be zero-length");
    return super.range(t, e);
  }
}
Pe.prototype.mapMode = nt.TrackBefore;
Pe.prototype.point = !0;
class Ht extends H {
  constructor(t, e, i, n, s, l) {
    super(e, i, s, t), this.block = n, this.isReplace = l, this.mapMode = n ? e <= 0 ? nt.TrackBefore : nt.TrackAfter : nt.TrackDel;
  }
  // Only relevant when this.block == true
  get type() {
    return this.startSide != this.endSide ? Ct.WidgetRange : this.startSide <= 0 ? Ct.WidgetBefore : Ct.WidgetAfter;
  }
  get heightRelevant() {
    return this.block || !!this.widget && (this.widget.estimatedHeight >= 5 || this.widget.lineBreaks > 0);
  }
  eq(t) {
    return t instanceof Ht && vl(this.widget, t.widget) && this.block == t.block && this.startSide == t.startSide && this.endSide == t.endSide;
  }
  range(t, e = t) {
    if (this.isReplace && (t > e || t == e && this.startSide > 0 && this.endSide <= 0))
      throw new RangeError("Invalid range for replacement decoration");
    if (!this.isReplace && e != t)
      throw new RangeError("Widget decorations can only have zero-length ranges");
    return super.range(t, e);
  }
}
Ht.prototype.point = !0;
function nr(r, t = !1) {
  let { inclusiveStart: e, inclusiveEnd: i } = r;
  return e == null && (e = r.inclusive), i == null && (i = r.inclusive), { start: e ?? t, end: i ?? t };
}
function vl(r, t) {
  return r == t || !!(r && t && r.compare(t));
}
function $i(r, t, e, i = 0) {
  let n = e.length - 1;
  n >= 0 && e[n] + i >= r ? e[n] = Math.max(e[n], t) : e.push(r, t);
}
class xe {
  constructor(t, e, i, n) {
    this.doc = t, this.pos = e, this.end = i, this.disallowBlockEffectsFor = n, this.content = [], this.curLine = null, this.breakAtStart = 0, this.pendingBuffer = 0, this.bufferMarks = [], this.atCursorPos = !0, this.openStart = -1, this.openEnd = -1, this.text = "", this.textOff = 0, this.cursor = t.iter(), this.skip = e;
  }
  posCovered() {
    if (this.content.length == 0)
      return !this.breakAtStart && this.doc.lineAt(this.pos).from != this.pos;
    let t = this.content[this.content.length - 1];
    return !(t.breakAfter || t instanceof jt && t.deco.endSide < 0);
  }
  getLine() {
    return this.curLine || (this.content.push(this.curLine = new K()), this.atCursorPos = !0), this.curLine;
  }
  flushBuffer(t = this.bufferMarks) {
    this.pendingBuffer && (this.curLine.append(Le(new se(-1), t), t.length), this.pendingBuffer = 0);
  }
  addBlockWidget(t) {
    this.flushBuffer(), this.curLine = null, this.content.push(t);
  }
  finish(t) {
    this.pendingBuffer && t <= this.bufferMarks.length ? this.flushBuffer() : this.pendingBuffer = 0, !t && !this.posCovered() && this.getLine();
  }
  buildText(t, e, i) {
    for (; t > 0; ) {
      if (this.textOff == this.text.length) {
        let { value: s, lineBreak: l, done: o } = this.cursor.next(this.skip);
        if (this.skip = 0, o)
          throw new Error("Ran out of text content when drawing inline views");
        if (l) {
          this.posCovered() || this.getLine(), this.content.length ? this.content[this.content.length - 1].breakAfter = 1 : this.breakAtStart = 1, this.flushBuffer(), this.curLine = null, this.atCursorPos = !0, t--;
          continue;
        } else
          this.text = s, this.textOff = 0;
      }
      let n = Math.min(
        this.text.length - this.textOff,
        t,
        512
        /* T.Chunk */
      );
      this.flushBuffer(e.slice(e.length - i)), this.getLine().append(Le(new Ot(this.text.slice(this.textOff, this.textOff + n)), e), i), this.atCursorPos = !0, this.textOff += n, t -= n, i = 0;
    }
  }
  span(t, e, i, n) {
    this.buildText(e - t, i, n), this.pos = e, this.openStart < 0 && (this.openStart = n);
  }
  point(t, e, i, n, s, l) {
    if (this.disallowBlockEffectsFor[l] && i instanceof Ht) {
      if (i.block)
        throw new RangeError("Block decorations may not be specified via plugins");
      if (e > this.doc.lineAt(this.pos).to)
        throw new RangeError("Decorations that replace line breaks may not be specified via plugins");
    }
    let o = e - t;
    if (i instanceof Ht)
      if (i.block)
        i.startSide > 0 && !this.posCovered() && this.getLine(), this.addBlockWidget(new jt(i.widget || new Vn("div"), o, i));
      else {
        let h = Kt.create(i.widget || new Vn("span"), o, o ? 0 : i.startSide), a = this.atCursorPos && !h.isEditable && s <= n.length && (t < e || i.startSide > 0), f = !h.isEditable && (t < e || s > n.length || i.startSide <= 0), c = this.getLine();
        this.pendingBuffer == 2 && !a && !h.isEditable && (this.pendingBuffer = 0), this.flushBuffer(n), a && (c.append(Le(new se(1), n), s), s = n.length + Math.max(0, s - n.length)), c.append(Le(h, n), s), this.atCursorPos = f, this.pendingBuffer = f ? t < e || s > n.length ? 1 : 2 : 0, this.pendingBuffer && (this.bufferMarks = n.slice());
      }
    else
      this.doc.lineAt(this.pos).from == this.pos && this.getLine().addLineDeco(i);
    o && (this.textOff + o <= this.text.length ? this.textOff += o : (this.skip += o - (this.text.length - this.textOff), this.text = "", this.textOff = 0), this.pos = e), this.openStart < 0 && (this.openStart = s);
  }
  static build(t, e, i, n, s) {
    let l = new xe(t, e, i, s);
    return l.openEnd = W.spans(n, e, i, l), l.openStart < 0 && (l.openStart = l.openEnd), l.finish(l.openEnd), l;
  }
}
function Le(r, t) {
  for (let e of t)
    r = new Dt(e, [r], r.length);
  return r;
}
class Vn extends De {
  constructor(t) {
    super(), this.tag = t;
  }
  eq(t) {
    return t.tag == this.tag;
  }
  toDOM() {
    return document.createElement(this.tag);
  }
  updateDOM(t) {
    return t.nodeName.toLowerCase() == this.tag;
  }
  get isHidden() {
    return !0;
  }
}
const sr = /* @__PURE__ */ O.define(), rr = /* @__PURE__ */ O.define(), lr = /* @__PURE__ */ O.define(), or = /* @__PURE__ */ O.define(), Yi = /* @__PURE__ */ O.define(), hr = /* @__PURE__ */ O.define(), ar = /* @__PURE__ */ O.define(), fr = /* @__PURE__ */ O.define({
  combine: (r) => r.some((t) => t)
}), Sl = /* @__PURE__ */ O.define({
  combine: (r) => r.some((t) => t)
});
class ni {
  constructor(t, e = "nearest", i = "nearest", n = 5, s = 5) {
    this.range = t, this.y = e, this.x = i, this.yMargin = n, this.xMargin = s;
  }
  map(t) {
    return t.empty ? this : new ni(this.range.map(t), this.y, this.x, this.yMargin, this.xMargin);
  }
}
const Wn = /* @__PURE__ */ q.define({ map: (r, t) => r.map(t) });
function $t(r, t, e) {
  let i = r.facet(or);
  i.length ? i[0](t) : window.onerror ? window.onerror(String(t), e, void 0, void 0, t) : e ? console.error(e + ":", t) : console.error(t);
}
const ci = /* @__PURE__ */ O.define({ combine: (r) => r.length ? r[0] : !0 });
let Al = 0;
const ge = /* @__PURE__ */ O.define();
class Tt {
  constructor(t, e, i, n, s) {
    this.id = t, this.create = e, this.domEventHandlers = i, this.domEventObservers = n, this.extension = s(this);
  }
  /**
  Define a plugin from a constructor function that creates the
  plugin's value, given an editor view.
  */
  static define(t, e) {
    const { eventHandlers: i, eventObservers: n, provide: s, decorations: l } = e || {};
    return new Tt(Al++, t, i, n, (o) => {
      let h = [ge.of(o)];
      return l && h.push(Ce.of((a) => {
        let f = a.plugin(o);
        return f ? l(f) : H.none;
      })), s && h.push(s(o)), h;
    });
  }
  /**
  Create a plugin for a class whose constructor takes a single
  editor view as argument.
  */
  static fromClass(t, e) {
    return Tt.define((i) => new t(i), e);
  }
}
class mi {
  constructor(t) {
    this.spec = t, this.mustUpdate = null, this.value = null;
  }
  update(t) {
    if (this.value) {
      if (this.mustUpdate) {
        let e = this.mustUpdate;
        if (this.mustUpdate = null, this.value.update)
          try {
            this.value.update(e);
          } catch (i) {
            if ($t(e.state, i, "CodeMirror plugin crashed"), this.value.destroy)
              try {
                this.value.destroy();
              } catch {
              }
            this.deactivate();
          }
      }
    } else if (this.spec)
      try {
        this.value = this.spec.create(t);
      } catch (e) {
        $t(t.state, e, "CodeMirror plugin crashed"), this.deactivate();
      }
    return this;
  }
  destroy(t) {
    var e;
    if (!((e = this.value) === null || e === void 0) && e.destroy)
      try {
        this.value.destroy();
      } catch (i) {
        $t(t.state, i, "CodeMirror plugin crashed");
      }
  }
  deactivate() {
    this.spec = this.value = null;
  }
}
const cr = /* @__PURE__ */ O.define(), un = /* @__PURE__ */ O.define(), Ce = /* @__PURE__ */ O.define(), dn = /* @__PURE__ */ O.define(), ur = /* @__PURE__ */ O.define();
function zn(r, t, e) {
  let i = r.state.facet(ur);
  if (!i.length)
    return i;
  let n = i.map((l) => l instanceof Function ? l(r) : l), s = [];
  return W.spans(n, t, e, {
    point() {
    },
    span(l, o, h, a) {
      let f = s;
      for (let c = h.length - 1; c >= 0; c--, a--) {
        let u = h[c].spec.bidiIsolate, d;
        if (u != null)
          if (a > 0 && f.length && (d = f[f.length - 1]).to == l && d.direction == u)
            d.to = o, f = d.inner;
          else {
            let g = { from: l, to: o, direction: u, inner: [] };
            f.push(g), f = g.inner;
          }
      }
    }
  }), s;
}
const dr = /* @__PURE__ */ O.define();
function gr(r) {
  let t = 0, e = 0, i = 0, n = 0;
  for (let s of r.state.facet(dr)) {
    let l = s(r);
    l && (l.left != null && (t = Math.max(t, l.left)), l.right != null && (e = Math.max(e, l.right)), l.top != null && (i = Math.max(i, l.top)), l.bottom != null && (n = Math.max(n, l.bottom)));
  }
  return { left: t, right: e, top: i, bottom: n };
}
const pe = /* @__PURE__ */ O.define();
class rt {
  constructor(t, e, i, n) {
    this.fromA = t, this.toA = e, this.fromB = i, this.toB = n;
  }
  join(t) {
    return new rt(Math.min(this.fromA, t.fromA), Math.max(this.toA, t.toA), Math.min(this.fromB, t.fromB), Math.max(this.toB, t.toB));
  }
  addToSet(t) {
    let e = t.length, i = this;
    for (; e > 0; e--) {
      let n = t[e - 1];
      if (!(n.fromA > i.toA)) {
        if (n.toA < i.fromA)
          break;
        i = i.join(n), t.splice(e - 1, 1);
      }
    }
    return t.splice(e, 0, i), t;
  }
  static extendWithRanges(t, e) {
    if (e.length == 0)
      return t;
    let i = [];
    for (let n = 0, s = 0, l = 0, o = 0; ; n++) {
      let h = n == t.length ? null : t[n], a = l - o, f = h ? h.fromB : 1e9;
      for (; s < e.length && e[s] < f; ) {
        let c = e[s], u = e[s + 1], d = Math.max(o, c), g = Math.min(f, u);
        if (d <= g && new rt(d + a, g + a, d, g).addToSet(i), u > f)
          break;
        s += 2;
      }
      if (!h)
        return i;
      new rt(h.fromA, h.toA, h.fromB, h.toB).addToSet(i), l = h.toA, o = h.toB;
    }
  }
}
class si {
  constructor(t, e, i) {
    this.view = t, this.state = e, this.transactions = i, this.flags = 0, this.startState = t.state, this.changes = j.empty(this.startState.doc.length);
    for (let s of i)
      this.changes = this.changes.compose(s.changes);
    let n = [];
    this.changes.iterChangedRanges((s, l, o, h) => n.push(new rt(s, l, o, h))), this.changedRanges = n;
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new si(t, e, i);
  }
  /**
  Tells you whether the [viewport](https://codemirror.net/6/docs/ref/#view.EditorView.viewport) or
  [visible ranges](https://codemirror.net/6/docs/ref/#view.EditorView.visibleRanges) changed in this
  update.
  */
  get viewportChanged() {
    return (this.flags & 4) > 0;
  }
  /**
  Indicates whether the height of a block element in the editor
  changed in this update.
  */
  get heightChanged() {
    return (this.flags & 2) > 0;
  }
  /**
  Returns true when the document was modified or the size of the
  editor, or elements within the editor, changed.
  */
  get geometryChanged() {
    return this.docChanged || (this.flags & 10) > 0;
  }
  /**
  True when this update indicates a focus change.
  */
  get focusChanged() {
    return (this.flags & 1) > 0;
  }
  /**
  Whether the document changed in this update.
  */
  get docChanged() {
    return !this.changes.empty;
  }
  /**
  Whether the selection was explicitly set in this update.
  */
  get selectionSet() {
    return this.transactions.some((t) => t.selection);
  }
  /**
  @internal
  */
  get empty() {
    return this.flags == 0 && this.transactions.length == 0;
  }
}
var Q = /* @__PURE__ */ function(r) {
  return r[r.LTR = 0] = "LTR", r[r.RTL = 1] = "RTL", r;
}(Q || (Q = {}));
const Me = Q.LTR, pr = Q.RTL;
function mr(r) {
  let t = [];
  for (let e = 0; e < r.length; e++)
    t.push(1 << +r[e]);
  return t;
}
const Cl = /* @__PURE__ */ mr("88888888888888888888888888888888888666888888787833333333337888888000000000000000000000000008888880000000000000000000000000088888888888888888888888888888888888887866668888088888663380888308888800000000000000000000000800000000000000000000000000000008"), Ml = /* @__PURE__ */ mr("4444448826627288999999999992222222222222222222222222222222222222222222222229999999999999999999994444444444644222822222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222999999949999999229989999223333333333"), Gi = /* @__PURE__ */ Object.create(null), ct = [];
for (let r of ["()", "[]", "{}"]) {
  let t = /* @__PURE__ */ r.charCodeAt(0), e = /* @__PURE__ */ r.charCodeAt(1);
  Gi[t] = e, Gi[e] = -t;
}
function Ol(r) {
  return r <= 247 ? Cl[r] : 1424 <= r && r <= 1524 ? 2 : 1536 <= r && r <= 1785 ? Ml[r - 1536] : 1774 <= r && r <= 2220 ? 4 : 8192 <= r && r <= 8203 ? 256 : 64336 <= r && r <= 65023 ? 4 : r == 8204 ? 256 : 1;
}
const Dl = /[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac\ufb50-\ufdff]/;
class Nt {
  /**
  The direction of this span.
  */
  get dir() {
    return this.level % 2 ? pr : Me;
  }
  /**
  @internal
  */
  constructor(t, e, i) {
    this.from = t, this.to = e, this.level = i;
  }
  /**
  @internal
  */
  side(t, e) {
    return this.dir == e == t ? this.to : this.from;
  }
  /**
  @internal
  */
  static find(t, e, i, n) {
    let s = -1;
    for (let l = 0; l < t.length; l++) {
      let o = t[l];
      if (o.from <= e && o.to >= e) {
        if (o.level == i)
          return l;
        (s < 0 || (n != 0 ? n < 0 ? o.from < e : o.to > e : t[s].level > o.level)) && (s = l);
      }
    }
    if (s < 0)
      throw new RangeError("Index out of range");
    return s;
  }
}
function yr(r, t) {
  if (r.length != t.length)
    return !1;
  for (let e = 0; e < r.length; e++) {
    let i = r[e], n = t[e];
    if (i.from != n.from || i.to != n.to || i.direction != n.direction || !yr(i.inner, n.inner))
      return !1;
  }
  return !0;
}
const L = [];
function Tl(r, t, e, i, n) {
  for (let s = 0; s <= i.length; s++) {
    let l = s ? i[s - 1].to : t, o = s < i.length ? i[s].from : e, h = s ? 256 : n;
    for (let a = l, f = h, c = h; a < o; a++) {
      let u = Ol(r.charCodeAt(a));
      u == 512 ? u = f : u == 8 && c == 4 && (u = 16), L[a] = u == 4 ? 2 : u, u & 7 && (c = u), f = u;
    }
    for (let a = l, f = h, c = h; a < o; a++) {
      let u = L[a];
      if (u == 128)
        a < o - 1 && f == L[a + 1] && f & 24 ? u = L[a] = f : L[a] = 256;
      else if (u == 64) {
        let d = a + 1;
        for (; d < o && L[d] == 64; )
          d++;
        let g = a && f == 8 || d < e && L[d] == 8 ? c == 1 ? 1 : 8 : 256;
        for (let m = a; m < d; m++)
          L[m] = g;
        a = d - 1;
      } else
        u == 8 && c == 1 && (L[a] = 1);
      f = u, u & 7 && (c = u);
    }
  }
}
function Pl(r, t, e, i, n) {
  let s = n == 1 ? 2 : 1;
  for (let l = 0, o = 0, h = 0; l <= i.length; l++) {
    let a = l ? i[l - 1].to : t, f = l < i.length ? i[l].from : e;
    for (let c = a, u, d, g; c < f; c++)
      if (d = Gi[u = r.charCodeAt(c)])
        if (d < 0) {
          for (let m = o - 3; m >= 0; m -= 3)
            if (ct[m + 1] == -d) {
              let p = ct[m + 2], y = p & 2 ? n : p & 4 ? p & 1 ? s : n : 0;
              y && (L[c] = L[ct[m]] = y), o = m;
              break;
            }
        } else {
          if (ct.length == 189)
            break;
          ct[o++] = c, ct[o++] = u, ct[o++] = h;
        }
      else if ((g = L[c]) == 2 || g == 1) {
        let m = g == n;
        h = m ? 0 : 1;
        for (let p = o - 3; p >= 0; p -= 3) {
          let y = ct[p + 2];
          if (y & 2)
            break;
          if (m)
            ct[p + 2] |= 2;
          else {
            if (y & 4)
              break;
            ct[p + 2] |= 4;
          }
        }
      }
  }
}
function Bl(r, t, e, i) {
  for (let n = 0, s = i; n <= e.length; n++) {
    let l = n ? e[n - 1].to : r, o = n < e.length ? e[n].from : t;
    for (let h = l; h < o; ) {
      let a = L[h];
      if (a == 256) {
        let f = h + 1;
        for (; ; )
          if (f == o) {
            if (n == e.length)
              break;
            f = e[n++].to, o = n < e.length ? e[n].from : t;
          } else if (L[f] == 256)
            f++;
          else
            break;
        let c = s == 1, u = (f < t ? L[f] : i) == 1, d = c == u ? c ? 1 : 2 : i;
        for (let g = f, m = n, p = m ? e[m - 1].to : r; g > h; )
          g == p && (g = e[--m].from, p = m ? e[m - 1].to : r), L[--g] = d;
        h = f;
      } else
        s = a, h++;
    }
  }
}
function Ji(r, t, e, i, n, s, l) {
  let o = i % 2 ? 2 : 1;
  if (i % 2 == n % 2)
    for (let h = t, a = 0; h < e; ) {
      let f = !0, c = !1;
      if (a == s.length || h < s[a].from) {
        let m = L[h];
        m != o && (f = !1, c = m == 16);
      }
      let u = !f && o == 1 ? [] : null, d = f ? i : i + 1, g = h;
      t:
        for (; ; )
          if (a < s.length && g == s[a].from) {
            if (c)
              break t;
            let m = s[a];
            if (!f)
              for (let p = m.to, y = a + 1; ; ) {
                if (p == e)
                  break t;
                if (y < s.length && s[y].from == p)
                  p = s[y++].to;
                else {
                  if (L[p] == o)
                    break t;
                  break;
                }
              }
            if (a++, u)
              u.push(m);
            else {
              m.from > h && l.push(new Nt(h, m.from, d));
              let p = m.direction == Me != !(d % 2);
              Ui(r, p ? i + 1 : i, n, m.inner, m.from, m.to, l), h = m.to;
            }
            g = m.to;
          } else {
            if (g == e || (f ? L[g] != o : L[g] == o))
              break;
            g++;
          }
      u ? Ji(r, h, g, i + 1, n, u, l) : h < g && l.push(new Nt(h, g, d)), h = g;
    }
  else
    for (let h = e, a = s.length; h > t; ) {
      let f = !0, c = !1;
      if (!a || h > s[a - 1].to) {
        let m = L[h - 1];
        m != o && (f = !1, c = m == 16);
      }
      let u = !f && o == 1 ? [] : null, d = f ? i : i + 1, g = h;
      t:
        for (; ; )
          if (a && g == s[a - 1].to) {
            if (c)
              break t;
            let m = s[--a];
            if (!f)
              for (let p = m.from, y = a; ; ) {
                if (p == t)
                  break t;
                if (y && s[y - 1].to == p)
                  p = s[--y].from;
                else {
                  if (L[p - 1] == o)
                    break t;
                  break;
                }
              }
            if (u)
              u.push(m);
            else {
              m.to < h && l.push(new Nt(m.to, h, d));
              let p = m.direction == Me != !(d % 2);
              Ui(r, p ? i + 1 : i, n, m.inner, m.from, m.to, l), h = m.from;
            }
            g = m.from;
          } else {
            if (g == t || (f ? L[g - 1] != o : L[g - 1] == o))
              break;
            g--;
          }
      u ? Ji(r, g, h, i + 1, n, u, l) : g < h && l.push(new Nt(g, h, d)), h = g;
    }
}
function Ui(r, t, e, i, n, s, l) {
  let o = t % 2 ? 2 : 1;
  Tl(r, n, s, i, o), Pl(r, n, s, i, o), Bl(n, s, i, o), Ji(r, n, s, t, e, i, l);
}
function Rl(r, t, e) {
  if (!r)
    return [new Nt(0, 0, t == pr ? 1 : 0)];
  if (t == Me && !e.length && !Dl.test(r))
    return wr(r.length);
  if (e.length)
    for (; r.length > L.length; )
      L[L.length] = 256;
  let i = [], n = t == Me ? 0 : 1;
  return Ui(r, n, n, e, 0, r.length, i), i;
}
function wr(r) {
  return [new Nt(0, r, 0)];
}
let br = "";
function El(r, t, e, i, n) {
  var s;
  let l = i.head - r.from, o = -1;
  if (l == 0) {
    if (!n || !r.length)
      return null;
    t[0].level != e && (l = t[0].side(!1, e), o = 0);
  } else if (l == r.length) {
    if (n)
      return null;
    let u = t[t.length - 1];
    u.level != e && (l = u.side(!0, e), o = t.length - 1);
  }
  o < 0 && (o = Nt.find(t, l, (s = i.bidiLevel) !== null && s !== void 0 ? s : -1, i.assoc));
  let h = t[o];
  l == h.side(n, e) && (h = t[o += n ? 1 : -1], l = h.side(!n, e));
  let a = n == (h.dir == e), f = kt(r.text, l, a);
  if (br = r.text.slice(Math.min(l, f), Math.max(l, f)), f != h.side(n, e))
    return x.cursor(f + r.from, a ? -1 : 1, h.level);
  let c = o == (n ? t.length - 1 : 0) ? null : t[o + (n ? 1 : -1)];
  return !c && h.level != e ? x.cursor(n ? r.to : r.from, n ? -1 : 1, e) : c && c.level < h.level ? x.cursor(c.side(!n, e) + r.from, n ? 1 : -1, c.level) : x.cursor(f + r.from, n ? -1 : 1, h.level);
}
class qn extends I {
  get length() {
    return this.view.state.doc.length;
  }
  constructor(t) {
    super(), this.view = t, this.decorations = [], this.dynamicDecorationMap = [], this.domChanged = null, this.hasComposition = null, this.markedForComposition = /* @__PURE__ */ new Set(), this.minWidth = 0, this.minWidthFrom = 0, this.minWidthTo = 0, this.impreciseAnchor = null, this.impreciseHead = null, this.forceSelection = !1, this.lastUpdate = Date.now(), this.setDOM(t.contentDOM), this.children = [new K()], this.children[0].setParent(this), this.updateDeco(), this.updateInner([new rt(0, 0, 0, t.state.doc.length)], 0, null);
  }
  // Update the document view to a given state.
  update(t) {
    var e;
    let i = t.changedRanges;
    this.minWidth > 0 && i.length && (i.every(({ fromA: a, toA: f }) => f < this.minWidthFrom || a > this.minWidthTo) ? (this.minWidthFrom = t.changes.mapPos(this.minWidthFrom, 1), this.minWidthTo = t.changes.mapPos(this.minWidthTo, 1)) : this.minWidth = this.minWidthFrom = this.minWidthTo = 0);
    let n = -1;
    this.view.inputState.composing >= 0 && (!((e = this.domChanged) === null || e === void 0) && e.newSel ? n = this.domChanged.newSel.head : !Wl(t.changes, this.hasComposition) && !t.selectionSet && (n = t.state.selection.main.head));
    let s = n > -1 ? Nl(this.view, t.changes, n) : null;
    if (this.domChanged = null, this.hasComposition) {
      this.markedForComposition.clear();
      let { from: a, to: f } = this.hasComposition;
      i = new rt(a, f, t.changes.mapPos(a, -1), t.changes.mapPos(f, 1)).addToSet(i.slice());
    }
    this.hasComposition = s ? { from: s.range.fromB, to: s.range.toB } : null, (k.ie || k.chrome) && !s && t && t.state.doc.lines != t.startState.doc.lines && (this.forceSelection = !0);
    let l = this.decorations, o = this.updateDeco(), h = Fl(l, o, t.changes);
    return i = rt.extendWithRanges(i, h), !(this.flags & 7) && i.length == 0 ? !1 : (this.updateInner(i, t.startState.doc.length, s), t.transactions.length && (this.lastUpdate = Date.now()), !0);
  }
  // Used by update and the constructor do perform the actual DOM
  // update
  updateInner(t, e, i) {
    this.view.viewState.mustMeasureContent = !0, this.updateChildren(t, e, i);
    let { observer: n } = this.view;
    n.ignore(() => {
      this.dom.style.height = this.view.viewState.contentHeight / this.view.scaleY + "px", this.dom.style.flexBasis = this.minWidth ? this.minWidth + "px" : "";
      let l = k.chrome || k.ios ? { node: n.selectionRange.focusNode, written: !1 } : void 0;
      this.sync(this.view, l), this.flags &= -8, l && (l.written || n.selectionRange.focusNode != l.node) && (this.forceSelection = !0), this.dom.style.height = "";
    }), this.markedForComposition.forEach(
      (l) => l.flags &= -9
      /* ViewFlag.Composition */
    );
    let s = [];
    if (this.view.viewport.from || this.view.viewport.to < this.view.state.doc.length)
      for (let l of this.children)
        l instanceof jt && l.widget instanceof Kn && s.push(l.dom);
    n.updateGaps(s);
  }
  updateChildren(t, e, i) {
    let n = i ? i.range.addToSet(t.slice()) : t, s = this.childCursor(e);
    for (let l = n.length - 1; ; l--) {
      let o = l >= 0 ? n[l] : null;
      if (!o)
        break;
      let { fromA: h, toA: a, fromB: f, toB: c } = o, u, d, g, m;
      if (i && i.range.fromB < c && i.range.toB > f) {
        let b = xe.build(this.view.state.doc, f, i.range.fromB, this.decorations, this.dynamicDecorationMap), A = xe.build(this.view.state.doc, i.range.toB, c, this.decorations, this.dynamicDecorationMap);
        d = b.breakAtStart, g = b.openStart, m = A.openEnd;
        let v = this.compositionView(i);
        A.breakAtStart ? v.breakAfter = 1 : A.content.length && v.merge(v.length, v.length, A.content[0], !1, A.openStart, 0) && (v.breakAfter = A.content[0].breakAfter, A.content.shift()), b.content.length && v.merge(0, 0, b.content[b.content.length - 1], !0, 0, b.openEnd) && b.content.pop(), u = b.content.concat(v).concat(A.content);
      } else
        ({ content: u, breakAtStart: d, openStart: g, openEnd: m } = xe.build(this.view.state.doc, f, c, this.decorations, this.dynamicDecorationMap));
      let { i: p, off: y } = s.findPos(a, 1), { i: S, off: C } = s.findPos(h, -1);
      Xs(this, S, C, p, y, u, d, g, m);
    }
    i && this.fixCompositionDOM(i);
  }
  compositionView(t) {
    let e = new Ot(t.text.nodeValue);
    e.flags |= 8;
    for (let { deco: n } of t.marks)
      e = new Dt(n, [e], e.length);
    let i = new K();
    return i.append(e, 0), i;
  }
  fixCompositionDOM(t) {
    let e = (s, l) => {
      l.flags |= 8 | (l.children.some(
        (h) => h.flags & 7
        /* ViewFlag.Dirty */
      ) ? 1 : 0), this.markedForComposition.add(l);
      let o = I.get(s);
      o && o != l && (o.dom = null), l.setDOM(s);
    }, i = this.childPos(t.range.fromB, 1), n = this.children[i.i];
    e(t.line, n);
    for (let s = t.marks.length - 1; s >= -1; s--)
      i = n.childPos(i.off, 1), n = n.children[i.i], e(s >= 0 ? t.marks[s].node : t.text, n);
  }
  // Sync the DOM selection to this.state.selection
  updateSelection(t = !1, e = !1) {
    (t || !this.view.observer.selectionRange.focusNode) && this.view.observer.readSelectionRange();
    let i = this.view.root.activeElement, n = i == this.dom, s = !n && Je(this.dom, this.view.observer.selectionRange) && !(i && this.dom.contains(i));
    if (!(n || e || s))
      return;
    let l = this.forceSelection;
    this.forceSelection = !1;
    let o = this.view.state.selection.main, h = this.moveToLine(this.domAtPos(o.anchor)), a = o.empty ? h : this.moveToLine(this.domAtPos(o.head));
    if (k.gecko && o.empty && !this.hasComposition && Ll(h)) {
      let c = document.createTextNode("");
      this.view.observer.ignore(() => h.node.insertBefore(c, h.node.childNodes[h.offset] || null)), h = a = new X(c, 0), l = !0;
    }
    let f = this.view.observer.selectionRange;
    (l || !f.focusNode || !ii(h.node, h.offset, f.anchorNode, f.anchorOffset) || !ii(a.node, a.offset, f.focusNode, f.focusOffset)) && (this.view.observer.ignore(() => {
      k.android && k.chrome && this.dom.contains(f.focusNode) && Vl(f.focusNode, this.dom) && (this.dom.blur(), this.dom.focus({ preventScroll: !0 }));
      let c = ei(this.view.root);
      if (c)
        if (o.empty) {
          if (k.gecko) {
            let u = Il(h.node, h.offset);
            if (u && u != 3) {
              let d = kr(h.node, h.offset, u == 1 ? 1 : -1);
              d && (h = new X(d.node, d.offset));
            }
          }
          c.collapse(h.node, h.offset), o.bidiLevel != null && c.caretBidiLevel !== void 0 && (c.caretBidiLevel = o.bidiLevel);
        } else if (c.extend) {
          c.collapse(h.node, h.offset);
          try {
            c.extend(a.node, a.offset);
          } catch {
          }
        } else {
          let u = document.createRange();
          o.anchor > o.head && ([h, a] = [a, h]), u.setEnd(a.node, a.offset), u.setStart(h.node, h.offset), c.removeAllRanges(), c.addRange(u);
        }
      s && this.view.root.activeElement == this.dom && (this.dom.blur(), i && i.focus());
    }), this.view.observer.setSelectionRange(h, a)), this.impreciseAnchor = h.precise ? null : new X(f.anchorNode, f.anchorOffset), this.impreciseHead = a.precise ? null : new X(f.focusNode, f.focusOffset);
  }
  enforceCursorAssoc() {
    if (this.hasComposition)
      return;
    let { view: t } = this, e = t.state.selection.main, i = ei(t.root), { anchorNode: n, anchorOffset: s } = t.observer.selectionRange;
    if (!i || !e.empty || !e.assoc || !i.modify)
      return;
    let l = K.find(this, e.head);
    if (!l)
      return;
    let o = l.posAtStart;
    if (e.head == o || e.head == o + l.length)
      return;
    let h = this.coordsAt(e.head, -1), a = this.coordsAt(e.head, 1);
    if (!h || !a || h.bottom > a.top)
      return;
    let f = this.domAtPos(e.head + e.assoc);
    i.collapse(f.node, f.offset), i.modify("move", e.assoc < 0 ? "forward" : "backward", "lineboundary"), t.observer.readSelectionRange();
    let c = t.observer.selectionRange;
    t.docView.posFromDOM(c.anchorNode, c.anchorOffset) != e.from && i.collapse(n, s);
  }
  // If a position is in/near a block widget, move it to a nearby text
  // line, since we don't want the cursor inside a block widget.
  moveToLine(t) {
    let e = this.dom, i;
    if (t.node != e)
      return t;
    for (let n = t.offset; !i && n < e.childNodes.length; n++) {
      let s = I.get(e.childNodes[n]);
      s instanceof K && (i = s.domAtPos(0));
    }
    for (let n = t.offset - 1; !i && n >= 0; n--) {
      let s = I.get(e.childNodes[n]);
      s instanceof K && (i = s.domAtPos(s.length));
    }
    return i ? new X(i.node, i.offset, !0) : t;
  }
  nearest(t) {
    for (let e = t; e; ) {
      let i = I.get(e);
      if (i && i.rootView == this)
        return i;
      e = e.parentNode;
    }
    return null;
  }
  posFromDOM(t, e) {
    let i = this.nearest(t);
    if (!i)
      throw new RangeError("Trying to find position for a DOM position outside of the document");
    return i.localPosFromDOM(t, e) + i.posAtStart;
  }
  domAtPos(t) {
    let { i: e, off: i } = this.childCursor().findPos(t, -1);
    for (; e < this.children.length - 1; ) {
      let n = this.children[e];
      if (i < n.length || n instanceof K)
        break;
      e++, i = 0;
    }
    return this.children[e].domAtPos(i);
  }
  coordsAt(t, e) {
    let i = null, n = 0;
    for (let s = this.length, l = this.children.length - 1; l >= 0; l--) {
      let o = this.children[l], h = s - o.breakAfter, a = h - o.length;
      if (h < t)
        break;
      a <= t && (a < t || o.covers(-1)) && (h > t || o.covers(1)) && (!i || o instanceof K && !(i instanceof K && e >= 0)) && (i = o, n = a), s = a;
    }
    return i ? i.coordsAt(t - n, e) : null;
  }
  coordsForChar(t) {
    let { i: e, off: i } = this.childPos(t, 1), n = this.children[e];
    if (!(n instanceof K))
      return null;
    for (; n.children.length; ) {
      let { i: o, off: h } = n.childPos(i, 1);
      for (; ; o++) {
        if (o == n.children.length)
          return null;
        if ((n = n.children[o]).length)
          break;
      }
      i = h;
    }
    if (!(n instanceof Ot))
      return null;
    let s = kt(n.text, i);
    if (s == i)
      return null;
    let l = Gt(n.dom, i, s).getClientRects();
    return !l.length || l[0].top >= l[0].bottom ? null : l[0];
  }
  measureVisibleLineHeights(t) {
    let e = [], { from: i, to: n } = t, s = this.view.contentDOM.clientWidth, l = s > Math.max(this.view.scrollDOM.clientWidth, this.minWidth) + 1, o = -1, h = this.view.textDirection == Q.LTR;
    for (let a = 0, f = 0; f < this.children.length; f++) {
      let c = this.children[f], u = a + c.length;
      if (u > n)
        break;
      if (a >= i) {
        let d = c.dom.getBoundingClientRect();
        if (e.push(d.height), l) {
          let g = c.dom.lastChild, m = g ? Se(g) : [];
          if (m.length) {
            let p = m[m.length - 1], y = h ? p.right - d.left : d.right - p.left;
            y > o && (o = y, this.minWidth = s, this.minWidthFrom = a, this.minWidthTo = u);
          }
        }
      }
      a = u + c.breakAfter;
    }
    return e;
  }
  textDirectionAt(t) {
    let { i: e } = this.childPos(t, 1);
    return getComputedStyle(this.children[e].dom).direction == "rtl" ? Q.RTL : Q.LTR;
  }
  measureTextSize() {
    for (let s of this.children)
      if (s instanceof K) {
        let l = s.measureTextSize();
        if (l)
          return l;
      }
    let t = document.createElement("div"), e, i, n;
    return t.className = "cm-line", t.style.width = "99999px", t.style.position = "absolute", t.textContent = "abc def ghi jkl mno pqr stu", this.view.observer.ignore(() => {
      this.dom.appendChild(t);
      let s = Se(t.firstChild)[0];
      e = t.getBoundingClientRect().height, i = s ? s.width / 27 : 7, n = s ? s.height : e, t.remove();
    }), { lineHeight: e, charWidth: i, textHeight: n };
  }
  childCursor(t = this.length) {
    let e = this.children.length;
    return e && (t -= this.children[--e].length), new Us(this.children, t, e);
  }
  computeBlockGapDeco() {
    let t = [], e = this.view.viewState;
    for (let i = 0, n = 0; ; n++) {
      let s = n == e.viewports.length ? null : e.viewports[n], l = s ? s.from - 1 : this.length;
      if (l > i) {
        let o = (e.lineBlockAt(l).bottom - e.lineBlockAt(i).top) / this.view.scaleY;
        t.push(H.replace({
          widget: new Kn(o),
          block: !0,
          inclusive: !0,
          isBlockGap: !0
        }).range(i, l));
      }
      if (!s)
        break;
      i = s.to + 1;
    }
    return H.set(t);
  }
  updateDeco() {
    let t = this.view.state.facet(Ce).map((e, i) => (this.dynamicDecorationMap[i] = typeof e == "function") ? e(this.view) : e);
    for (let e = t.length; e < t.length + 3; e++)
      this.dynamicDecorationMap[e] = !1;
    return this.decorations = [
      ...t,
      this.computeBlockGapDeco(),
      this.view.viewState.lineGapDeco
    ];
  }
  scrollIntoView(t) {
    let { range: e } = t, i = this.coordsAt(e.head, e.empty ? e.assoc : e.head > e.anchor ? -1 : 1), n;
    if (!i)
      return;
    !e.empty && (n = this.coordsAt(e.anchor, e.anchor > e.head ? -1 : 1)) && (i = {
      left: Math.min(i.left, n.left),
      top: Math.min(i.top, n.top),
      right: Math.max(i.right, n.right),
      bottom: Math.max(i.bottom, n.bottom)
    });
    let s = gr(this.view), l = {
      left: i.left - s.left,
      top: i.top - s.top,
      right: i.right + s.right,
      bottom: i.bottom + s.bottom
    };
    dl(this.view.scrollDOM, l, e.head < e.anchor ? -1 : 1, t.x, t.y, t.xMargin, t.yMargin, this.view.textDirection == Q.LTR);
  }
}
function Ll(r) {
  return r.node.nodeType == 1 && r.node.firstChild && (r.offset == 0 || r.node.childNodes[r.offset - 1].contentEditable == "false") && (r.offset == r.node.childNodes.length || r.node.childNodes[r.offset].contentEditable == "false");
}
class Kn extends De {
  constructor(t) {
    super(), this.height = t;
  }
  toDOM() {
    let t = document.createElement("div");
    return this.updateDOM(t), t;
  }
  eq(t) {
    return t.height == this.height;
  }
  updateDOM(t) {
    return t.style.height = this.height + "px", !0;
  }
  get estimatedHeight() {
    return this.height;
  }
}
function xr(r, t) {
  let e = r.observer.selectionRange, i = e.focusNode && kr(e.focusNode, e.focusOffset, 0);
  if (!i)
    return null;
  let n = t - i.offset;
  return { from: n, to: n + i.node.nodeValue.length, node: i.node };
}
function Nl(r, t, e) {
  let i = xr(r, e);
  if (!i)
    return null;
  let { node: n, from: s, to: l } = i, o = n.nodeValue;
  if (/[\n\r]/.test(o) || r.state.doc.sliceString(i.from, i.to) != o)
    return null;
  let h = t.invertedDesc, a = new rt(h.mapPos(s), h.mapPos(l), s, l), f = [];
  for (let c = n.parentNode; ; c = c.parentNode) {
    let u = I.get(c);
    if (u instanceof Dt)
      f.push({ node: c, deco: u.mark });
    else {
      if (u instanceof K || c.nodeName == "DIV" && c.parentNode == r.contentDOM)
        return { range: a, text: n, marks: f, line: c };
      if (c != r.contentDOM)
        f.push({ node: c, deco: new Te({
          inclusive: !0,
          attributes: kl(c),
          tagName: c.tagName.toLowerCase()
        }) });
      else
        return null;
    }
  }
}
function kr(r, t, e) {
  if (e <= 0)
    for (let i = r, n = t; ; ) {
      if (i.nodeType == 3)
        return { node: i, offset: n };
      if (i.nodeType == 1 && n > 0)
        i = i.childNodes[n - 1], n = Mt(i);
      else
        break;
    }
  if (e >= 0)
    for (let i = r, n = t; ; ) {
      if (i.nodeType == 3)
        return { node: i, offset: n };
      if (i.nodeType == 1 && n < i.childNodes.length && e >= 0)
        i = i.childNodes[n], n = 0;
      else
        break;
    }
  return null;
}
function Il(r, t) {
  return r.nodeType != 1 ? 0 : (t && r.childNodes[t - 1].contentEditable == "false" ? 1 : 0) | (t < r.childNodes.length && r.childNodes[t].contentEditable == "false" ? 2 : 0);
}
let Hl = class {
  constructor() {
    this.changes = [];
  }
  compareRange(t, e) {
    $i(t, e, this.changes);
  }
  comparePoint(t, e) {
    $i(t, e, this.changes);
  }
};
function Fl(r, t, e) {
  let i = new Hl();
  return W.compare(r, t, e, i), i.changes;
}
function Vl(r, t) {
  for (let e = r; e && e != t; e = e.assignedSlot || e.parentNode)
    if (e.nodeType == 1 && e.contentEditable == "false")
      return !0;
  return !1;
}
function Wl(r, t) {
  let e = !1;
  return t && r.iterChangedRanges((i, n) => {
    i < t.to && n > t.from && (e = !0);
  }), e;
}
function zl(r, t, e = 1) {
  let i = r.charCategorizer(t), n = r.doc.lineAt(t), s = t - n.from;
  if (n.length == 0)
    return x.cursor(t);
  s == 0 ? e = 1 : s == n.length && (e = -1);
  let l = s, o = s;
  e < 0 ? l = kt(n.text, s, !1) : o = kt(n.text, s);
  let h = i(n.text.slice(l, o));
  for (; l > 0; ) {
    let a = kt(n.text, l, !1);
    if (i(n.text.slice(a, l)) != h)
      break;
    l = a;
  }
  for (; o < n.length; ) {
    let a = kt(n.text, o);
    if (i(n.text.slice(o, a)) != h)
      break;
    o = a;
  }
  return x.range(l + n.from, o + n.from);
}
function ql(r, t) {
  return t.left > r ? t.left - r : Math.max(0, r - t.right);
}
function Kl(r, t) {
  return t.top > r ? t.top - r : Math.max(0, r - t.bottom);
}
function yi(r, t) {
  return r.top < t.bottom - 1 && r.bottom > t.top + 1;
}
function jn(r, t) {
  return t < r.top ? { top: t, left: r.left, right: r.right, bottom: r.bottom } : r;
}
function $n(r, t) {
  return t > r.bottom ? { top: r.top, left: r.left, right: r.right, bottom: t } : r;
}
function Xi(r, t, e) {
  let i, n, s, l, o = !1, h, a, f, c;
  for (let g = r.firstChild; g; g = g.nextSibling) {
    let m = Se(g);
    for (let p = 0; p < m.length; p++) {
      let y = m[p];
      n && yi(n, y) && (y = jn($n(y, n.bottom), n.top));
      let S = ql(t, y), C = Kl(e, y);
      if (S == 0 && C == 0)
        return g.nodeType == 3 ? Yn(g, t, e) : Xi(g, t, e);
      if (!i || l > C || l == C && s > S) {
        i = g, n = y, s = S, l = C;
        let b = C ? e < y.top ? -1 : 1 : S ? t < y.left ? -1 : 1 : 0;
        o = !b || (b > 0 ? p < m.length - 1 : p > 0);
      }
      S == 0 ? e > y.bottom && (!f || f.bottom < y.bottom) ? (h = g, f = y) : e < y.top && (!c || c.top > y.top) && (a = g, c = y) : f && yi(f, y) ? f = $n(f, y.bottom) : c && yi(c, y) && (c = jn(c, y.top));
    }
  }
  if (f && f.bottom >= e ? (i = h, n = f) : c && c.top <= e && (i = a, n = c), !i)
    return { node: r, offset: 0 };
  let u = Math.max(n.left, Math.min(n.right, t));
  if (i.nodeType == 3)
    return Yn(i, u, e);
  if (o && i.contentEditable != "false")
    return Xi(i, u, e);
  let d = Array.prototype.indexOf.call(r.childNodes, i) + (t >= (n.left + n.right) / 2 ? 1 : 0);
  return { node: r, offset: d };
}
function Yn(r, t, e) {
  let i = r.nodeValue.length, n = -1, s = 1e9, l = 0;
  for (let o = 0; o < i; o++) {
    let h = Gt(r, o, o + 1).getClientRects();
    for (let a = 0; a < h.length; a++) {
      let f = h[a];
      if (f.top == f.bottom)
        continue;
      l || (l = t - f.left);
      let c = (f.top > e ? f.top - e : e - f.bottom) - 1;
      if (f.left - 1 <= t && f.right + 1 >= t && c < s) {
        let u = t >= (f.left + f.right) / 2, d = u;
        if ((k.chrome || k.gecko) && Gt(r, o).getBoundingClientRect().left == f.right && (d = !u), c <= 0)
          return { node: r, offset: o + (d ? 1 : 0) };
        n = o + (d ? 1 : 0), s = c;
      }
    }
  }
  return { node: r, offset: n > -1 ? n : l > 0 ? r.nodeValue.length : 0 };
}
function vr(r, t, e, i = -1) {
  var n, s;
  let l = r.contentDOM.getBoundingClientRect(), o = l.top + r.viewState.paddingTop, h, { docHeight: a } = r.viewState, { x: f, y: c } = t, u = c - o;
  if (u < 0)
    return 0;
  if (u > a)
    return r.state.doc.length;
  for (let b = r.viewState.heightOracle.textHeight / 2, A = !1; h = r.elementAtHeight(u), h.type != Ct.Text; )
    for (; u = i > 0 ? h.bottom + b : h.top - b, !(u >= 0 && u <= a); ) {
      if (A)
        return e ? null : 0;
      A = !0, i = -i;
    }
  c = o + u;
  let d = h.from;
  if (d < r.viewport.from)
    return r.viewport.from == 0 ? 0 : e ? null : Gn(r, l, h, f, c);
  if (d > r.viewport.to)
    return r.viewport.to == r.state.doc.length ? r.state.doc.length : e ? null : Gn(r, l, h, f, c);
  let g = r.dom.ownerDocument, m = r.root.elementFromPoint ? r.root : g, p = m.elementFromPoint(f, c);
  p && !r.contentDOM.contains(p) && (p = null), p || (f = Math.max(l.left + 1, Math.min(l.right - 1, f)), p = m.elementFromPoint(f, c), p && !r.contentDOM.contains(p) && (p = null));
  let y, S = -1;
  if (p && ((n = r.docView.nearest(p)) === null || n === void 0 ? void 0 : n.isEditable) != !1) {
    if (g.caretPositionFromPoint) {
      let b = g.caretPositionFromPoint(f, c);
      b && ({ offsetNode: y, offset: S } = b);
    } else if (g.caretRangeFromPoint) {
      let b = g.caretRangeFromPoint(f, c);
      b && ({ startContainer: y, startOffset: S } = b, (!r.contentDOM.contains(y) || k.safari && jl(y, S, f) || k.chrome && $l(y, S, f)) && (y = void 0));
    }
  }
  if (!y || !r.docView.dom.contains(y)) {
    let b = K.find(r.docView, d);
    if (!b)
      return u > h.top + h.height / 2 ? h.to : h.from;
    ({ node: y, offset: S } = Xi(b.dom, f, c));
  }
  let C = r.docView.nearest(y);
  if (!C)
    return null;
  if (C.isWidget && ((s = C.dom) === null || s === void 0 ? void 0 : s.nodeType) == 1) {
    let b = C.dom.getBoundingClientRect();
    return t.y < b.top || t.y <= b.bottom && t.x <= (b.left + b.right) / 2 ? C.posAtStart : C.posAtEnd;
  } else
    return C.localPosFromDOM(y, S) + C.posAtStart;
}
function Gn(r, t, e, i, n) {
  let s = Math.round((i - t.left) * r.defaultCharacterWidth);
  if (r.lineWrapping && e.height > r.defaultLineHeight * 1.5) {
    let o = r.viewState.heightOracle.textHeight, h = Math.floor((n - e.top - (r.defaultLineHeight - o) * 0.5) / o);
    s += h * r.viewState.heightOracle.lineLength;
  }
  let l = r.state.sliceDoc(e.from, e.to);
  return e.from + al(l, s, r.state.tabSize);
}
function jl(r, t, e) {
  let i;
  if (r.nodeType != 3 || t != (i = r.nodeValue.length))
    return !1;
  for (let n = r.nextSibling; n; n = n.nextSibling)
    if (n.nodeType != 1 || n.nodeName != "BR")
      return !1;
  return Gt(r, i - 1, i).getBoundingClientRect().left > e;
}
function $l(r, t, e) {
  if (t != 0)
    return !1;
  for (let n = r; ; ) {
    let s = n.parentNode;
    if (!s || s.nodeType != 1 || s.firstChild != n)
      return !1;
    if (s.classList.contains("cm-line"))
      break;
    n = s;
  }
  let i = r.nodeType == 1 ? r.getBoundingClientRect() : Gt(r, 0, Math.max(r.nodeValue.length, 1)).getBoundingClientRect();
  return e - i.left > 5;
}
function Yl(r, t) {
  let e = r.lineBlockAt(t);
  if (Array.isArray(e.type)) {
    for (let i of e.type)
      if (i.to > t || i.to == t && (i.to == e.to || i.type == Ct.Text))
        return i;
  }
  return e;
}
function Gl(r, t, e, i) {
  let n = Yl(r, t.head), s = !i || n.type != Ct.Text || !(r.lineWrapping || n.widgetLineBreaks) ? null : r.coordsAtPos(t.assoc < 0 && t.head > n.from ? t.head - 1 : t.head);
  if (s) {
    let l = r.dom.getBoundingClientRect(), o = r.textDirectionAt(n.from), h = r.posAtCoords({
      x: e == (o == Q.LTR) ? l.right - 1 : l.left + 1,
      y: (s.top + s.bottom) / 2
    });
    if (h != null)
      return x.cursor(h, e ? -1 : 1);
  }
  return x.cursor(e ? n.to : n.from, e ? -1 : 1);
}
function Jn(r, t, e, i) {
  let n = r.state.doc.lineAt(t.head), s = r.bidiSpans(n), l = r.textDirectionAt(n.from);
  for (let o = t, h = null; ; ) {
    let a = El(n, s, l, o, e), f = br;
    if (!a) {
      if (n.number == (e ? r.state.doc.lines : 1))
        return o;
      f = `
`, n = r.state.doc.line(n.number + (e ? 1 : -1)), s = r.bidiSpans(n), a = x.cursor(e ? n.from : n.to);
    }
    if (h) {
      if (!h(f))
        return o;
    } else {
      if (!i)
        return a;
      h = i(f);
    }
    o = a;
  }
}
function Jl(r, t, e) {
  let i = r.state.charCategorizer(t), n = i(e);
  return (s) => {
    let l = i(s);
    return n == vt.Space && (n = l), n == l;
  };
}
function Ul(r, t, e, i) {
  let n = t.head, s = e ? 1 : -1;
  if (n == (e ? r.state.doc.length : 0))
    return x.cursor(n, t.assoc);
  let l = t.goalColumn, o, h = r.contentDOM.getBoundingClientRect(), a = r.coordsAtPos(n, t.assoc || -1), f = r.documentTop;
  if (a)
    l == null && (l = a.left - h.left), o = s < 0 ? a.top : a.bottom;
  else {
    let d = r.viewState.lineBlockAt(n);
    l == null && (l = Math.min(h.right - h.left, r.defaultCharacterWidth * (n - d.from))), o = (s < 0 ? d.top : d.bottom) + f;
  }
  let c = h.left + l, u = i ?? r.viewState.heightOracle.textHeight >> 1;
  for (let d = 0; ; d += 10) {
    let g = o + (u + d) * s, m = vr(r, { x: c, y: g }, !1, s);
    if (g < h.top || g > h.bottom || (s < 0 ? m < n : m > n)) {
      let p = r.docView.coordsForChar(m), y = !p || g < p.top ? -1 : 1;
      return x.cursor(m, y, void 0, l);
    }
  }
}
function Ue(r, t, e) {
  for (; ; ) {
    let i = 0;
    for (let n of r)
      n.between(t - 1, t + 1, (s, l, o) => {
        if (t > s && t < l) {
          let h = i || e || (t - s < l - t ? -1 : 1);
          t = h < 0 ? s : l, i = h;
        }
      });
    if (!i)
      return t;
  }
}
function wi(r, t, e) {
  let i = Ue(r.state.facet(dn).map((n) => n(r)), e.from, t.head > e.from ? -1 : 1);
  return i == e.from ? e : x.cursor(i, i < e.from ? 1 : -1);
}
class Xl {
  setSelectionOrigin(t) {
    this.lastSelectionOrigin = t, this.lastSelectionTime = Date.now();
  }
  constructor(t) {
    this.view = t, this.lastKeyCode = 0, this.lastKeyTime = 0, this.lastTouchTime = 0, this.lastFocusTime = 0, this.lastScrollTop = 0, this.lastScrollLeft = 0, this.pendingIOSKey = void 0, this.lastSelectionOrigin = null, this.lastSelectionTime = 0, this.lastEscPress = 0, this.lastContextMenu = 0, this.scrollHandlers = [], this.handlers = /* @__PURE__ */ Object.create(null), this.composing = -1, this.compositionFirstChange = null, this.compositionEndedAt = 0, this.compositionPendingKey = !1, this.compositionPendingChange = !1, this.mouseSelection = null, this.handleEvent = this.handleEvent.bind(this), t.scrollDOM.addEventListener("mousedown", (e) => {
      if (e.target == t.scrollDOM && e.clientY > t.contentDOM.getBoundingClientRect().bottom && (this.runHandlers("mousedown", e), !e.defaultPrevented && e.button == 2)) {
        let i = t.contentDOM.style.minHeight;
        t.contentDOM.style.minHeight = "100%", setTimeout(() => t.contentDOM.style.minHeight = i, 200);
      }
    }), t.scrollDOM.addEventListener("drop", (e) => {
      e.target == t.scrollDOM && e.clientY > t.contentDOM.getBoundingClientRect().bottom && this.runHandlers("drop", e);
    }), this.notifiedFocused = t.hasFocus, k.safari && t.contentDOM.addEventListener("input", () => null), k.gecko && go(t.contentDOM.ownerDocument);
  }
  handleEvent(t) {
    !ro(this.view, t) || this.ignoreDuringComposition(t) || t.type == "keydown" && this.keydown(t) || this.runHandlers(t.type, t);
  }
  runHandlers(t, e) {
    let i = this.handlers[t];
    if (i) {
      for (let n of i.observers)
        n(this.view, e);
      for (let n of i.handlers) {
        if (e.defaultPrevented)
          break;
        if (n(this.view, e)) {
          e.preventDefault();
          break;
        }
      }
    }
  }
  ensureHandlers(t) {
    let e = _l(t), i = this.handlers, n = this.view.contentDOM;
    for (let s in e)
      if (s != "scroll") {
        let l = !e[s].handlers.length, o = i[s];
        o && l != !o.handlers.length && (n.removeEventListener(s, this.handleEvent), o = null), o || n.addEventListener(s, this.handleEvent, { passive: l });
      }
    for (let s in i)
      s != "scroll" && !e[s] && n.removeEventListener(s, this.handleEvent);
    this.handlers = e;
  }
  keydown(t) {
    if (this.lastKeyCode = t.keyCode, this.lastKeyTime = Date.now(), t.keyCode == 9 && Date.now() < this.lastEscPress + 2e3)
      return !0;
    if (t.keyCode != 27 && Ql.indexOf(t.keyCode) < 0 && (this.view.inputState.lastEscPress = 0), k.android && k.chrome && !t.synthetic && (t.keyCode == 13 || t.keyCode == 8))
      return this.view.observer.delayAndroidKey(t.key, t.keyCode), !0;
    let e;
    return k.ios && !t.synthetic && !t.altKey && !t.metaKey && ((e = Sr.find((i) => i.keyCode == t.keyCode)) && !t.ctrlKey || Zl.indexOf(t.key) > -1 && t.ctrlKey && !t.shiftKey) ? (this.pendingIOSKey = e || t, setTimeout(() => this.flushIOSKey(), 250), !0) : (t.keyCode != 229 && this.view.observer.forceFlush(), !1);
  }
  flushIOSKey() {
    let t = this.pendingIOSKey;
    return t ? (this.pendingIOSKey = void 0, te(this.view.contentDOM, t.key, t.keyCode)) : !1;
  }
  ignoreDuringComposition(t) {
    return /^key/.test(t.type) ? this.composing > 0 ? !0 : k.safari && !k.ios && this.compositionPendingKey && Date.now() - this.compositionEndedAt < 100 ? (this.compositionPendingKey = !1, !0) : !1 : !1;
  }
  startMouseSelection(t) {
    this.mouseSelection && this.mouseSelection.destroy(), this.mouseSelection = t;
  }
  update(t) {
    this.mouseSelection && this.mouseSelection.update(t), t.transactions.length && (this.lastKeyCode = this.lastSelectionTime = 0);
  }
  destroy() {
    this.mouseSelection && this.mouseSelection.destroy();
  }
}
function Un(r, t) {
  return (e, i) => {
    try {
      return t.call(r, i, e);
    } catch (n) {
      $t(e.state, n);
    }
  };
}
function _l(r) {
  let t = /* @__PURE__ */ Object.create(null);
  function e(i) {
    return t[i] || (t[i] = { observers: [], handlers: [] });
  }
  for (let i of r) {
    let n = i.spec;
    if (n && n.domEventHandlers)
      for (let s in n.domEventHandlers) {
        let l = n.domEventHandlers[s];
        l && e(s).handlers.push(Un(i.value, l));
      }
    if (n && n.domEventObservers)
      for (let s in n.domEventObservers) {
        let l = n.domEventObservers[s];
        l && e(s).observers.push(Un(i.value, l));
      }
  }
  for (let i in wt)
    e(i).handlers.push(wt[i]);
  for (let i in ft)
    e(i).observers.push(ft[i]);
  return t;
}
const Sr = [
  { key: "Backspace", keyCode: 8, inputType: "deleteContentBackward" },
  { key: "Enter", keyCode: 13, inputType: "insertParagraph" },
  { key: "Enter", keyCode: 13, inputType: "insertLineBreak" },
  { key: "Delete", keyCode: 46, inputType: "deleteContentForward" }
], Zl = "dthko", Ql = [16, 17, 18, 20, 91, 92, 224, 225], Ne = 6;
function Ie(r) {
  return Math.max(0, r) * 0.7 + 8;
}
function to(r, t) {
  return Math.max(Math.abs(r.clientX - t.clientX), Math.abs(r.clientY - t.clientY));
}
class eo {
  constructor(t, e, i, n) {
    this.view = t, this.startEvent = e, this.style = i, this.mustSelect = n, this.scrollSpeed = { x: 0, y: 0 }, this.scrolling = -1, this.lastEvent = e, this.scrollParent = gl(t.contentDOM), this.atoms = t.state.facet(dn).map((l) => l(t));
    let s = t.contentDOM.ownerDocument;
    s.addEventListener("mousemove", this.move = this.move.bind(this)), s.addEventListener("mouseup", this.up = this.up.bind(this)), this.extend = e.shiftKey, this.multiple = t.state.facet(E.allowMultipleSelections) && io(t, e), this.dragging = so(t, e) && Or(e) == 1 ? null : !1;
  }
  start(t) {
    this.dragging === !1 && this.select(t);
  }
  move(t) {
    var e;
    if (t.buttons == 0)
      return this.destroy();
    if (this.dragging || this.dragging == null && to(this.startEvent, t) < 10)
      return;
    this.select(this.lastEvent = t);
    let i = 0, n = 0, s = ((e = this.scrollParent) === null || e === void 0 ? void 0 : e.getBoundingClientRect()) || { left: 0, top: 0, right: this.view.win.innerWidth, bottom: this.view.win.innerHeight }, l = gr(this.view);
    t.clientX - l.left <= s.left + Ne ? i = -Ie(s.left - t.clientX) : t.clientX + l.right >= s.right - Ne && (i = Ie(t.clientX - s.right)), t.clientY - l.top <= s.top + Ne ? n = -Ie(s.top - t.clientY) : t.clientY + l.bottom >= s.bottom - Ne && (n = Ie(t.clientY - s.bottom)), this.setScrollSpeed(i, n);
  }
  up(t) {
    this.dragging == null && this.select(this.lastEvent), this.dragging || t.preventDefault(), this.destroy();
  }
  destroy() {
    this.setScrollSpeed(0, 0);
    let t = this.view.contentDOM.ownerDocument;
    t.removeEventListener("mousemove", this.move), t.removeEventListener("mouseup", this.up), this.view.inputState.mouseSelection = null;
  }
  setScrollSpeed(t, e) {
    this.scrollSpeed = { x: t, y: e }, t || e ? this.scrolling < 0 && (this.scrolling = setInterval(() => this.scroll(), 50)) : this.scrolling > -1 && (clearInterval(this.scrolling), this.scrolling = -1);
  }
  scroll() {
    this.scrollParent ? (this.scrollParent.scrollLeft += this.scrollSpeed.x, this.scrollParent.scrollTop += this.scrollSpeed.y) : this.view.win.scrollBy(this.scrollSpeed.x, this.scrollSpeed.y), this.dragging === !1 && this.select(this.lastEvent);
  }
  skipAtoms(t) {
    let e = null;
    for (let i = 0; i < t.ranges.length; i++) {
      let n = t.ranges[i], s = null;
      if (n.empty) {
        let l = Ue(this.atoms, n.from, 0);
        l != n.from && (s = x.cursor(l, -1));
      } else {
        let l = Ue(this.atoms, n.from, -1), o = Ue(this.atoms, n.to, 1);
        (l != n.from || o != n.to) && (s = x.range(n.from == n.anchor ? l : o, n.from == n.head ? l : o));
      }
      s && (e || (e = t.ranges.slice()), e[i] = s);
    }
    return e ? x.create(e, t.mainIndex) : t;
  }
  select(t) {
    let { view: e } = this, i = this.skipAtoms(this.style.get(t, this.extend, this.multiple));
    (this.mustSelect || !i.eq(e.state.selection) || i.main.assoc != e.state.selection.main.assoc && this.dragging === !1) && this.view.dispatch({
      selection: i,
      userEvent: "select.pointer"
    }), this.mustSelect = !1;
  }
  update(t) {
    t.docChanged && this.dragging && (this.dragging = this.dragging.map(t.changes)), this.style.update(t) && setTimeout(() => this.select(this.lastEvent), 20);
  }
}
function io(r, t) {
  let e = r.state.facet(sr);
  return e.length ? e[0](t) : k.mac ? t.metaKey : t.ctrlKey;
}
function no(r, t) {
  let e = r.state.facet(rr);
  return e.length ? e[0](t) : k.mac ? !t.altKey : !t.ctrlKey;
}
function so(r, t) {
  let { main: e } = r.state.selection;
  if (e.empty)
    return !1;
  let i = ei(r.root);
  if (!i || i.rangeCount == 0)
    return !0;
  let n = i.getRangeAt(0).getClientRects();
  for (let s = 0; s < n.length; s++) {
    let l = n[s];
    if (l.left <= t.clientX && l.right >= t.clientX && l.top <= t.clientY && l.bottom >= t.clientY)
      return !0;
  }
  return !1;
}
function ro(r, t) {
  if (!t.bubbles)
    return !0;
  if (t.defaultPrevented)
    return !1;
  for (let e = t.target, i; e != r.contentDOM; e = e.parentNode)
    if (!e || e.nodeType == 11 || (i = I.get(e)) && i.ignoreEvent(t))
      return !1;
  return !0;
}
const wt = /* @__PURE__ */ Object.create(null), ft = /* @__PURE__ */ Object.create(null), Ar = k.ie && k.ie_version < 15 || k.ios && k.webkit_version < 604;
function lo(r) {
  let t = r.dom.parentNode;
  if (!t)
    return;
  let e = t.appendChild(document.createElement("textarea"));
  e.style.cssText = "position: fixed; left: -10000px; top: 10px", e.focus(), setTimeout(() => {
    r.focus(), e.remove(), Cr(r, e.value);
  }, 50);
}
function Cr(r, t) {
  let { state: e } = r, i, n = 1, s = e.toText(t), l = s.lines == e.selection.ranges.length;
  if (_i != null && e.selection.ranges.every((h) => h.empty) && _i == s.toString()) {
    let h = -1;
    i = e.changeByRange((a) => {
      let f = e.doc.lineAt(a.from);
      if (f.from == h)
        return { range: a };
      h = f.from;
      let c = e.toText((l ? s.line(n++).text : t) + e.lineBreak);
      return {
        changes: { from: f.from, insert: c },
        range: x.cursor(a.from + c.length)
      };
    });
  } else
    l ? i = e.changeByRange((h) => {
      let a = s.line(n++);
      return {
        changes: { from: h.from, to: h.to, insert: a.text },
        range: x.cursor(h.from + a.length)
      };
    }) : i = e.replaceSelection(s);
  r.dispatch(i, {
    userEvent: "input.paste",
    scrollIntoView: !0
  });
}
ft.scroll = (r) => {
  r.inputState.lastScrollTop = r.scrollDOM.scrollTop, r.inputState.lastScrollLeft = r.scrollDOM.scrollLeft;
};
wt.keydown = (r, t) => (r.inputState.setSelectionOrigin("select"), t.keyCode == 27 && (r.inputState.lastEscPress = Date.now()), !1);
ft.touchstart = (r, t) => {
  r.inputState.lastTouchTime = Date.now(), r.inputState.setSelectionOrigin("select.pointer");
};
ft.touchmove = (r) => {
  r.inputState.setSelectionOrigin("select.pointer");
};
wt.mousedown = (r, t) => {
  if (r.observer.flush(), r.inputState.lastTouchTime > Date.now() - 2e3)
    return !1;
  let e = null;
  for (let i of r.state.facet(lr))
    if (e = i(r, t), e)
      break;
  if (!e && t.button == 0 && (e = ao(r, t)), e) {
    let i = !r.hasFocus;
    r.inputState.startMouseSelection(new eo(r, t, e, i)), i && r.observer.ignore(() => Ys(r.contentDOM));
    let n = r.inputState.mouseSelection;
    if (n)
      return n.start(t), !n.dragging;
  }
  return !1;
};
function Xn(r, t, e, i) {
  if (i == 1)
    return x.cursor(t, e);
  if (i == 2)
    return zl(r.state, t, e);
  {
    let n = K.find(r.docView, t), s = r.state.doc.lineAt(n ? n.posAtEnd : t), l = n ? n.posAtStart : s.from, o = n ? n.posAtEnd : s.to;
    return o < r.state.doc.length && o == s.to && o++, x.range(l, o);
  }
}
let Mr = (r, t) => r >= t.top && r <= t.bottom, _n = (r, t, e) => Mr(t, e) && r >= e.left && r <= e.right;
function oo(r, t, e, i) {
  let n = K.find(r.docView, t);
  if (!n)
    return 1;
  let s = t - n.posAtStart;
  if (s == 0)
    return 1;
  if (s == n.length)
    return -1;
  let l = n.coordsAt(s, -1);
  if (l && _n(e, i, l))
    return -1;
  let o = n.coordsAt(s, 1);
  return o && _n(e, i, o) ? 1 : l && Mr(i, l) ? -1 : 1;
}
function Zn(r, t) {
  let e = r.posAtCoords({ x: t.clientX, y: t.clientY }, !1);
  return { pos: e, bias: oo(r, e, t.clientX, t.clientY) };
}
const ho = k.ie && k.ie_version <= 11;
let Qn = null, ts = 0, es = 0;
function Or(r) {
  if (!ho)
    return r.detail;
  let t = Qn, e = es;
  return Qn = r, es = Date.now(), ts = !t || e > Date.now() - 400 && Math.abs(t.clientX - r.clientX) < 2 && Math.abs(t.clientY - r.clientY) < 2 ? (ts + 1) % 3 : 1;
}
function ao(r, t) {
  let e = Zn(r, t), i = Or(t), n = r.state.selection;
  return {
    update(s) {
      s.docChanged && (e.pos = s.changes.mapPos(e.pos), n = n.map(s.changes));
    },
    get(s, l, o) {
      let h = Zn(r, s), a, f = Xn(r, h.pos, h.bias, i);
      if (e.pos != h.pos && !l) {
        let c = Xn(r, e.pos, e.bias, i), u = Math.min(c.from, f.from), d = Math.max(c.to, f.to);
        f = u < f.from ? x.range(u, d) : x.range(d, u);
      }
      return l ? n.replaceRange(n.main.extend(f.from, f.to)) : o && i == 1 && n.ranges.length > 1 && (a = fo(n, h.pos)) ? a : o ? n.addRange(f) : x.create([f]);
    }
  };
}
function fo(r, t) {
  for (let e = 0; e < r.ranges.length; e++) {
    let { from: i, to: n } = r.ranges[e];
    if (i <= t && n >= t)
      return x.create(r.ranges.slice(0, e).concat(r.ranges.slice(e + 1)), r.mainIndex == e ? 0 : r.mainIndex - (r.mainIndex > e ? 1 : 0));
  }
  return null;
}
wt.dragstart = (r, t) => {
  let { selection: { main: e } } = r.state, { mouseSelection: i } = r.inputState;
  return i && (i.dragging = e), t.dataTransfer && (t.dataTransfer.setData("Text", r.state.sliceDoc(e.from, e.to)), t.dataTransfer.effectAllowed = "copyMove"), !1;
};
function is(r, t, e, i) {
  if (!e)
    return;
  let n = r.posAtCoords({ x: t.clientX, y: t.clientY }, !1), { mouseSelection: s } = r.inputState, l = i && s && s.dragging && no(r, t) ? { from: s.dragging.from, to: s.dragging.to } : null, o = { from: n, insert: e }, h = r.state.changes(l ? [l, o] : o);
  r.focus(), r.dispatch({
    changes: h,
    selection: { anchor: h.mapPos(n, -1), head: h.mapPos(n, 1) },
    userEvent: l ? "move.drop" : "input.drop"
  });
}
wt.drop = (r, t) => {
  if (!t.dataTransfer)
    return !1;
  if (r.state.readOnly)
    return !0;
  let e = t.dataTransfer.files;
  if (e && e.length) {
    let i = Array(e.length), n = 0, s = () => {
      ++n == e.length && is(r, t, i.filter((l) => l != null).join(r.state.lineBreak), !1);
    };
    for (let l = 0; l < e.length; l++) {
      let o = new FileReader();
      o.onerror = s, o.onload = () => {
        /[\x00-\x08\x0e-\x1f]{2}/.test(o.result) || (i[l] = o.result), s();
      }, o.readAsText(e[l]);
    }
    return !0;
  } else {
    let i = t.dataTransfer.getData("Text");
    if (i)
      return is(r, t, i, !0), !0;
  }
  return !1;
};
wt.paste = (r, t) => {
  if (r.state.readOnly)
    return !0;
  r.observer.flush();
  let e = Ar ? null : t.clipboardData;
  return e ? (Cr(r, e.getData("text/plain") || e.getData("text/uri-text")), !0) : (lo(r), !1);
};
function co(r, t) {
  let e = r.dom.parentNode;
  if (!e)
    return;
  let i = e.appendChild(document.createElement("textarea"));
  i.style.cssText = "position: fixed; left: -10000px; top: 10px", i.value = t, i.focus(), i.selectionEnd = t.length, i.selectionStart = 0, setTimeout(() => {
    i.remove(), r.focus();
  }, 50);
}
function uo(r) {
  let t = [], e = [], i = !1;
  for (let n of r.selection.ranges)
    n.empty || (t.push(r.sliceDoc(n.from, n.to)), e.push(n));
  if (!t.length) {
    let n = -1;
    for (let { from: s } of r.selection.ranges) {
      let l = r.doc.lineAt(s);
      l.number > n && (t.push(l.text), e.push({ from: l.from, to: Math.min(r.doc.length, l.to + 1) })), n = l.number;
    }
    i = !0;
  }
  return { text: t.join(r.lineBreak), ranges: e, linewise: i };
}
let _i = null;
wt.copy = wt.cut = (r, t) => {
  let { text: e, ranges: i, linewise: n } = uo(r.state);
  if (!e && !n)
    return !1;
  _i = n ? e : null, t.type == "cut" && !r.state.readOnly && r.dispatch({
    changes: i,
    scrollIntoView: !0,
    userEvent: "delete.cut"
  });
  let s = Ar ? null : t.clipboardData;
  return s ? (s.clearData(), s.setData("text/plain", e), !0) : (co(r, e), !1);
};
const Dr = /* @__PURE__ */ ae.define();
function Tr(r, t) {
  let e = [];
  for (let i of r.facet(ar)) {
    let n = i(r, t);
    n && e.push(n);
  }
  return e ? r.update({ effects: e, annotations: Dr.of(!0) }) : null;
}
function Pr(r) {
  setTimeout(() => {
    let t = r.hasFocus;
    if (t != r.inputState.notifiedFocused) {
      let e = Tr(r.state, t);
      e ? r.dispatch(e) : r.update([]);
    }
  }, 10);
}
ft.focus = (r) => {
  r.inputState.lastFocusTime = Date.now(), !r.scrollDOM.scrollTop && (r.inputState.lastScrollTop || r.inputState.lastScrollLeft) && (r.scrollDOM.scrollTop = r.inputState.lastScrollTop, r.scrollDOM.scrollLeft = r.inputState.lastScrollLeft), Pr(r);
};
ft.blur = (r) => {
  r.observer.clearSelectionRange(), Pr(r);
};
ft.compositionstart = ft.compositionupdate = (r) => {
  r.inputState.compositionFirstChange == null && (r.inputState.compositionFirstChange = !0), r.inputState.composing < 0 && (r.inputState.composing = 0);
};
ft.compositionend = (r) => {
  r.inputState.composing = -1, r.inputState.compositionEndedAt = Date.now(), r.inputState.compositionPendingKey = !0, r.inputState.compositionPendingChange = r.observer.pendingRecords().length > 0, r.inputState.compositionFirstChange = null, k.chrome && k.android ? r.observer.flushSoon() : r.inputState.compositionPendingChange ? Promise.resolve().then(() => r.observer.flush()) : setTimeout(() => {
    r.inputState.composing < 0 && r.docView.hasComposition && r.update([]);
  }, 50);
};
ft.contextmenu = (r) => {
  r.inputState.lastContextMenu = Date.now();
};
wt.beforeinput = (r, t) => {
  var e;
  let i;
  if (k.chrome && k.android && (i = Sr.find((n) => n.inputType == t.inputType)) && (r.observer.delayAndroidKey(i.key, i.keyCode), i.key == "Backspace" || i.key == "Delete")) {
    let n = ((e = window.visualViewport) === null || e === void 0 ? void 0 : e.height) || 0;
    setTimeout(() => {
      var s;
      (((s = window.visualViewport) === null || s === void 0 ? void 0 : s.height) || 0) > n + 10 && r.hasFocus && (r.contentDOM.blur(), r.focus());
    }, 100);
  }
  return !1;
};
const ns = /* @__PURE__ */ new Set();
function go(r) {
  ns.has(r) || (ns.add(r), r.addEventListener("copy", () => {
  }), r.addEventListener("cut", () => {
  }));
}
const ss = ["pre-wrap", "normal", "pre-line", "break-spaces"];
class po {
  constructor(t) {
    this.lineWrapping = t, this.doc = R.empty, this.heightSamples = {}, this.lineHeight = 14, this.charWidth = 7, this.textHeight = 14, this.lineLength = 30, this.heightChanged = !1;
  }
  heightForGap(t, e) {
    let i = this.doc.lineAt(e).number - this.doc.lineAt(t).number + 1;
    return this.lineWrapping && (i += Math.max(0, Math.ceil((e - t - i * this.lineLength * 0.5) / this.lineLength))), this.lineHeight * i;
  }
  heightForLine(t) {
    return this.lineWrapping ? (1 + Math.max(0, Math.ceil((t - this.lineLength) / (this.lineLength - 5)))) * this.lineHeight : this.lineHeight;
  }
  setDoc(t) {
    return this.doc = t, this;
  }
  mustRefreshForWrapping(t) {
    return ss.indexOf(t) > -1 != this.lineWrapping;
  }
  mustRefreshForHeights(t) {
    let e = !1;
    for (let i = 0; i < t.length; i++) {
      let n = t[i];
      n < 0 ? i++ : this.heightSamples[Math.floor(n * 10)] || (e = !0, this.heightSamples[Math.floor(n * 10)] = !0);
    }
    return e;
  }
  refresh(t, e, i, n, s, l) {
    let o = ss.indexOf(t) > -1, h = Math.round(e) != Math.round(this.lineHeight) || this.lineWrapping != o;
    if (this.lineWrapping = o, this.lineHeight = e, this.charWidth = i, this.textHeight = n, this.lineLength = s, h) {
      this.heightSamples = {};
      for (let a = 0; a < l.length; a++) {
        let f = l[a];
        f < 0 ? a++ : this.heightSamples[Math.floor(f * 10)] = !0;
      }
    }
    return h;
  }
}
class mo {
  constructor(t, e) {
    this.from = t, this.heights = e, this.index = 0;
  }
  get more() {
    return this.index < this.heights.length;
  }
}
class yt {
  /**
  @internal
  */
  constructor(t, e, i, n, s) {
    this.from = t, this.length = e, this.top = i, this.height = n, this._content = s;
  }
  /**
  The type of element this is. When querying lines, this may be
  an array of all the blocks that make up the line.
  */
  get type() {
    return typeof this._content == "number" ? Ct.Text : Array.isArray(this._content) ? this._content : this._content.type;
  }
  /**
  The end of the element as a document position.
  */
  get to() {
    return this.from + this.length;
  }
  /**
  The bottom position of the element.
  */
  get bottom() {
    return this.top + this.height;
  }
  /**
  If this is a widget block, this will return the widget
  associated with it.
  */
  get widget() {
    return this._content instanceof Ht ? this._content.widget : null;
  }
  /**
  If this is a textblock, this holds the number of line breaks
  that appear in widgets inside the block.
  */
  get widgetLineBreaks() {
    return typeof this._content == "number" ? this._content : 0;
  }
  /**
  @internal
  */
  join(t) {
    let e = (Array.isArray(this._content) ? this._content : [this]).concat(Array.isArray(t._content) ? t._content : [t]);
    return new yt(this.from, this.length + t.length, this.top, this.height + t.height, e);
  }
}
var N = /* @__PURE__ */ function(r) {
  return r[r.ByPos = 0] = "ByPos", r[r.ByHeight = 1] = "ByHeight", r[r.ByPosNoHeight = 2] = "ByPosNoHeight", r;
}(N || (N = {}));
const Xe = 1e-3;
class tt {
  constructor(t, e, i = 2) {
    this.length = t, this.height = e, this.flags = i;
  }
  get outdated() {
    return (this.flags & 2) > 0;
  }
  set outdated(t) {
    this.flags = (t ? 2 : 0) | this.flags & -3;
  }
  setHeight(t, e) {
    this.height != e && (Math.abs(this.height - e) > Xe && (t.heightChanged = !0), this.height = e);
  }
  // Base case is to replace a leaf node, which simply builds a tree
  // from the new nodes and returns that (HeightMapBranch and
  // HeightMapGap override this to actually use from/to)
  replace(t, e, i) {
    return tt.of(i);
  }
  // Again, these are base cases, and are overridden for branch and gap nodes.
  decomposeLeft(t, e) {
    e.push(this);
  }
  decomposeRight(t, e) {
    e.push(this);
  }
  applyChanges(t, e, i, n) {
    let s = this, l = i.doc;
    for (let o = n.length - 1; o >= 0; o--) {
      let { fromA: h, toA: a, fromB: f, toB: c } = n[o], u = s.lineAt(h, N.ByPosNoHeight, i.setDoc(e), 0, 0), d = u.to >= a ? u : s.lineAt(a, N.ByPosNoHeight, i, 0, 0);
      for (c += d.to - a, a = d.to; o > 0 && u.from <= n[o - 1].toA; )
        h = n[o - 1].fromA, f = n[o - 1].fromB, o--, h < u.from && (u = s.lineAt(h, N.ByPosNoHeight, i, 0, 0));
      f += u.from - h, h = u.from;
      let g = gn.build(i.setDoc(l), t, f, c);
      s = s.replace(h, a, g);
    }
    return s.updateHeight(i, 0);
  }
  static empty() {
    return new it(0, 0);
  }
  // nodes uses null values to indicate the position of line breaks.
  // There are never line breaks at the start or end of the array, or
  // two line breaks next to each other, and the array isn't allowed
  // to be empty (same restrictions as return value from the builder).
  static of(t) {
    if (t.length == 1)
      return t[0];
    let e = 0, i = t.length, n = 0, s = 0;
    for (; ; )
      if (e == i)
        if (n > s * 2) {
          let o = t[e - 1];
          o.break ? t.splice(--e, 1, o.left, null, o.right) : t.splice(--e, 1, o.left, o.right), i += 1 + o.break, n -= o.size;
        } else if (s > n * 2) {
          let o = t[i];
          o.break ? t.splice(i, 1, o.left, null, o.right) : t.splice(i, 1, o.left, o.right), i += 2 + o.break, s -= o.size;
        } else
          break;
      else if (n < s) {
        let o = t[e++];
        o && (n += o.size);
      } else {
        let o = t[--i];
        o && (s += o.size);
      }
    let l = 0;
    return t[e - 1] == null ? (l = 1, e--) : t[e] == null && (l = 1, i++), new yo(tt.of(t.slice(0, e)), l, tt.of(t.slice(i)));
  }
}
tt.prototype.size = 1;
class Br extends tt {
  constructor(t, e, i) {
    super(t, e), this.deco = i;
  }
  blockAt(t, e, i, n) {
    return new yt(n, this.length, i, this.height, this.deco || 0);
  }
  lineAt(t, e, i, n, s) {
    return this.blockAt(0, i, n, s);
  }
  forEachLine(t, e, i, n, s, l) {
    t <= s + this.length && e >= s && l(this.blockAt(0, i, n, s));
  }
  updateHeight(t, e = 0, i = !1, n) {
    return n && n.from <= e && n.more && this.setHeight(t, n.heights[n.index++]), this.outdated = !1, this;
  }
  toString() {
    return `block(${this.length})`;
  }
}
class it extends Br {
  constructor(t, e) {
    super(t, e, null), this.collapsed = 0, this.widgetHeight = 0, this.breaks = 0;
  }
  blockAt(t, e, i, n) {
    return new yt(n, this.length, i, this.height, this.breaks);
  }
  replace(t, e, i) {
    let n = i[0];
    return i.length == 1 && (n instanceof it || n instanceof Y && n.flags & 4) && Math.abs(this.length - n.length) < 10 ? (n instanceof Y ? n = new it(n.length, this.height) : n.height = this.height, this.outdated || (n.outdated = !1), n) : tt.of(i);
  }
  updateHeight(t, e = 0, i = !1, n) {
    return n && n.from <= e && n.more ? this.setHeight(t, n.heights[n.index++]) : (i || this.outdated) && this.setHeight(t, Math.max(this.widgetHeight, t.heightForLine(this.length - this.collapsed)) + this.breaks * t.lineHeight), this.outdated = !1, this;
  }
  toString() {
    return `line(${this.length}${this.collapsed ? -this.collapsed : ""}${this.widgetHeight ? ":" + this.widgetHeight : ""})`;
  }
}
class Y extends tt {
  constructor(t) {
    super(t, 0);
  }
  heightMetrics(t, e) {
    let i = t.doc.lineAt(e).number, n = t.doc.lineAt(e + this.length).number, s = n - i + 1, l, o = 0;
    if (t.lineWrapping) {
      let h = Math.min(this.height, t.lineHeight * s);
      l = h / s, this.length > s + 1 && (o = (this.height - h) / (this.length - s - 1));
    } else
      l = this.height / s;
    return { firstLine: i, lastLine: n, perLine: l, perChar: o };
  }
  blockAt(t, e, i, n) {
    let { firstLine: s, lastLine: l, perLine: o, perChar: h } = this.heightMetrics(e, n);
    if (e.lineWrapping) {
      let a = n + Math.round(Math.max(0, Math.min(1, (t - i) / this.height)) * this.length), f = e.doc.lineAt(a), c = o + f.length * h, u = Math.max(i, t - c / 2);
      return new yt(f.from, f.length, u, c, 0);
    } else {
      let a = Math.max(0, Math.min(l - s, Math.floor((t - i) / o))), { from: f, length: c } = e.doc.line(s + a);
      return new yt(f, c, i + o * a, o, 0);
    }
  }
  lineAt(t, e, i, n, s) {
    if (e == N.ByHeight)
      return this.blockAt(t, i, n, s);
    if (e == N.ByPosNoHeight) {
      let { from: d, to: g } = i.doc.lineAt(t);
      return new yt(d, g - d, 0, 0, 0);
    }
    let { firstLine: l, perLine: o, perChar: h } = this.heightMetrics(i, s), a = i.doc.lineAt(t), f = o + a.length * h, c = a.number - l, u = n + o * c + h * (a.from - s - c);
    return new yt(a.from, a.length, Math.max(n, Math.min(u, n + this.height - f)), f, 0);
  }
  forEachLine(t, e, i, n, s, l) {
    t = Math.max(t, s), e = Math.min(e, s + this.length);
    let { firstLine: o, perLine: h, perChar: a } = this.heightMetrics(i, s);
    for (let f = t, c = n; f <= e; ) {
      let u = i.doc.lineAt(f);
      if (f == t) {
        let g = u.number - o;
        c += h * g + a * (t - s - g);
      }
      let d = h + a * u.length;
      l(new yt(u.from, u.length, c, d, 0)), c += d, f = u.to + 1;
    }
  }
  replace(t, e, i) {
    let n = this.length - e;
    if (n > 0) {
      let s = i[i.length - 1];
      s instanceof Y ? i[i.length - 1] = new Y(s.length + n) : i.push(null, new Y(n - 1));
    }
    if (t > 0) {
      let s = i[0];
      s instanceof Y ? i[0] = new Y(t + s.length) : i.unshift(new Y(t - 1), null);
    }
    return tt.of(i);
  }
  decomposeLeft(t, e) {
    e.push(new Y(t - 1), null);
  }
  decomposeRight(t, e) {
    e.push(null, new Y(this.length - t - 1));
  }
  updateHeight(t, e = 0, i = !1, n) {
    let s = e + this.length;
    if (n && n.from <= e + this.length && n.more) {
      let l = [], o = Math.max(e, n.from), h = -1;
      for (n.from > e && l.push(new Y(n.from - e - 1).updateHeight(t, e)); o <= s && n.more; ) {
        let f = t.doc.lineAt(o).length;
        l.length && l.push(null);
        let c = n.heights[n.index++];
        h == -1 ? h = c : Math.abs(c - h) >= Xe && (h = -2);
        let u = new it(f, c);
        u.outdated = !1, l.push(u), o += f + 1;
      }
      o <= s && l.push(null, new Y(s - o).updateHeight(t, o));
      let a = tt.of(l);
      return (h < 0 || Math.abs(a.height - this.height) >= Xe || Math.abs(h - this.heightMetrics(t, e).perLine) >= Xe) && (t.heightChanged = !0), a;
    } else
      (i || this.outdated) && (this.setHeight(t, t.heightForGap(e, e + this.length)), this.outdated = !1);
    return this;
  }
  toString() {
    return `gap(${this.length})`;
  }
}
class yo extends tt {
  constructor(t, e, i) {
    super(t.length + e + i.length, t.height + i.height, e | (t.outdated || i.outdated ? 2 : 0)), this.left = t, this.right = i, this.size = t.size + i.size;
  }
  get break() {
    return this.flags & 1;
  }
  blockAt(t, e, i, n) {
    let s = i + this.left.height;
    return t < s ? this.left.blockAt(t, e, i, n) : this.right.blockAt(t, e, s, n + this.left.length + this.break);
  }
  lineAt(t, e, i, n, s) {
    let l = n + this.left.height, o = s + this.left.length + this.break, h = e == N.ByHeight ? t < l : t < o, a = h ? this.left.lineAt(t, e, i, n, s) : this.right.lineAt(t, e, i, l, o);
    if (this.break || (h ? a.to < o : a.from > o))
      return a;
    let f = e == N.ByPosNoHeight ? N.ByPosNoHeight : N.ByPos;
    return h ? a.join(this.right.lineAt(o, f, i, l, o)) : this.left.lineAt(o, f, i, n, s).join(a);
  }
  forEachLine(t, e, i, n, s, l) {
    let o = n + this.left.height, h = s + this.left.length + this.break;
    if (this.break)
      t < h && this.left.forEachLine(t, e, i, n, s, l), e >= h && this.right.forEachLine(t, e, i, o, h, l);
    else {
      let a = this.lineAt(h, N.ByPos, i, n, s);
      t < a.from && this.left.forEachLine(t, a.from - 1, i, n, s, l), a.to >= t && a.from <= e && l(a), e > a.to && this.right.forEachLine(a.to + 1, e, i, o, h, l);
    }
  }
  replace(t, e, i) {
    let n = this.left.length + this.break;
    if (e < n)
      return this.balanced(this.left.replace(t, e, i), this.right);
    if (t > this.left.length)
      return this.balanced(this.left, this.right.replace(t - n, e - n, i));
    let s = [];
    t > 0 && this.decomposeLeft(t, s);
    let l = s.length;
    for (let o of i)
      s.push(o);
    if (t > 0 && rs(s, l - 1), e < this.length) {
      let o = s.length;
      this.decomposeRight(e, s), rs(s, o);
    }
    return tt.of(s);
  }
  decomposeLeft(t, e) {
    let i = this.left.length;
    if (t <= i)
      return this.left.decomposeLeft(t, e);
    e.push(this.left), this.break && (i++, t >= i && e.push(null)), t > i && this.right.decomposeLeft(t - i, e);
  }
  decomposeRight(t, e) {
    let i = this.left.length, n = i + this.break;
    if (t >= n)
      return this.right.decomposeRight(t - n, e);
    t < i && this.left.decomposeRight(t, e), this.break && t < n && e.push(null), e.push(this.right);
  }
  balanced(t, e) {
    return t.size > 2 * e.size || e.size > 2 * t.size ? tt.of(this.break ? [t, null, e] : [t, e]) : (this.left = t, this.right = e, this.height = t.height + e.height, this.outdated = t.outdated || e.outdated, this.size = t.size + e.size, this.length = t.length + this.break + e.length, this);
  }
  updateHeight(t, e = 0, i = !1, n) {
    let { left: s, right: l } = this, o = e + s.length + this.break, h = null;
    return n && n.from <= e + s.length && n.more ? h = s = s.updateHeight(t, e, i, n) : s.updateHeight(t, e, i), n && n.from <= o + l.length && n.more ? h = l = l.updateHeight(t, o, i, n) : l.updateHeight(t, o, i), h ? this.balanced(s, l) : (this.height = this.left.height + this.right.height, this.outdated = !1, this);
  }
  toString() {
    return this.left + (this.break ? " " : "-") + this.right;
  }
}
function rs(r, t) {
  let e, i;
  r[t] == null && (e = r[t - 1]) instanceof Y && (i = r[t + 1]) instanceof Y && r.splice(t - 1, 3, new Y(e.length + 1 + i.length));
}
const wo = 5;
class gn {
  constructor(t, e) {
    this.pos = t, this.oracle = e, this.nodes = [], this.lineStart = -1, this.lineEnd = -1, this.covering = null, this.writtenTo = t;
  }
  get isCovered() {
    return this.covering && this.nodes[this.nodes.length - 1] == this.covering;
  }
  span(t, e) {
    if (this.lineStart > -1) {
      let i = Math.min(e, this.lineEnd), n = this.nodes[this.nodes.length - 1];
      n instanceof it ? n.length += i - this.pos : (i > this.pos || !this.isCovered) && this.nodes.push(new it(i - this.pos, -1)), this.writtenTo = i, e > i && (this.nodes.push(null), this.writtenTo++, this.lineStart = -1);
    }
    this.pos = e;
  }
  point(t, e, i) {
    if (t < e || i.heightRelevant) {
      let n = i.widget ? i.widget.estimatedHeight : 0, s = i.widget ? i.widget.lineBreaks : 0;
      n < 0 && (n = this.oracle.lineHeight);
      let l = e - t;
      i.block ? this.addBlock(new Br(l, n, i)) : (l || s || n >= wo) && this.addLineDeco(n, s, l);
    } else
      e > t && this.span(t, e);
    this.lineEnd > -1 && this.lineEnd < this.pos && (this.lineEnd = this.oracle.doc.lineAt(this.pos).to);
  }
  enterLine() {
    if (this.lineStart > -1)
      return;
    let { from: t, to: e } = this.oracle.doc.lineAt(this.pos);
    this.lineStart = t, this.lineEnd = e, this.writtenTo < t && ((this.writtenTo < t - 1 || this.nodes[this.nodes.length - 1] == null) && this.nodes.push(this.blankContent(this.writtenTo, t - 1)), this.nodes.push(null)), this.pos > t && this.nodes.push(new it(this.pos - t, -1)), this.writtenTo = this.pos;
  }
  blankContent(t, e) {
    let i = new Y(e - t);
    return this.oracle.doc.lineAt(t).to == e && (i.flags |= 4), i;
  }
  ensureLine() {
    this.enterLine();
    let t = this.nodes.length ? this.nodes[this.nodes.length - 1] : null;
    if (t instanceof it)
      return t;
    let e = new it(0, -1);
    return this.nodes.push(e), e;
  }
  addBlock(t) {
    this.enterLine();
    let e = t.deco;
    e && e.startSide > 0 && !this.isCovered && this.ensureLine(), this.nodes.push(t), this.writtenTo = this.pos = this.pos + t.length, e && e.endSide > 0 && (this.covering = t);
  }
  addLineDeco(t, e, i) {
    let n = this.ensureLine();
    n.length += i, n.collapsed += i, n.widgetHeight = Math.max(n.widgetHeight, t), n.breaks += e, this.writtenTo = this.pos = this.pos + i;
  }
  finish(t) {
    let e = this.nodes.length == 0 ? null : this.nodes[this.nodes.length - 1];
    this.lineStart > -1 && !(e instanceof it) && !this.isCovered ? this.nodes.push(new it(0, -1)) : (this.writtenTo < this.pos || e == null) && this.nodes.push(this.blankContent(this.writtenTo, this.pos));
    let i = t;
    for (let n of this.nodes)
      n instanceof it && n.updateHeight(this.oracle, i), i += n ? n.length : 1;
    return this.nodes;
  }
  // Always called with a region that on both sides either stretches
  // to a line break or the end of the document.
  // The returned array uses null to indicate line breaks, but never
  // starts or ends in a line break, or has multiple line breaks next
  // to each other.
  static build(t, e, i, n) {
    let s = new gn(i, t);
    return W.spans(e, i, n, s, 0), s.finish(i);
  }
}
function bo(r, t, e) {
  let i = new xo();
  return W.compare(r, t, e, i, 0), i.changes;
}
class xo {
  constructor() {
    this.changes = [];
  }
  compareRange() {
  }
  comparePoint(t, e, i, n) {
    (t < e || i && i.heightRelevant || n && n.heightRelevant) && $i(t, e, this.changes, 5);
  }
}
function ko(r, t) {
  let e = r.getBoundingClientRect(), i = r.ownerDocument, n = i.defaultView || window, s = Math.max(0, e.left), l = Math.min(n.innerWidth, e.right), o = Math.max(0, e.top), h = Math.min(n.innerHeight, e.bottom);
  for (let a = r.parentNode; a && a != i.body; )
    if (a.nodeType == 1) {
      let f = a, c = window.getComputedStyle(f);
      if ((f.scrollHeight > f.clientHeight || f.scrollWidth > f.clientWidth) && c.overflow != "visible") {
        let u = f.getBoundingClientRect();
        s = Math.max(s, u.left), l = Math.min(l, u.right), o = Math.max(o, u.top), h = a == r.parentNode ? u.bottom : Math.min(h, u.bottom);
      }
      a = c.position == "absolute" || c.position == "fixed" ? f.offsetParent : f.parentNode;
    } else if (a.nodeType == 11)
      a = a.host;
    else
      break;
  return {
    left: s - e.left,
    right: Math.max(s, l) - e.left,
    top: o - (e.top + t),
    bottom: Math.max(o, h) - (e.top + t)
  };
}
function vo(r, t) {
  let e = r.getBoundingClientRect();
  return {
    left: 0,
    right: e.right - e.left,
    top: t,
    bottom: e.bottom - (e.top + t)
  };
}
class bi {
  constructor(t, e, i) {
    this.from = t, this.to = e, this.size = i;
  }
  static same(t, e) {
    if (t.length != e.length)
      return !1;
    for (let i = 0; i < t.length; i++) {
      let n = t[i], s = e[i];
      if (n.from != s.from || n.to != s.to || n.size != s.size)
        return !1;
    }
    return !0;
  }
  draw(t, e) {
    return H.replace({
      widget: new So(this.size * (e ? t.scaleY : t.scaleX), e)
    }).range(this.from, this.to);
  }
}
class So extends De {
  constructor(t, e) {
    super(), this.size = t, this.vertical = e;
  }
  eq(t) {
    return t.size == this.size && t.vertical == this.vertical;
  }
  toDOM() {
    let t = document.createElement("div");
    return this.vertical ? t.style.height = this.size + "px" : (t.style.width = this.size + "px", t.style.height = "2px", t.style.display = "inline-block"), t;
  }
  get estimatedHeight() {
    return this.vertical ? this.size : -1;
  }
}
class ls {
  constructor(t) {
    this.state = t, this.pixelViewport = { left: 0, right: window.innerWidth, top: 0, bottom: 0 }, this.inView = !0, this.paddingTop = 0, this.paddingBottom = 0, this.contentDOMWidth = 0, this.contentDOMHeight = 0, this.editorHeight = 0, this.editorWidth = 0, this.scrollTop = 0, this.scrolledToBottom = !0, this.scaleX = 1, this.scaleY = 1, this.scrollAnchorPos = 0, this.scrollAnchorHeight = -1, this.scaler = os, this.scrollTarget = null, this.printing = !1, this.mustMeasureContent = !0, this.defaultTextDirection = Q.LTR, this.visibleRanges = [], this.mustEnforceCursorAssoc = !1;
    let e = t.facet(un).some((i) => typeof i != "function" && i.class == "cm-lineWrapping");
    this.heightOracle = new po(e), this.stateDeco = t.facet(Ce).filter((i) => typeof i != "function"), this.heightMap = tt.empty().applyChanges(this.stateDeco, R.empty, this.heightOracle.setDoc(t.doc), [new rt(0, 0, 0, t.doc.length)]), this.viewport = this.getViewport(0, null), this.updateViewportLines(), this.updateForViewport(), this.lineGaps = this.ensureLineGaps([]), this.lineGapDeco = H.set(this.lineGaps.map((i) => i.draw(this, !1))), this.computeVisibleRanges();
  }
  updateForViewport() {
    let t = [this.viewport], { main: e } = this.state.selection;
    for (let i = 0; i <= 1; i++) {
      let n = i ? e.head : e.anchor;
      if (!t.some(({ from: s, to: l }) => n >= s && n <= l)) {
        let { from: s, to: l } = this.lineBlockAt(n);
        t.push(new He(s, l));
      }
    }
    this.viewports = t.sort((i, n) => i.from - n.from), this.scaler = this.heightMap.height <= 7e6 ? os : new Mo(this.heightOracle, this.heightMap, this.viewports);
  }
  updateViewportLines() {
    this.viewportLines = [], this.heightMap.forEachLine(this.viewport.from, this.viewport.to, this.heightOracle.setDoc(this.state.doc), 0, 0, (t) => {
      this.viewportLines.push(this.scaler.scale == 1 ? t : me(t, this.scaler));
    });
  }
  update(t, e = null) {
    this.state = t.state;
    let i = this.stateDeco;
    this.stateDeco = this.state.facet(Ce).filter((f) => typeof f != "function");
    let n = t.changedRanges, s = rt.extendWithRanges(n, bo(i, this.stateDeco, t ? t.changes : j.empty(this.state.doc.length))), l = this.heightMap.height, o = this.scrolledToBottom ? null : this.scrollAnchorAt(this.scrollTop);
    this.heightMap = this.heightMap.applyChanges(this.stateDeco, t.startState.doc, this.heightOracle.setDoc(this.state.doc), s), this.heightMap.height != l && (t.flags |= 2), o ? (this.scrollAnchorPos = t.changes.mapPos(o.from, -1), this.scrollAnchorHeight = o.top) : (this.scrollAnchorPos = -1, this.scrollAnchorHeight = this.heightMap.height);
    let h = s.length ? this.mapViewport(this.viewport, t.changes) : this.viewport;
    (e && (e.range.head < h.from || e.range.head > h.to) || !this.viewportIsAppropriate(h)) && (h = this.getViewport(0, e));
    let a = !t.changes.empty || t.flags & 2 || h.from != this.viewport.from || h.to != this.viewport.to;
    this.viewport = h, this.updateForViewport(), a && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(this.mapLineGaps(this.lineGaps, t.changes))), t.flags |= this.computeVisibleRanges(), e && (this.scrollTarget = e), !this.mustEnforceCursorAssoc && t.selectionSet && t.view.lineWrapping && t.state.selection.main.empty && t.state.selection.main.assoc && !t.state.facet(Sl) && (this.mustEnforceCursorAssoc = !0);
  }
  measure(t) {
    let e = t.contentDOM, i = window.getComputedStyle(e), n = this.heightOracle, s = i.whiteSpace;
    this.defaultTextDirection = i.direction == "rtl" ? Q.RTL : Q.LTR;
    let l = this.heightOracle.mustRefreshForWrapping(s), o = e.getBoundingClientRect(), h = l || this.mustMeasureContent || this.contentDOMHeight != o.height;
    this.contentDOMHeight = o.height, this.mustMeasureContent = !1;
    let a = 0, f = 0;
    if (o.width && o.height) {
      let b = o.width / e.offsetWidth, A = o.height / e.offsetHeight;
      (b > 0.995 && b < 1.005 || !isFinite(b) || Math.abs(o.width - e.offsetWidth) < 1) && (b = 1), (A > 0.995 && A < 1.005 || !isFinite(A) || Math.abs(o.height - e.offsetHeight) < 1) && (A = 1), (this.scaleX != b || this.scaleY != A) && (this.scaleX = b, this.scaleY = A, a |= 8, l = h = !0);
    }
    let c = (parseInt(i.paddingTop) || 0) * this.scaleY, u = (parseInt(i.paddingBottom) || 0) * this.scaleY;
    (this.paddingTop != c || this.paddingBottom != u) && (this.paddingTop = c, this.paddingBottom = u, a |= 10), this.editorWidth != t.scrollDOM.clientWidth && (n.lineWrapping && (h = !0), this.editorWidth = t.scrollDOM.clientWidth, a |= 8);
    let d = t.scrollDOM.scrollTop * this.scaleY;
    this.scrollTop != d && (this.scrollAnchorHeight = -1, this.scrollTop = d), this.scrolledToBottom = Js(t.scrollDOM);
    let g = (this.printing ? vo : ko)(e, this.paddingTop), m = g.top - this.pixelViewport.top, p = g.bottom - this.pixelViewport.bottom;
    this.pixelViewport = g;
    let y = this.pixelViewport.bottom > this.pixelViewport.top && this.pixelViewport.right > this.pixelViewport.left;
    if (y != this.inView && (this.inView = y, y && (h = !0)), !this.inView && !this.scrollTarget)
      return 0;
    let S = o.width;
    if ((this.contentDOMWidth != S || this.editorHeight != t.scrollDOM.clientHeight) && (this.contentDOMWidth = o.width, this.editorHeight = t.scrollDOM.clientHeight, a |= 8), h) {
      let b = t.docView.measureVisibleLineHeights(this.viewport);
      if (n.mustRefreshForHeights(b) && (l = !0), l || n.lineWrapping && Math.abs(S - this.contentDOMWidth) > n.charWidth) {
        let { lineHeight: A, charWidth: v, textHeight: P } = t.docView.measureTextSize();
        l = A > 0 && n.refresh(s, A, v, P, S / v, b), l && (t.docView.minWidth = 0, a |= 8);
      }
      m > 0 && p > 0 ? f = Math.max(m, p) : m < 0 && p < 0 && (f = Math.min(m, p)), n.heightChanged = !1;
      for (let A of this.viewports) {
        let v = A.from == this.viewport.from ? b : t.docView.measureVisibleLineHeights(A);
        this.heightMap = (l ? tt.empty().applyChanges(this.stateDeco, R.empty, this.heightOracle, [new rt(0, 0, 0, t.state.doc.length)]) : this.heightMap).updateHeight(n, 0, l, new mo(A.from, v));
      }
      n.heightChanged && (a |= 2);
    }
    let C = !this.viewportIsAppropriate(this.viewport, f) || this.scrollTarget && (this.scrollTarget.range.head < this.viewport.from || this.scrollTarget.range.head > this.viewport.to);
    return C && (this.viewport = this.getViewport(f, this.scrollTarget)), this.updateForViewport(), (a & 2 || C) && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(l ? [] : this.lineGaps, t)), a |= this.computeVisibleRanges(), this.mustEnforceCursorAssoc && (this.mustEnforceCursorAssoc = !1, t.docView.enforceCursorAssoc()), a;
  }
  get visibleTop() {
    return this.scaler.fromDOM(this.pixelViewport.top);
  }
  get visibleBottom() {
    return this.scaler.fromDOM(this.pixelViewport.bottom);
  }
  getViewport(t, e) {
    let i = 0.5 - Math.max(-0.5, Math.min(0.5, t / 1e3 / 2)), n = this.heightMap, s = this.heightOracle, { visibleTop: l, visibleBottom: o } = this, h = new He(n.lineAt(l - i * 1e3, N.ByHeight, s, 0, 0).from, n.lineAt(o + (1 - i) * 1e3, N.ByHeight, s, 0, 0).to);
    if (e) {
      let { head: a } = e.range;
      if (a < h.from || a > h.to) {
        let f = Math.min(this.editorHeight, this.pixelViewport.bottom - this.pixelViewport.top), c = n.lineAt(a, N.ByPos, s, 0, 0), u;
        e.y == "center" ? u = (c.top + c.bottom) / 2 - f / 2 : e.y == "start" || e.y == "nearest" && a < h.from ? u = c.top : u = c.bottom - f, h = new He(n.lineAt(u - 1e3 / 2, N.ByHeight, s, 0, 0).from, n.lineAt(u + f + 1e3 / 2, N.ByHeight, s, 0, 0).to);
      }
    }
    return h;
  }
  mapViewport(t, e) {
    let i = e.mapPos(t.from, -1), n = e.mapPos(t.to, 1);
    return new He(this.heightMap.lineAt(i, N.ByPos, this.heightOracle, 0, 0).from, this.heightMap.lineAt(n, N.ByPos, this.heightOracle, 0, 0).to);
  }
  // Checks if a given viewport covers the visible part of the
  // document and not too much beyond that.
  viewportIsAppropriate({ from: t, to: e }, i = 0) {
    if (!this.inView)
      return !0;
    let { top: n } = this.heightMap.lineAt(t, N.ByPos, this.heightOracle, 0, 0), { bottom: s } = this.heightMap.lineAt(e, N.ByPos, this.heightOracle, 0, 0), { visibleTop: l, visibleBottom: o } = this;
    return (t == 0 || n <= l - Math.max(10, Math.min(
      -i,
      250
      /* VP.MaxCoverMargin */
    ))) && (e == this.state.doc.length || s >= o + Math.max(10, Math.min(
      i,
      250
      /* VP.MaxCoverMargin */
    ))) && n > l - 2 * 1e3 && s < o + 2 * 1e3;
  }
  mapLineGaps(t, e) {
    if (!t.length || e.empty)
      return t;
    let i = [];
    for (let n of t)
      e.touchesRange(n.from, n.to) || i.push(new bi(e.mapPos(n.from), e.mapPos(n.to), n.size));
    return i;
  }
  // Computes positions in the viewport where the start or end of a
  // line should be hidden, trying to reuse existing line gaps when
  // appropriate to avoid unneccesary redraws.
  // Uses crude character-counting for the positioning and sizing,
  // since actual DOM coordinates aren't always available and
  // predictable. Relies on generous margins (see LG.Margin) to hide
  // the artifacts this might produce from the user.
  ensureLineGaps(t, e) {
    let i = this.heightOracle.lineWrapping, n = i ? 1e4 : 2e3, s = n >> 1, l = n << 1;
    if (this.defaultTextDirection != Q.LTR && !i)
      return [];
    let o = [], h = (a, f, c, u) => {
      if (f - a < s)
        return;
      let d = this.state.selection.main, g = [d.from];
      d.empty || g.push(d.to);
      for (let p of g)
        if (p > a && p < f) {
          h(a, p - 10, c, u), h(p + 10, f, c, u);
          return;
        }
      let m = Co(t, (p) => p.from >= c.from && p.to <= c.to && Math.abs(p.from - a) < s && Math.abs(p.to - f) < s && !g.some((y) => p.from < y && p.to > y));
      if (!m) {
        if (f < c.to && e && i && e.visibleRanges.some((p) => p.from <= f && p.to >= f)) {
          let p = e.moveToLineBoundary(x.cursor(f), !1, !0).head;
          p > a && (f = p);
        }
        m = new bi(a, f, this.gapSize(c, a, f, u));
      }
      o.push(m);
    };
    for (let a of this.viewportLines) {
      if (a.length < l)
        continue;
      let f = Ao(a.from, a.to, this.stateDeco);
      if (f.total < l)
        continue;
      let c = this.scrollTarget ? this.scrollTarget.range.head : null, u, d;
      if (i) {
        let g = n / this.heightOracle.lineLength * this.heightOracle.lineHeight, m, p;
        if (c != null) {
          let y = Ve(f, c), S = ((this.visibleBottom - this.visibleTop) / 2 + g) / a.height;
          m = y - S, p = y + S;
        } else
          m = (this.visibleTop - a.top - g) / a.height, p = (this.visibleBottom - a.top + g) / a.height;
        u = Fe(f, m), d = Fe(f, p);
      } else {
        let g = f.total * this.heightOracle.charWidth, m = n * this.heightOracle.charWidth, p, y;
        if (c != null) {
          let S = Ve(f, c), C = ((this.pixelViewport.right - this.pixelViewport.left) / 2 + m) / g;
          p = S - C, y = S + C;
        } else
          p = (this.pixelViewport.left - m) / g, y = (this.pixelViewport.right + m) / g;
        u = Fe(f, p), d = Fe(f, y);
      }
      u > a.from && h(a.from, u, a, f), d < a.to && h(d, a.to, a, f);
    }
    return o;
  }
  gapSize(t, e, i, n) {
    let s = Ve(n, i) - Ve(n, e);
    return this.heightOracle.lineWrapping ? t.height * s : n.total * this.heightOracle.charWidth * s;
  }
  updateLineGaps(t) {
    bi.same(t, this.lineGaps) || (this.lineGaps = t, this.lineGapDeco = H.set(t.map((e) => e.draw(this, this.heightOracle.lineWrapping))));
  }
  computeVisibleRanges() {
    let t = this.stateDeco;
    this.lineGaps.length && (t = t.concat(this.lineGapDeco));
    let e = [];
    W.spans(t, this.viewport.from, this.viewport.to, {
      span(n, s) {
        e.push({ from: n, to: s });
      },
      point() {
      }
    }, 20);
    let i = e.length != this.visibleRanges.length || this.visibleRanges.some((n, s) => n.from != e[s].from || n.to != e[s].to);
    return this.visibleRanges = e, i ? 4 : 0;
  }
  lineBlockAt(t) {
    return t >= this.viewport.from && t <= this.viewport.to && this.viewportLines.find((e) => e.from <= t && e.to >= t) || me(this.heightMap.lineAt(t, N.ByPos, this.heightOracle, 0, 0), this.scaler);
  }
  lineBlockAtHeight(t) {
    return me(this.heightMap.lineAt(this.scaler.fromDOM(t), N.ByHeight, this.heightOracle, 0, 0), this.scaler);
  }
  scrollAnchorAt(t) {
    let e = this.lineBlockAtHeight(t + 8);
    return e.from >= this.viewport.from || this.viewportLines[0].top - t > 200 ? e : this.viewportLines[0];
  }
  elementAtHeight(t) {
    return me(this.heightMap.blockAt(this.scaler.fromDOM(t), this.heightOracle, 0, 0), this.scaler);
  }
  get docHeight() {
    return this.scaler.toDOM(this.heightMap.height);
  }
  get contentHeight() {
    return this.docHeight + this.paddingTop + this.paddingBottom;
  }
}
class He {
  constructor(t, e) {
    this.from = t, this.to = e;
  }
}
function Ao(r, t, e) {
  let i = [], n = r, s = 0;
  return W.spans(e, r, t, {
    span() {
    },
    point(l, o) {
      l > n && (i.push({ from: n, to: l }), s += l - n), n = o;
    }
  }, 20), n < t && (i.push({ from: n, to: t }), s += t - n), { total: s, ranges: i };
}
function Fe({ total: r, ranges: t }, e) {
  if (e <= 0)
    return t[0].from;
  if (e >= 1)
    return t[t.length - 1].to;
  let i = Math.floor(r * e);
  for (let n = 0; ; n++) {
    let { from: s, to: l } = t[n], o = l - s;
    if (i <= o)
      return s + i;
    i -= o;
  }
}
function Ve(r, t) {
  let e = 0;
  for (let { from: i, to: n } of r.ranges) {
    if (t <= n) {
      e += t - i;
      break;
    }
    e += n - i;
  }
  return e / r.total;
}
function Co(r, t) {
  for (let e of r)
    if (t(e))
      return e;
}
const os = {
  toDOM(r) {
    return r;
  },
  fromDOM(r) {
    return r;
  },
  scale: 1
};
class Mo {
  constructor(t, e, i) {
    let n = 0, s = 0, l = 0;
    this.viewports = i.map(({ from: o, to: h }) => {
      let a = e.lineAt(o, N.ByPos, t, 0, 0).top, f = e.lineAt(h, N.ByPos, t, 0, 0).bottom;
      return n += f - a, { from: o, to: h, top: a, bottom: f, domTop: 0, domBottom: 0 };
    }), this.scale = (7e6 - n) / (e.height - n);
    for (let o of this.viewports)
      o.domTop = l + (o.top - s) * this.scale, l = o.domBottom = o.domTop + (o.bottom - o.top), s = o.bottom;
  }
  toDOM(t) {
    for (let e = 0, i = 0, n = 0; ; e++) {
      let s = e < this.viewports.length ? this.viewports[e] : null;
      if (!s || t < s.top)
        return n + (t - i) * this.scale;
      if (t <= s.bottom)
        return s.domTop + (t - s.top);
      i = s.bottom, n = s.domBottom;
    }
  }
  fromDOM(t) {
    for (let e = 0, i = 0, n = 0; ; e++) {
      let s = e < this.viewports.length ? this.viewports[e] : null;
      if (!s || t < s.domTop)
        return i + (t - n) / this.scale;
      if (t <= s.domBottom)
        return s.top + (t - s.domTop);
      i = s.bottom, n = s.domBottom;
    }
  }
}
function me(r, t) {
  if (t.scale == 1)
    return r;
  let e = t.toDOM(r.top), i = t.toDOM(r.bottom);
  return new yt(r.from, r.length, e, i - e, Array.isArray(r._content) ? r._content.map((n) => me(n, t)) : r._content);
}
const We = /* @__PURE__ */ O.define({ combine: (r) => r.join(" ") }), Zi = /* @__PURE__ */ O.define({ combine: (r) => r.indexOf(!0) > -1 }), Qi = /* @__PURE__ */ ie.newName(), Rr = /* @__PURE__ */ ie.newName(), Er = /* @__PURE__ */ ie.newName(), Lr = { "&light": "." + Rr, "&dark": "." + Er };
function tn(r, t, e) {
  return new ie(t, {
    finish(i) {
      return /&/.test(i) ? i.replace(/&\w*/, (n) => {
        if (n == "&")
          return r;
        if (!e || !e[n])
          throw new RangeError(`Unsupported selector: ${n}`);
        return e[n];
      }) : r + " " + i;
    }
  });
}
const Oo = /* @__PURE__ */ tn("." + Qi, {
  "&": {
    position: "relative !important",
    boxSizing: "border-box",
    "&.cm-focused": {
      // Provide a simple default outline to make sure a focused
      // editor is visually distinct. Can't leave the default behavior
      // because that will apply to the content element, which is
      // inside the scrollable container and doesn't include the
      // gutters. We also can't use an 'auto' outline, since those
      // are, for some reason, drawn behind the element content, which
      // will cause things like the active line background to cover
      // the outline (#297).
      outline: "1px dotted #212121"
    },
    display: "flex !important",
    flexDirection: "column"
  },
  ".cm-scroller": {
    display: "flex !important",
    alignItems: "flex-start !important",
    fontFamily: "monospace",
    lineHeight: 1.4,
    height: "100%",
    overflowX: "auto",
    position: "relative",
    zIndex: 0
  },
  ".cm-content": {
    margin: 0,
    flexGrow: 2,
    flexShrink: 0,
    display: "block",
    whiteSpace: "pre",
    wordWrap: "normal",
    boxSizing: "border-box",
    padding: "4px 0",
    outline: "none",
    "&[contenteditable=true]": {
      WebkitUserModify: "read-write-plaintext-only"
    }
  },
  ".cm-lineWrapping": {
    whiteSpace_fallback: "pre-wrap",
    whiteSpace: "break-spaces",
    wordBreak: "break-word",
    overflowWrap: "anywhere",
    flexShrink: 1
  },
  "&light .cm-content": { caretColor: "black" },
  "&dark .cm-content": { caretColor: "white" },
  ".cm-line": {
    display: "block",
    padding: "0 2px 0 6px"
  },
  ".cm-layer": {
    position: "absolute",
    left: 0,
    top: 0,
    contain: "size style",
    "& > *": {
      position: "absolute"
    }
  },
  "&light .cm-selectionBackground": {
    background: "#d9d9d9"
  },
  "&dark .cm-selectionBackground": {
    background: "#222"
  },
  "&light.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
    background: "#d7d4f0"
  },
  "&dark.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
    background: "#233"
  },
  ".cm-cursorLayer": {
    pointerEvents: "none"
  },
  "&.cm-focused > .cm-scroller > .cm-cursorLayer": {
    animation: "steps(1) cm-blink 1.2s infinite"
  },
  // Two animations defined so that we can switch between them to
  // restart the animation without forcing another style
  // recomputation.
  "@keyframes cm-blink": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
  "@keyframes cm-blink2": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
  ".cm-cursor, .cm-dropCursor": {
    borderLeft: "1.2px solid black",
    marginLeft: "-0.6px",
    pointerEvents: "none"
  },
  ".cm-cursor": {
    display: "none"
  },
  "&dark .cm-cursor": {
    borderLeftColor: "#444"
  },
  ".cm-dropCursor": {
    position: "absolute"
  },
  "&.cm-focused > .cm-scroller > .cm-cursorLayer .cm-cursor": {
    display: "block"
  },
  "&light .cm-activeLine": { backgroundColor: "#cceeff44" },
  "&dark .cm-activeLine": { backgroundColor: "#99eeff33" },
  "&light .cm-specialChar": { color: "red" },
  "&dark .cm-specialChar": { color: "#f78" },
  ".cm-gutters": {
    flexShrink: 0,
    display: "flex",
    height: "100%",
    boxSizing: "border-box",
    insetInlineStart: 0,
    zIndex: 200
  },
  "&light .cm-gutters": {
    backgroundColor: "#f5f5f5",
    color: "#6c6c6c",
    borderRight: "1px solid #ddd"
  },
  "&dark .cm-gutters": {
    backgroundColor: "#333338",
    color: "#ccc"
  },
  ".cm-gutter": {
    display: "flex !important",
    flexDirection: "column",
    flexShrink: 0,
    boxSizing: "border-box",
    minHeight: "100%",
    overflow: "hidden"
  },
  ".cm-gutterElement": {
    boxSizing: "border-box"
  },
  ".cm-lineNumbers .cm-gutterElement": {
    padding: "0 3px 0 5px",
    minWidth: "20px",
    textAlign: "right",
    whiteSpace: "nowrap"
  },
  "&light .cm-activeLineGutter": {
    backgroundColor: "#e2f2ff"
  },
  "&dark .cm-activeLineGutter": {
    backgroundColor: "#222227"
  },
  ".cm-panels": {
    boxSizing: "border-box",
    position: "sticky",
    left: 0,
    right: 0
  },
  "&light .cm-panels": {
    backgroundColor: "#f5f5f5",
    color: "black"
  },
  "&light .cm-panels-top": {
    borderBottom: "1px solid #ddd"
  },
  "&light .cm-panels-bottom": {
    borderTop: "1px solid #ddd"
  },
  "&dark .cm-panels": {
    backgroundColor: "#333338",
    color: "white"
  },
  ".cm-tab": {
    display: "inline-block",
    overflow: "hidden",
    verticalAlign: "bottom"
  },
  ".cm-widgetBuffer": {
    verticalAlign: "text-top",
    height: "1em",
    width: 0,
    display: "inline"
  },
  ".cm-placeholder": {
    color: "#888",
    display: "inline-block",
    verticalAlign: "top"
  },
  ".cm-highlightSpace:before": {
    content: "attr(data-display)",
    position: "absolute",
    pointerEvents: "none",
    color: "#888"
  },
  ".cm-highlightTab": {
    backgroundImage: `url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" width="200" height="20"><path stroke="%23888" stroke-width="1" fill="none" d="M1 10H196L190 5M190 15L196 10M197 4L197 16"/></svg>')`,
    backgroundSize: "auto 100%",
    backgroundPosition: "right 90%",
    backgroundRepeat: "no-repeat"
  },
  ".cm-trailingSpace": {
    backgroundColor: "#ff332255"
  },
  ".cm-button": {
    verticalAlign: "middle",
    color: "inherit",
    fontSize: "70%",
    padding: ".2em 1em",
    borderRadius: "1px"
  },
  "&light .cm-button": {
    backgroundImage: "linear-gradient(#eff1f5, #d9d9df)",
    border: "1px solid #888",
    "&:active": {
      backgroundImage: "linear-gradient(#b4b4b4, #d0d3d6)"
    }
  },
  "&dark .cm-button": {
    backgroundImage: "linear-gradient(#393939, #111)",
    border: "1px solid #888",
    "&:active": {
      backgroundImage: "linear-gradient(#111, #333)"
    }
  },
  ".cm-textfield": {
    verticalAlign: "middle",
    color: "inherit",
    fontSize: "70%",
    border: "1px solid silver",
    padding: ".2em .5em"
  },
  "&light .cm-textfield": {
    backgroundColor: "white"
  },
  "&dark .cm-textfield": {
    border: "1px solid #555",
    backgroundColor: "inherit"
  }
}, Lr), ye = "￿";
class Do {
  constructor(t, e) {
    this.points = t, this.text = "", this.lineSeparator = e.facet(E.lineSeparator);
  }
  append(t) {
    this.text += t;
  }
  lineBreak() {
    this.text += ye;
  }
  readRange(t, e) {
    if (!t)
      return this;
    let i = t.parentNode;
    for (let n = t; ; ) {
      this.findPointBefore(i, n);
      let s = this.text.length;
      this.readNode(n);
      let l = n.nextSibling;
      if (l == e)
        break;
      let o = I.get(n), h = I.get(l);
      (o && h ? o.breakAfter : (o ? o.breakAfter : hs(n)) || hs(l) && (n.nodeName != "BR" || n.cmIgnore) && this.text.length > s) && this.lineBreak(), n = l;
    }
    return this.findPointBefore(i, e), this;
  }
  readTextNode(t) {
    let e = t.nodeValue;
    for (let i of this.points)
      i.node == t && (i.pos = this.text.length + Math.min(i.offset, e.length));
    for (let i = 0, n = this.lineSeparator ? null : /\r\n?|\n/g; ; ) {
      let s = -1, l = 1, o;
      if (this.lineSeparator ? (s = e.indexOf(this.lineSeparator, i), l = this.lineSeparator.length) : (o = n.exec(e)) && (s = o.index, l = o[0].length), this.append(e.slice(i, s < 0 ? e.length : s)), s < 0)
        break;
      if (this.lineBreak(), l > 1)
        for (let h of this.points)
          h.node == t && h.pos > this.text.length && (h.pos -= l - 1);
      i = s + l;
    }
  }
  readNode(t) {
    if (t.cmIgnore)
      return;
    let e = I.get(t), i = e && e.overrideDOMText;
    if (i != null) {
      this.findPointInside(t, i.length);
      for (let n = i.iter(); !n.next().done; )
        n.lineBreak ? this.lineBreak() : this.append(n.value);
    } else
      t.nodeType == 3 ? this.readTextNode(t) : t.nodeName == "BR" ? t.nextSibling && this.lineBreak() : t.nodeType == 1 && this.readRange(t.firstChild, null);
  }
  findPointBefore(t, e) {
    for (let i of this.points)
      i.node == t && t.childNodes[i.offset] == e && (i.pos = this.text.length);
  }
  findPointInside(t, e) {
    for (let i of this.points)
      (t.nodeType == 3 ? i.node == t : t.contains(i.node)) && (i.pos = this.text.length + (To(t, i.node, i.offset) ? e : 0));
  }
}
function To(r, t, e) {
  for (; ; ) {
    if (!t || e < Mt(t))
      return !1;
    if (t == r)
      return !0;
    e = Ae(t) + 1, t = t.parentNode;
  }
}
function hs(r) {
  return r.nodeType == 1 && /^(DIV|P|LI|UL|OL|BLOCKQUOTE|DD|DT|H\d|SECTION|PRE)$/.test(r.nodeName);
}
class as {
  constructor(t, e) {
    this.node = t, this.offset = e, this.pos = -1;
  }
}
class Po {
  constructor(t, e, i, n) {
    this.typeOver = n, this.bounds = null, this.text = "";
    let { impreciseHead: s, impreciseAnchor: l } = t.docView;
    if (t.state.readOnly && e > -1)
      this.newSel = null;
    else if (e > -1 && (this.bounds = t.docView.domBoundsAround(e, i, 0))) {
      let o = s || l ? [] : Eo(t), h = new Do(o, t.state);
      h.readRange(this.bounds.startDOM, this.bounds.endDOM), this.text = h.text, this.newSel = Lo(o, this.bounds.from);
    } else {
      let o = t.observer.selectionRange, h = s && s.node == o.focusNode && s.offset == o.focusOffset || !Vi(t.contentDOM, o.focusNode) ? t.state.selection.main.head : t.docView.posFromDOM(o.focusNode, o.focusOffset), a = l && l.node == o.anchorNode && l.offset == o.anchorOffset || !Vi(t.contentDOM, o.anchorNode) ? t.state.selection.main.anchor : t.docView.posFromDOM(o.anchorNode, o.anchorOffset);
      this.newSel = x.single(a, h);
    }
  }
}
function Nr(r, t) {
  let e, { newSel: i } = t, n = r.state.selection.main, s = r.inputState.lastKeyTime > Date.now() - 100 ? r.inputState.lastKeyCode : -1;
  if (t.bounds) {
    let { from: l, to: o } = t.bounds, h = n.from, a = null;
    (s === 8 || k.android && t.text.length < o - l) && (h = n.to, a = "end");
    let f = Ro(r.state.doc.sliceString(l, o, ye), t.text, h - l, a);
    f && (k.chrome && s == 13 && f.toB == f.from + 2 && t.text.slice(f.from, f.toB) == ye + ye && f.toB--, e = {
      from: l + f.from,
      to: l + f.toA,
      insert: R.of(t.text.slice(f.from, f.toB).split(ye))
    });
  } else
    i && (!r.hasFocus && r.state.facet(ci) || i.main.eq(n)) && (i = null);
  if (!e && !i)
    return !1;
  if (!e && t.typeOver && !n.empty && i && i.main.empty ? e = { from: n.from, to: n.to, insert: r.state.doc.slice(n.from, n.to) } : e && e.from >= n.from && e.to <= n.to && (e.from != n.from || e.to != n.to) && n.to - n.from - (e.to - e.from) <= 4 ? e = {
    from: n.from,
    to: n.to,
    insert: r.state.doc.slice(n.from, e.from).append(e.insert).append(r.state.doc.slice(e.to, n.to))
  } : (k.mac || k.android) && e && e.from == e.to && e.from == n.head - 1 && /^\. ?$/.test(e.insert.toString()) && r.contentDOM.getAttribute("autocorrect") == "off" ? (i && e.insert.length == 2 && (i = x.single(i.main.anchor - 1, i.main.head - 1)), e = { from: n.from, to: n.to, insert: R.of([" "]) }) : k.chrome && e && e.from == e.to && e.from == n.head && e.insert.toString() == `
 ` && r.lineWrapping && (i && (i = x.single(i.main.anchor - 1, i.main.head - 1)), e = { from: n.from, to: n.to, insert: R.of([" "]) }), e) {
    if (k.ios && r.inputState.flushIOSKey() || k.android && (e.from == n.from && e.to == n.to && e.insert.length == 1 && e.insert.lines == 2 && te(r.contentDOM, "Enter", 13) || (e.from == n.from - 1 && e.to == n.to && e.insert.length == 0 || s == 8 && e.insert.length < e.to - e.from && e.to > n.head) && te(r.contentDOM, "Backspace", 8) || e.from == n.from && e.to == n.to + 1 && e.insert.length == 0 && te(r.contentDOM, "Delete", 46)))
      return !0;
    let l = e.insert.toString();
    r.inputState.composing >= 0 && r.inputState.composing++;
    let o, h = () => o || (o = Bo(r, e, i));
    return r.state.facet(hr).some((a) => a(r, e.from, e.to, l, h)) || r.dispatch(h()), !0;
  } else if (i && !i.main.eq(n)) {
    let l = !1, o = "select";
    return r.inputState.lastSelectionTime > Date.now() - 50 && (r.inputState.lastSelectionOrigin == "select" && (l = !0), o = r.inputState.lastSelectionOrigin), r.dispatch({ selection: i, scrollIntoView: l, userEvent: o }), !0;
  } else
    return !1;
}
function Bo(r, t, e) {
  let i, n = r.state, s = n.selection.main;
  if (t.from >= s.from && t.to <= s.to && t.to - t.from >= (s.to - s.from) / 3 && (!e || e.main.empty && e.main.from == t.from + t.insert.length) && r.inputState.composing < 0) {
    let o = s.from < t.from ? n.sliceDoc(s.from, t.from) : "", h = s.to > t.to ? n.sliceDoc(t.to, s.to) : "";
    i = n.replaceSelection(r.state.toText(o + t.insert.sliceString(0, void 0, r.state.lineBreak) + h));
  } else {
    let o = n.changes(t), h = e && e.main.to <= o.newLength ? e.main : void 0;
    if (n.selection.ranges.length > 1 && r.inputState.composing >= 0 && t.to <= s.to && t.to >= s.to - 10) {
      let a = r.state.sliceDoc(t.from, t.to), f, c = e && xr(r, e.main.head);
      if (c) {
        let g = t.insert.length - (t.to - t.from);
        f = { from: c.from, to: c.to - g };
      } else
        f = r.state.doc.lineAt(s.head);
      let u = s.to - t.to, d = s.to - s.from;
      i = n.changeByRange((g) => {
        if (g.from == s.from && g.to == s.to)
          return { changes: o, range: h || g.map(o) };
        let m = g.to - u, p = m - a.length;
        if (g.to - g.from != d || r.state.sliceDoc(p, m) != a || // Unfortunately, there's no way to make multiple
        // changes in the same node work without aborting
        // composition, so cursors in the composition range are
        // ignored.
        g.to >= f.from && g.from <= f.to)
          return { range: g };
        let y = n.changes({ from: p, to: m, insert: t.insert }), S = g.to - s.to;
        return {
          changes: y,
          range: h ? x.range(Math.max(0, h.anchor + S), Math.max(0, h.head + S)) : g.map(y)
        };
      });
    } else
      i = {
        changes: o,
        selection: h && n.selection.replaceRange(h)
      };
  }
  let l = "input.type";
  return (r.composing || r.inputState.compositionPendingChange && r.inputState.compositionEndedAt > Date.now() - 50) && (r.inputState.compositionPendingChange = !1, l += ".compose", r.inputState.compositionFirstChange && (l += ".start", r.inputState.compositionFirstChange = !1)), n.update(i, { userEvent: l, scrollIntoView: !0 });
}
function Ro(r, t, e, i) {
  let n = Math.min(r.length, t.length), s = 0;
  for (; s < n && r.charCodeAt(s) == t.charCodeAt(s); )
    s++;
  if (s == n && r.length == t.length)
    return null;
  let l = r.length, o = t.length;
  for (; l > 0 && o > 0 && r.charCodeAt(l - 1) == t.charCodeAt(o - 1); )
    l--, o--;
  if (i == "end") {
    let h = Math.max(0, s - Math.min(l, o));
    e -= l + h - s;
  }
  if (l < s && r.length < t.length) {
    let h = e <= s && e >= l ? s - e : 0;
    s -= h, o = s + (o - l), l = s;
  } else if (o < s) {
    let h = e <= s && e >= o ? s - e : 0;
    s -= h, l = s + (l - o), o = s;
  }
  return { from: s, toA: l, toB: o };
}
function Eo(r) {
  let t = [];
  if (r.root.activeElement != r.contentDOM)
    return t;
  let { anchorNode: e, anchorOffset: i, focusNode: n, focusOffset: s } = r.observer.selectionRange;
  return e && (t.push(new as(e, i)), (n != e || s != i) && t.push(new as(n, s))), t;
}
function Lo(r, t) {
  if (r.length == 0)
    return null;
  let e = r[0].pos, i = r.length == 2 ? r[1].pos : e;
  return e > -1 && i > -1 ? x.single(e + t, i + t) : null;
}
const No = {
  childList: !0,
  characterData: !0,
  subtree: !0,
  attributes: !0,
  characterDataOldValue: !0
}, xi = k.ie && k.ie_version <= 11;
class Io {
  constructor(t) {
    this.view = t, this.active = !1, this.selectionRange = new pl(), this.selectionChanged = !1, this.delayedFlush = -1, this.resizeTimeout = -1, this.queue = [], this.delayedAndroidKey = null, this.flushingAndroidKey = -1, this.lastChange = 0, this.scrollTargets = [], this.intersection = null, this.resizeScroll = null, this.resizeContent = null, this.intersecting = !1, this.gapIntersection = null, this.gaps = [], this.parentCheck = -1, this.dom = t.contentDOM, this.observer = new MutationObserver((e) => {
      for (let i of e)
        this.queue.push(i);
      (k.ie && k.ie_version <= 11 || k.ios && t.composing) && e.some((i) => i.type == "childList" && i.removedNodes.length || i.type == "characterData" && i.oldValue.length > i.target.nodeValue.length) ? this.flushSoon() : this.flush();
    }), xi && (this.onCharData = (e) => {
      this.queue.push({
        target: e.target,
        type: "characterData",
        oldValue: e.prevValue
      }), this.flushSoon();
    }), this.onSelectionChange = this.onSelectionChange.bind(this), this.onResize = this.onResize.bind(this), this.onPrint = this.onPrint.bind(this), this.onScroll = this.onScroll.bind(this), typeof ResizeObserver == "function" && (this.resizeScroll = new ResizeObserver(() => {
      var e;
      ((e = this.view.docView) === null || e === void 0 ? void 0 : e.lastUpdate) < Date.now() - 75 && this.onResize();
    }), this.resizeScroll.observe(t.scrollDOM), this.resizeContent = new ResizeObserver(() => this.view.requestMeasure()), this.resizeContent.observe(t.contentDOM)), this.addWindowListeners(this.win = t.win), this.start(), typeof IntersectionObserver == "function" && (this.intersection = new IntersectionObserver((e) => {
      this.parentCheck < 0 && (this.parentCheck = setTimeout(this.listenForScroll.bind(this), 1e3)), e.length > 0 && e[e.length - 1].intersectionRatio > 0 != this.intersecting && (this.intersecting = !this.intersecting, this.intersecting != this.view.inView && this.onScrollChanged(document.createEvent("Event")));
    }, { threshold: [0, 1e-3] }), this.intersection.observe(this.dom), this.gapIntersection = new IntersectionObserver((e) => {
      e.length > 0 && e[e.length - 1].intersectionRatio > 0 && this.onScrollChanged(document.createEvent("Event"));
    }, {})), this.listenForScroll(), this.readSelectionRange();
  }
  onScrollChanged(t) {
    this.view.inputState.runHandlers("scroll", t), this.intersecting && this.view.measure();
  }
  onScroll(t) {
    this.intersecting && this.flush(!1), this.onScrollChanged(t);
  }
  onResize() {
    this.resizeTimeout < 0 && (this.resizeTimeout = setTimeout(() => {
      this.resizeTimeout = -1, this.view.requestMeasure();
    }, 50));
  }
  onPrint() {
    this.view.viewState.printing = !0, this.view.measure(), setTimeout(() => {
      this.view.viewState.printing = !1, this.view.requestMeasure();
    }, 500);
  }
  updateGaps(t) {
    if (this.gapIntersection && (t.length != this.gaps.length || this.gaps.some((e, i) => e != t[i]))) {
      this.gapIntersection.disconnect();
      for (let e of t)
        this.gapIntersection.observe(e);
      this.gaps = t;
    }
  }
  onSelectionChange(t) {
    let e = this.selectionChanged;
    if (!this.readSelectionRange() || this.delayedAndroidKey)
      return;
    let { view: i } = this, n = this.selectionRange;
    if (i.state.facet(ci) ? i.root.activeElement != this.dom : !Je(i.dom, n))
      return;
    let s = n.anchorNode && i.docView.nearest(n.anchorNode);
    if (s && s.ignoreEvent(t)) {
      e || (this.selectionChanged = !1);
      return;
    }
    (k.ie && k.ie_version <= 11 || k.android && k.chrome) && !i.state.selection.main.empty && // (Selection.isCollapsed isn't reliable on IE)
    n.focusNode && ii(n.focusNode, n.focusOffset, n.anchorNode, n.anchorOffset) ? this.flushSoon() : this.flush(!1);
  }
  readSelectionRange() {
    let { view: t } = this, e = k.safari && t.root.nodeType == 11 && cl(this.dom.ownerDocument) == this.dom && Ho(this.view) || ei(t.root);
    if (!e || this.selectionRange.eq(e))
      return !1;
    let i = Je(this.dom, e);
    return i && !this.selectionChanged && t.inputState.lastFocusTime > Date.now() - 200 && t.inputState.lastTouchTime < Date.now() - 300 && yl(this.dom, e) ? (this.view.inputState.lastFocusTime = 0, t.docView.updateSelection(), !1) : (this.selectionRange.setRange(e), i && (this.selectionChanged = !0), !0);
  }
  setSelectionRange(t, e) {
    this.selectionRange.set(t.node, t.offset, e.node, e.offset), this.selectionChanged = !1;
  }
  clearSelectionRange() {
    this.selectionRange.set(null, 0, null, 0);
  }
  listenForScroll() {
    this.parentCheck = -1;
    let t = 0, e = null;
    for (let i = this.dom; i; )
      if (i.nodeType == 1)
        !e && t < this.scrollTargets.length && this.scrollTargets[t] == i ? t++ : e || (e = this.scrollTargets.slice(0, t)), e && e.push(i), i = i.assignedSlot || i.parentNode;
      else if (i.nodeType == 11)
        i = i.host;
      else
        break;
    if (t < this.scrollTargets.length && !e && (e = this.scrollTargets.slice(0, t)), e) {
      for (let i of this.scrollTargets)
        i.removeEventListener("scroll", this.onScroll);
      for (let i of this.scrollTargets = e)
        i.addEventListener("scroll", this.onScroll);
    }
  }
  ignore(t) {
    if (!this.active)
      return t();
    try {
      return this.stop(), t();
    } finally {
      this.start(), this.clear();
    }
  }
  start() {
    this.active || (this.observer.observe(this.dom, No), xi && this.dom.addEventListener("DOMCharacterDataModified", this.onCharData), this.active = !0);
  }
  stop() {
    this.active && (this.active = !1, this.observer.disconnect(), xi && this.dom.removeEventListener("DOMCharacterDataModified", this.onCharData));
  }
  // Throw away any pending changes
  clear() {
    this.processRecords(), this.queue.length = 0, this.selectionChanged = !1;
  }
  // Chrome Android, especially in combination with GBoard, not only
  // doesn't reliably fire regular key events, but also often
  // surrounds the effect of enter or backspace with a bunch of
  // composition events that, when interrupted, cause text duplication
  // or other kinds of corruption. This hack makes the editor back off
  // from handling DOM changes for a moment when such a key is
  // detected (via beforeinput or keydown), and then tries to flush
  // them or, if that has no effect, dispatches the given key.
  delayAndroidKey(t, e) {
    var i;
    if (!this.delayedAndroidKey) {
      let n = () => {
        let s = this.delayedAndroidKey;
        s && (this.clearDelayedAndroidKey(), this.view.inputState.lastKeyCode = s.keyCode, this.view.inputState.lastKeyTime = Date.now(), !this.flush() && s.force && te(this.dom, s.key, s.keyCode));
      };
      this.flushingAndroidKey = this.view.win.requestAnimationFrame(n);
    }
    (!this.delayedAndroidKey || t == "Enter") && (this.delayedAndroidKey = {
      key: t,
      keyCode: e,
      // Only run the key handler when no changes are detected if
      // this isn't coming right after another change, in which case
      // it is probably part of a weird chain of updates, and should
      // be ignored if it returns the DOM to its previous state.
      force: this.lastChange < Date.now() - 50 || !!(!((i = this.delayedAndroidKey) === null || i === void 0) && i.force)
    });
  }
  clearDelayedAndroidKey() {
    this.win.cancelAnimationFrame(this.flushingAndroidKey), this.delayedAndroidKey = null, this.flushingAndroidKey = -1;
  }
  flushSoon() {
    this.delayedFlush < 0 && (this.delayedFlush = this.view.win.requestAnimationFrame(() => {
      this.delayedFlush = -1, this.flush();
    }));
  }
  forceFlush() {
    this.delayedFlush >= 0 && (this.view.win.cancelAnimationFrame(this.delayedFlush), this.delayedFlush = -1), this.flush();
  }
  pendingRecords() {
    for (let t of this.observer.takeRecords())
      this.queue.push(t);
    return this.queue;
  }
  processRecords() {
    let t = this.pendingRecords();
    t.length && (this.queue = []);
    let e = -1, i = -1, n = !1;
    for (let s of t) {
      let l = this.readMutation(s);
      l && (l.typeOver && (n = !0), e == -1 ? { from: e, to: i } = l : (e = Math.min(l.from, e), i = Math.max(l.to, i)));
    }
    return { from: e, to: i, typeOver: n };
  }
  readChange() {
    let { from: t, to: e, typeOver: i } = this.processRecords(), n = this.selectionChanged && Je(this.dom, this.selectionRange);
    if (t < 0 && !n)
      return null;
    t > -1 && (this.lastChange = Date.now()), this.view.inputState.lastFocusTime = 0, this.selectionChanged = !1;
    let s = new Po(this.view, t, e, i);
    return this.view.docView.domChanged = { newSel: s.newSel ? s.newSel.main : null }, s;
  }
  // Apply pending changes, if any
  flush(t = !0) {
    if (this.delayedFlush >= 0 || this.delayedAndroidKey)
      return !1;
    t && this.readSelectionRange();
    let e = this.readChange();
    if (!e)
      return !1;
    let i = this.view.state, n = Nr(this.view, e);
    return this.view.state == i && this.view.update([]), n;
  }
  readMutation(t) {
    let e = this.view.docView.nearest(t.target);
    if (!e || e.ignoreMutation(t))
      return null;
    if (e.markDirty(t.type == "attributes"), t.type == "attributes" && (e.flags |= 4), t.type == "childList") {
      let i = fs(e, t.previousSibling || t.target.previousSibling, -1), n = fs(e, t.nextSibling || t.target.nextSibling, 1);
      return {
        from: i ? e.posAfter(i) : e.posAtStart,
        to: n ? e.posBefore(n) : e.posAtEnd,
        typeOver: !1
      };
    } else
      return t.type == "characterData" ? { from: e.posAtStart, to: e.posAtEnd, typeOver: t.target.nodeValue == t.oldValue } : null;
  }
  setWindow(t) {
    t != this.win && (this.removeWindowListeners(this.win), this.win = t, this.addWindowListeners(this.win));
  }
  addWindowListeners(t) {
    t.addEventListener("resize", this.onResize), t.addEventListener("beforeprint", this.onPrint), t.addEventListener("scroll", this.onScroll), t.document.addEventListener("selectionchange", this.onSelectionChange);
  }
  removeWindowListeners(t) {
    t.removeEventListener("scroll", this.onScroll), t.removeEventListener("resize", this.onResize), t.removeEventListener("beforeprint", this.onPrint), t.document.removeEventListener("selectionchange", this.onSelectionChange);
  }
  destroy() {
    var t, e, i, n;
    this.stop(), (t = this.intersection) === null || t === void 0 || t.disconnect(), (e = this.gapIntersection) === null || e === void 0 || e.disconnect(), (i = this.resizeScroll) === null || i === void 0 || i.disconnect(), (n = this.resizeContent) === null || n === void 0 || n.disconnect();
    for (let s of this.scrollTargets)
      s.removeEventListener("scroll", this.onScroll);
    this.removeWindowListeners(this.win), clearTimeout(this.parentCheck), clearTimeout(this.resizeTimeout), this.win.cancelAnimationFrame(this.delayedFlush), this.win.cancelAnimationFrame(this.flushingAndroidKey);
  }
}
function fs(r, t, e) {
  for (; t; ) {
    let i = I.get(t);
    if (i && i.parent == r)
      return i;
    let n = t.parentNode;
    t = n != r.dom ? n : e > 0 ? t.nextSibling : t.previousSibling;
  }
  return null;
}
function Ho(r) {
  let t = null;
  function e(h) {
    h.preventDefault(), h.stopImmediatePropagation(), t = h.getTargetRanges()[0];
  }
  if (r.contentDOM.addEventListener("beforeinput", e, !0), r.dom.ownerDocument.execCommand("indent"), r.contentDOM.removeEventListener("beforeinput", e, !0), !t)
    return null;
  let i = t.startContainer, n = t.startOffset, s = t.endContainer, l = t.endOffset, o = r.docView.domAtPos(r.state.selection.main.anchor);
  return ii(o.node, o.offset, s, l) && ([i, n, s, l] = [s, l, i, n]), { anchorNode: i, anchorOffset: n, focusNode: s, focusOffset: l };
}
class T {
  /**
  The current editor state.
  */
  get state() {
    return this.viewState.state;
  }
  /**
  To be able to display large documents without consuming too much
  memory or overloading the browser, CodeMirror only draws the
  code that is visible (plus a margin around it) to the DOM. This
  property tells you the extent of the current drawn viewport, in
  document positions.
  */
  get viewport() {
    return this.viewState.viewport;
  }
  /**
  When there are, for example, large collapsed ranges in the
  viewport, its size can be a lot bigger than the actual visible
  content. Thus, if you are doing something like styling the
  content in the viewport, it is preferable to only do so for
  these ranges, which are the subset of the viewport that is
  actually drawn.
  */
  get visibleRanges() {
    return this.viewState.visibleRanges;
  }
  /**
  Returns false when the editor is entirely scrolled out of view
  or otherwise hidden.
  */
  get inView() {
    return this.viewState.inView;
  }
  /**
  Indicates whether the user is currently composing text via
  [IME](https://en.wikipedia.org/wiki/Input_method), and at least
  one change has been made in the current composition.
  */
  get composing() {
    return this.inputState.composing > 0;
  }
  /**
  Indicates whether the user is currently in composing state. Note
  that on some platforms, like Android, this will be the case a
  lot, since just putting the cursor on a word starts a
  composition there.
  */
  get compositionStarted() {
    return this.inputState.composing >= 0;
  }
  /**
  The document or shadow root that the view lives in.
  */
  get root() {
    return this._root;
  }
  /**
  @internal
  */
  get win() {
    return this.dom.ownerDocument.defaultView || window;
  }
  /**
  Construct a new view. You'll want to either provide a `parent`
  option, or put `view.dom` into your document after creating a
  view, so that the user can see the editor.
  */
  constructor(t = {}) {
    this.plugins = [], this.pluginMap = /* @__PURE__ */ new Map(), this.editorAttrs = {}, this.contentAttrs = {}, this.bidiCache = [], this.destroyed = !1, this.updateState = 2, this.measureScheduled = -1, this.measureRequests = [], this.contentDOM = document.createElement("div"), this.scrollDOM = document.createElement("div"), this.scrollDOM.tabIndex = -1, this.scrollDOM.className = "cm-scroller", this.scrollDOM.appendChild(this.contentDOM), this.announceDOM = document.createElement("div"), this.announceDOM.style.cssText = "position: fixed; top: -10000px", this.announceDOM.setAttribute("aria-live", "polite"), this.dom = document.createElement("div"), this.dom.appendChild(this.announceDOM), this.dom.appendChild(this.scrollDOM);
    let { dispatch: e } = t;
    this.dispatchTransactions = t.dispatchTransactions || e && ((i) => i.forEach((n) => e(n, this))) || ((i) => this.update(i)), this.dispatch = this.dispatch.bind(this), this._root = t.root || ml(t.parent) || document, this.viewState = new ls(t.state || E.create(t)), this.plugins = this.state.facet(ge).map((i) => new mi(i));
    for (let i of this.plugins)
      i.update(this);
    this.observer = new Io(this), this.inputState = new Xl(this), this.inputState.ensureHandlers(this.plugins), this.docView = new qn(this), this.mountStyles(), this.updateAttrs(), this.updateState = 0, this.requestMeasure(), t.parent && t.parent.appendChild(this.dom);
  }
  dispatch(...t) {
    let e = t.length == 1 && t[0] instanceof _ ? t : t.length == 1 && Array.isArray(t[0]) ? t[0] : [this.state.update(...t)];
    this.dispatchTransactions(e, this);
  }
  /**
  Update the view for the given array of transactions. This will
  update the visible document and selection to match the state
  produced by the transactions, and notify view plugins of the
  change. You should usually call
  [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead, which uses this
  as a primitive.
  */
  update(t) {
    if (this.updateState != 0)
      throw new Error("Calls to EditorView.update are not allowed while an update is in progress");
    let e = !1, i = !1, n, s = this.state;
    for (let u of t) {
      if (u.startState != s)
        throw new RangeError("Trying to update state with a transaction that doesn't start from the previous state.");
      s = u.state;
    }
    if (this.destroyed) {
      this.viewState.state = s;
      return;
    }
    let l = this.hasFocus, o = 0, h = null;
    t.some((u) => u.annotation(Dr)) ? (this.inputState.notifiedFocused = l, o = 1) : l != this.inputState.notifiedFocused && (this.inputState.notifiedFocused = l, h = Tr(s, l), h || (o = 1));
    let a = this.observer.delayedAndroidKey, f = null;
    if (a ? (this.observer.clearDelayedAndroidKey(), f = this.observer.readChange(), (f && !this.state.doc.eq(s.doc) || !this.state.selection.eq(s.selection)) && (f = null)) : this.observer.clear(), s.facet(E.phrases) != this.state.facet(E.phrases))
      return this.setState(s);
    n = si.create(this, s, t), n.flags |= o;
    let c = this.viewState.scrollTarget;
    try {
      this.updateState = 2;
      for (let u of t) {
        if (c && (c = c.map(u.changes)), u.scrollIntoView) {
          let { main: d } = u.state.selection;
          c = new ni(d.empty ? d : x.cursor(d.head, d.head > d.anchor ? -1 : 1));
        }
        for (let d of u.effects)
          d.is(Wn) && (c = d.value);
      }
      this.viewState.update(n, c), this.bidiCache = ri.update(this.bidiCache, n.changes), n.empty || (this.updatePlugins(n), this.inputState.update(n)), e = this.docView.update(n), this.state.facet(pe) != this.styleModules && this.mountStyles(), i = this.updateAttrs(), this.showAnnouncements(t), this.docView.updateSelection(e, t.some((u) => u.isUserEvent("select.pointer")));
    } finally {
      this.updateState = 0;
    }
    if (n.startState.facet(We) != n.state.facet(We) && (this.viewState.mustMeasureContent = !0), (e || i || c || this.viewState.mustEnforceCursorAssoc || this.viewState.mustMeasureContent) && this.requestMeasure(), !n.empty)
      for (let u of this.state.facet(Yi))
        u(n);
    (h || f) && Promise.resolve().then(() => {
      h && this.state == h.startState && this.dispatch(h), f && !Nr(this, f) && a.force && te(this.contentDOM, a.key, a.keyCode);
    });
  }
  /**
  Reset the view to the given state. (This will cause the entire
  document to be redrawn and all view plugins to be reinitialized,
  so you should probably only use it when the new state isn't
  derived from the old state. Otherwise, use
  [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead.)
  */
  setState(t) {
    if (this.updateState != 0)
      throw new Error("Calls to EditorView.setState are not allowed while an update is in progress");
    if (this.destroyed) {
      this.viewState.state = t;
      return;
    }
    this.updateState = 2;
    let e = this.hasFocus;
    try {
      for (let i of this.plugins)
        i.destroy(this);
      this.viewState = new ls(t), this.plugins = t.facet(ge).map((i) => new mi(i)), this.pluginMap.clear();
      for (let i of this.plugins)
        i.update(this);
      this.docView = new qn(this), this.inputState.ensureHandlers(this.plugins), this.mountStyles(), this.updateAttrs(), this.bidiCache = [];
    } finally {
      this.updateState = 0;
    }
    e && this.focus(), this.requestMeasure();
  }
  updatePlugins(t) {
    let e = t.startState.facet(ge), i = t.state.facet(ge);
    if (e != i) {
      let n = [];
      for (let s of i) {
        let l = e.indexOf(s);
        if (l < 0)
          n.push(new mi(s));
        else {
          let o = this.plugins[l];
          o.mustUpdate = t, n.push(o);
        }
      }
      for (let s of this.plugins)
        s.mustUpdate != t && s.destroy(this);
      this.plugins = n, this.pluginMap.clear();
    } else
      for (let n of this.plugins)
        n.mustUpdate = t;
    for (let n = 0; n < this.plugins.length; n++)
      this.plugins[n].update(this);
    e != i && this.inputState.ensureHandlers(this.plugins);
  }
  /**
  @internal
  */
  measure(t = !0) {
    if (this.destroyed)
      return;
    if (this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.observer.delayedAndroidKey) {
      this.measureScheduled = -1, this.requestMeasure();
      return;
    }
    this.measureScheduled = 0, t && this.observer.forceFlush();
    let e = null, i = this.scrollDOM, n = i.scrollTop * this.scaleY, { scrollAnchorPos: s, scrollAnchorHeight: l } = this.viewState;
    Math.abs(n - this.viewState.scrollTop) > 1 && (l = -1), this.viewState.scrollAnchorHeight = -1;
    try {
      for (let o = 0; ; o++) {
        if (l < 0)
          if (Js(i))
            s = -1, l = this.viewState.heightMap.height;
          else {
            let d = this.viewState.scrollAnchorAt(n);
            s = d.from, l = d.top;
          }
        this.updateState = 1;
        let h = this.viewState.measure(this);
        if (!h && !this.measureRequests.length && this.viewState.scrollTarget == null)
          break;
        if (o > 5) {
          console.warn(this.measureRequests.length ? "Measure loop restarted more than 5 times" : "Viewport failed to stabilize");
          break;
        }
        let a = [];
        h & 4 || ([this.measureRequests, a] = [a, this.measureRequests]);
        let f = a.map((d) => {
          try {
            return d.read(this);
          } catch (g) {
            return $t(this.state, g), cs;
          }
        }), c = si.create(this, this.state, []), u = !1;
        c.flags |= h, e ? e.flags |= h : e = c, this.updateState = 2, c.empty || (this.updatePlugins(c), this.inputState.update(c), this.updateAttrs(), u = this.docView.update(c));
        for (let d = 0; d < a.length; d++)
          if (f[d] != cs)
            try {
              let g = a[d];
              g.write && g.write(f[d], this);
            } catch (g) {
              $t(this.state, g);
            }
        if (u && this.docView.updateSelection(!0), !c.viewportChanged && this.measureRequests.length == 0) {
          if (this.viewState.editorHeight)
            if (this.viewState.scrollTarget) {
              this.docView.scrollIntoView(this.viewState.scrollTarget), this.viewState.scrollTarget = null;
              continue;
            } else {
              let g = (s < 0 ? this.viewState.heightMap.height : this.viewState.lineBlockAt(s).top) - l;
              if (g > 1 || g < -1) {
                n = n + g, i.scrollTop = n / this.scaleY, l = -1;
                continue;
              }
            }
          break;
        }
      }
    } finally {
      this.updateState = 0, this.measureScheduled = -1;
    }
    if (e && !e.empty)
      for (let o of this.state.facet(Yi))
        o(e);
  }
  /**
  Get the CSS classes for the currently active editor themes.
  */
  get themeClasses() {
    return Qi + " " + (this.state.facet(Zi) ? Er : Rr) + " " + this.state.facet(We);
  }
  updateAttrs() {
    let t = us(this, cr, {
      class: "cm-editor" + (this.hasFocus ? " cm-focused " : " ") + this.themeClasses
    }), e = {
      spellcheck: "false",
      autocorrect: "off",
      autocapitalize: "off",
      translate: "no",
      contenteditable: this.state.facet(ci) ? "true" : "false",
      class: "cm-content",
      style: `${k.tabSize}: ${this.state.tabSize}`,
      role: "textbox",
      "aria-multiline": "true"
    };
    this.state.readOnly && (e["aria-readonly"] = "true"), us(this, un, e);
    let i = this.observer.ignore(() => {
      let n = ji(this.contentDOM, this.contentAttrs, e), s = ji(this.dom, this.editorAttrs, t);
      return n || s;
    });
    return this.editorAttrs = t, this.contentAttrs = e, i;
  }
  showAnnouncements(t) {
    let e = !0;
    for (let i of t)
      for (let n of i.effects)
        if (n.is(T.announce)) {
          e && (this.announceDOM.textContent = ""), e = !1;
          let s = this.announceDOM.appendChild(document.createElement("div"));
          s.textContent = n.value;
        }
  }
  mountStyles() {
    this.styleModules = this.state.facet(pe);
    let t = this.state.facet(T.cspNonce);
    ie.mount(this.root, this.styleModules.concat(Oo).reverse(), t ? { nonce: t } : void 0);
  }
  readMeasured() {
    if (this.updateState == 2)
      throw new Error("Reading the editor layout isn't allowed during an update");
    this.updateState == 0 && this.measureScheduled > -1 && this.measure(!1);
  }
  /**
  Schedule a layout measurement, optionally providing callbacks to
  do custom DOM measuring followed by a DOM write phase. Using
  this is preferable reading DOM layout directly from, for
  example, an event handler, because it'll make sure measuring and
  drawing done by other components is synchronized, avoiding
  unnecessary DOM layout computations.
  */
  requestMeasure(t) {
    if (this.measureScheduled < 0 && (this.measureScheduled = this.win.requestAnimationFrame(() => this.measure())), t) {
      if (this.measureRequests.indexOf(t) > -1)
        return;
      if (t.key != null) {
        for (let e = 0; e < this.measureRequests.length; e++)
          if (this.measureRequests[e].key === t.key) {
            this.measureRequests[e] = t;
            return;
          }
      }
      this.measureRequests.push(t);
    }
  }
  /**
  Get the value of a specific plugin, if present. Note that
  plugins that crash can be dropped from a view, so even when you
  know you registered a given plugin, it is recommended to check
  the return value of this method.
  */
  plugin(t) {
    let e = this.pluginMap.get(t);
    return (e === void 0 || e && e.spec != t) && this.pluginMap.set(t, e = this.plugins.find((i) => i.spec == t) || null), e && e.update(this).value;
  }
  /**
  The top position of the document, in screen coordinates. This
  may be negative when the editor is scrolled down. Points
  directly to the top of the first line, not above the padding.
  */
  get documentTop() {
    return this.contentDOM.getBoundingClientRect().top + this.viewState.paddingTop;
  }
  /**
  Reports the padding above and below the document.
  */
  get documentPadding() {
    return { top: this.viewState.paddingTop, bottom: this.viewState.paddingBottom };
  }
  /**
  If the editor is transformed with CSS, this provides the scale
  along the X axis. Otherwise, it will just be 1. Note that
  transforms other than translation and scaling are not supported.
  */
  get scaleX() {
    return this.viewState.scaleX;
  }
  /**
  Provide the CSS transformed scale along the Y axis.
  */
  get scaleY() {
    return this.viewState.scaleY;
  }
  /**
  Find the text line or block widget at the given vertical
  position (which is interpreted as relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop)).
  */
  elementAtHeight(t) {
    return this.readMeasured(), this.viewState.elementAtHeight(t);
  }
  /**
  Find the line block (see
  [`lineBlockAt`](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) at the given
  height, again interpreted relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop).
  */
  lineBlockAtHeight(t) {
    return this.readMeasured(), this.viewState.lineBlockAtHeight(t);
  }
  /**
  Get the extent and vertical position of all [line
  blocks](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) in the viewport. Positions
  are relative to the [top of the
  document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop);
  */
  get viewportLineBlocks() {
    return this.viewState.viewportLines;
  }
  /**
  Find the line block around the given document position. A line
  block is a range delimited on both sides by either a
  non-[hidden](https://codemirror.net/6/docs/ref/#view.Decoration^replace) line breaks, or the
  start/end of the document. It will usually just hold a line of
  text, but may be broken into multiple textblocks by block
  widgets.
  */
  lineBlockAt(t) {
    return this.viewState.lineBlockAt(t);
  }
  /**
  The editor's total content height.
  */
  get contentHeight() {
    return this.viewState.contentHeight;
  }
  /**
  Move a cursor position by [grapheme
  cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak). `forward` determines whether
  the motion is away from the line start, or towards it. In
  bidirectional text, the line is traversed in visual order, using
  the editor's [text direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection).
  When the start position was the last one on the line, the
  returned position will be across the line break. If there is no
  further line, the original position is returned.
  
  By default, this method moves over a single cluster. The
  optional `by` argument can be used to move across more. It will
  be called with the first cluster as argument, and should return
  a predicate that determines, for each subsequent cluster,
  whether it should also be moved over.
  */
  moveByChar(t, e, i) {
    return wi(this, t, Jn(this, t, e, i));
  }
  /**
  Move a cursor position across the next group of either
  [letters](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) or non-letter
  non-whitespace characters.
  */
  moveByGroup(t, e) {
    return wi(this, t, Jn(this, t, e, (i) => Jl(this, t.head, i)));
  }
  /**
  Move to the next line boundary in the given direction. If
  `includeWrap` is true, line wrapping is on, and there is a
  further wrap point on the current line, the wrap point will be
  returned. Otherwise this function will return the start or end
  of the line.
  */
  moveToLineBoundary(t, e, i = !0) {
    return Gl(this, t, e, i);
  }
  /**
  Move a cursor position vertically. When `distance` isn't given,
  it defaults to moving to the next line (including wrapped
  lines). Otherwise, `distance` should provide a positive distance
  in pixels.
  
  When `start` has a
  [`goalColumn`](https://codemirror.net/6/docs/ref/#state.SelectionRange.goalColumn), the vertical
  motion will use that as a target horizontal position. Otherwise,
  the cursor's own horizontal position is used. The returned
  cursor will have its goal column set to whichever column was
  used.
  */
  moveVertically(t, e, i) {
    return wi(this, t, Ul(this, t, e, i));
  }
  /**
  Find the DOM parent node and offset (child offset if `node` is
  an element, character offset when it is a text node) at the
  given document position.
  
  Note that for positions that aren't currently in
  `visibleRanges`, the resulting DOM position isn't necessarily
  meaningful (it may just point before or after a placeholder
  element).
  */
  domAtPos(t) {
    return this.docView.domAtPos(t);
  }
  /**
  Find the document position at the given DOM node. Can be useful
  for associating positions with DOM events. Will raise an error
  when `node` isn't part of the editor content.
  */
  posAtDOM(t, e = 0) {
    return this.docView.posFromDOM(t, e);
  }
  posAtCoords(t, e = !0) {
    return this.readMeasured(), vr(this, t, e);
  }
  /**
  Get the screen coordinates at the given document position.
  `side` determines whether the coordinates are based on the
  element before (-1) or after (1) the position (if no element is
  available on the given side, the method will transparently use
  another strategy to get reasonable coordinates).
  */
  coordsAtPos(t, e = 1) {
    this.readMeasured();
    let i = this.docView.coordsAt(t, e);
    if (!i || i.left == i.right)
      return i;
    let n = this.state.doc.lineAt(t), s = this.bidiSpans(n), l = s[Nt.find(s, t - n.from, -1, e)];
    return an(i, l.dir == Q.LTR == e > 0);
  }
  /**
  Return the rectangle around a given character. If `pos` does not
  point in front of a character that is in the viewport and
  rendered (i.e. not replaced, not a line break), this will return
  null. For space characters that are a line wrap point, this will
  return the position before the line break.
  */
  coordsForChar(t) {
    return this.readMeasured(), this.docView.coordsForChar(t);
  }
  /**
  The default width of a character in the editor. May not
  accurately reflect the width of all characters (given variable
  width fonts or styling of invididual ranges).
  */
  get defaultCharacterWidth() {
    return this.viewState.heightOracle.charWidth;
  }
  /**
  The default height of a line in the editor. May not be accurate
  for all lines.
  */
  get defaultLineHeight() {
    return this.viewState.heightOracle.lineHeight;
  }
  /**
  The text direction
  ([`direction`](https://developer.mozilla.org/en-US/docs/Web/CSS/direction)
  CSS property) of the editor's content element.
  */
  get textDirection() {
    return this.viewState.defaultTextDirection;
  }
  /**
  Find the text direction of the block at the given position, as
  assigned by CSS. If
  [`perLineTextDirection`](https://codemirror.net/6/docs/ref/#view.EditorView^perLineTextDirection)
  isn't enabled, or the given position is outside of the viewport,
  this will always return the same as
  [`textDirection`](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection). Note that
  this may trigger a DOM layout.
  */
  textDirectionAt(t) {
    return !this.state.facet(fr) || t < this.viewport.from || t > this.viewport.to ? this.textDirection : (this.readMeasured(), this.docView.textDirectionAt(t));
  }
  /**
  Whether this editor [wraps lines](https://codemirror.net/6/docs/ref/#view.EditorView.lineWrapping)
  (as determined by the
  [`white-space`](https://developer.mozilla.org/en-US/docs/Web/CSS/white-space)
  CSS property of its content element).
  */
  get lineWrapping() {
    return this.viewState.heightOracle.lineWrapping;
  }
  /**
  Returns the bidirectional text structure of the given line
  (which should be in the current document) as an array of span
  objects. The order of these spans matches the [text
  direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection)—if that is
  left-to-right, the leftmost spans come first, otherwise the
  rightmost spans come first.
  */
  bidiSpans(t) {
    if (t.length > Fo)
      return wr(t.length);
    let e = this.textDirectionAt(t.from), i;
    for (let s of this.bidiCache)
      if (s.from == t.from && s.dir == e && (s.fresh || yr(s.isolates, i = zn(this, t.from, t.to))))
        return s.order;
    i || (i = zn(this, t.from, t.to));
    let n = Rl(t.text, e, i);
    return this.bidiCache.push(new ri(t.from, t.to, e, i, !0, n)), n;
  }
  /**
  Check whether the editor has focus.
  */
  get hasFocus() {
    var t;
    return (this.dom.ownerDocument.hasFocus() || k.safari && ((t = this.inputState) === null || t === void 0 ? void 0 : t.lastContextMenu) > Date.now() - 3e4) && this.root.activeElement == this.contentDOM;
  }
  /**
  Put focus on the editor.
  */
  focus() {
    this.observer.ignore(() => {
      Ys(this.contentDOM), this.docView.updateSelection();
    });
  }
  /**
  Update the [root](https://codemirror.net/6/docs/ref/##view.EditorViewConfig.root) in which the editor lives. This is only
  necessary when moving the editor's existing DOM to a new window or shadow root.
  */
  setRoot(t) {
    this._root != t && (this._root = t, this.observer.setWindow((t.nodeType == 9 ? t : t.ownerDocument).defaultView || window), this.mountStyles());
  }
  /**
  Clean up this editor view, removing its element from the
  document, unregistering event handlers, and notifying
  plugins. The view instance can no longer be used after
  calling this.
  */
  destroy() {
    for (let t of this.plugins)
      t.destroy(this);
    this.plugins = [], this.inputState.destroy(), this.dom.remove(), this.observer.destroy(), this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.destroyed = !0;
  }
  /**
  Returns an effect that can be
  [added](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) to a transaction to
  cause it to scroll the given position or range into view.
  */
  static scrollIntoView(t, e = {}) {
    return Wn.of(new ni(typeof t == "number" ? x.cursor(t) : t, e.y, e.x, e.yMargin, e.xMargin));
  }
  /**
  Returns an extension that can be used to add DOM event handlers.
  The value should be an object mapping event names to handler
  functions. For any given event, such functions are ordered by
  extension precedence, and the first handler to return true will
  be assumed to have handled that event, and no other handlers or
  built-in behavior will be activated for it. These are registered
  on the [content element](https://codemirror.net/6/docs/ref/#view.EditorView.contentDOM), except
  for `scroll` handlers, which will be called any time the
  editor's [scroll element](https://codemirror.net/6/docs/ref/#view.EditorView.scrollDOM) or one of
  its parent nodes is scrolled.
  */
  static domEventHandlers(t) {
    return Tt.define(() => ({}), { eventHandlers: t });
  }
  /**
  Create an extension that registers DOM event observers. Contrary
  to event [handlers](https://codemirror.net/6/docs/ref/#view.EditorView^domEventHandlers),
  observers can't be prevented from running by a higher-precedence
  handler returning true. They also don't prevent other handlers
  and observers from running when they return true, and should not
  call `preventDefault`.
  */
  static domEventObservers(t) {
    return Tt.define(() => ({}), { eventObservers: t });
  }
  /**
  Create a theme extension. The first argument can be a
  [`style-mod`](https://github.com/marijnh/style-mod#documentation)
  style spec providing the styles for the theme. These will be
  prefixed with a generated class for the style.
  
  Because the selectors will be prefixed with a scope class, rule
  that directly match the editor's [wrapper
  element](https://codemirror.net/6/docs/ref/#view.EditorView.dom)—to which the scope class will be
  added—need to be explicitly differentiated by adding an `&` to
  the selector for that element—for example
  `&.cm-focused`.
  
  When `dark` is set to true, the theme will be marked as dark,
  which will cause the `&dark` rules from [base
  themes](https://codemirror.net/6/docs/ref/#view.EditorView^baseTheme) to be used (as opposed to
  `&light` when a light theme is active).
  */
  static theme(t, e) {
    let i = ie.newName(), n = [We.of(i), pe.of(tn(`.${i}`, t))];
    return e && e.dark && n.push(Zi.of(!0)), n;
  }
  /**
  Create an extension that adds styles to the base theme. Like
  with [`theme`](https://codemirror.net/6/docs/ref/#view.EditorView^theme), use `&` to indicate the
  place of the editor wrapper element when directly targeting
  that. You can also use `&dark` or `&light` instead to only
  target editors with a dark or light theme.
  */
  static baseTheme(t) {
    return Xr.lowest(pe.of(tn("." + Qi, t, Lr)));
  }
  /**
  Retrieve an editor view instance from the view's DOM
  representation.
  */
  static findFromDOM(t) {
    var e;
    let i = t.querySelector(".cm-content"), n = i && I.get(i) || I.get(t);
    return ((e = n == null ? void 0 : n.rootView) === null || e === void 0 ? void 0 : e.view) || null;
  }
}
T.styleModule = pe;
T.inputHandler = hr;
T.focusChangeEffect = ar;
T.perLineTextDirection = fr;
T.exceptionSink = or;
T.updateListener = Yi;
T.editable = ci;
T.mouseSelectionStyle = lr;
T.dragMovesSelection = rr;
T.clickAddsSelectionRange = sr;
T.decorations = Ce;
T.atomicRanges = dn;
T.bidiIsolatedRanges = ur;
T.scrollMargins = dr;
T.darkTheme = Zi;
T.cspNonce = /* @__PURE__ */ O.define({ combine: (r) => r.length ? r[0] : "" });
T.contentAttributes = un;
T.editorAttributes = cr;
T.lineWrapping = /* @__PURE__ */ T.contentAttributes.of({ class: "cm-lineWrapping" });
T.announce = /* @__PURE__ */ q.define();
const Fo = 4096, cs = {};
class ri {
  constructor(t, e, i, n, s, l) {
    this.from = t, this.to = e, this.dir = i, this.isolates = n, this.fresh = s, this.order = l;
  }
  static update(t, e) {
    if (e.empty && !t.some((s) => s.fresh))
      return t;
    let i = [], n = t.length ? t[t.length - 1].dir : Q.LTR;
    for (let s = Math.max(0, t.length - 10); s < t.length; s++) {
      let l = t[s];
      l.dir == n && !e.touchesRange(l.from, l.to) && i.push(new ri(e.mapPos(l.from, 1), e.mapPos(l.to, -1), l.dir, l.isolates, !1, l.order));
    }
    return i;
  }
}
function us(r, t, e) {
  for (let i = r.state.facet(t), n = i.length - 1; n >= 0; n--) {
    let s = i[n], l = typeof s == "function" ? s(r) : s;
    l && Ki(l, e);
  }
  return e;
}
const Vo = !k.ios, Wo = {
  ".cm-line": {
    "& ::selection": { backgroundColor: "transparent !important" },
    "&::selection": { backgroundColor: "transparent !important" }
  }
};
Vo && (Wo[".cm-line"].caretColor = "transparent !important");
class re extends ee {
  /**
  @internal
  */
  compare(t) {
    return this == t || this.constructor == t.constructor && this.eq(t);
  }
  /**
  Compare this marker to another marker of the same type.
  */
  eq(t) {
    return !1;
  }
  /**
  Called if the marker has a `toDOM` method and its representation
  was removed from a gutter.
  */
  destroy(t) {
  }
}
re.prototype.elementClass = "";
re.prototype.toDOM = void 0;
re.prototype.mapMode = nt.TrackBefore;
re.prototype.startSide = re.prototype.endSide = -1;
re.prototype.point = !0;
const zo = 1024;
let qo = 0;
class ki {
  constructor(t, e) {
    this.from = t, this.to = e;
  }
}
class F {
  /**
  Create a new node prop type.
  */
  constructor(t = {}) {
    this.id = qo++, this.perNode = !!t.perNode, this.deserialize = t.deserialize || (() => {
      throw new Error("This node type doesn't define a deserialize function");
    });
  }
  /**
  This is meant to be used with
  [`NodeSet.extend`](#common.NodeSet.extend) or
  [`LRParser.configure`](#lr.ParserConfig.props) to compute
  prop values for each node type in the set. Takes a [match
  object](#common.NodeType^match) or function that returns undefined
  if the node type doesn't get this prop, and the prop's value if
  it does.
  */
  add(t) {
    if (this.perNode)
      throw new RangeError("Can't add per-node props to node types");
    return typeof t != "function" && (t = ot.match(t)), (e) => {
      let i = t(e);
      return i === void 0 ? null : [this, i];
    };
  }
}
F.closedBy = new F({ deserialize: (r) => r.split(" ") });
F.openedBy = new F({ deserialize: (r) => r.split(" ") });
F.group = new F({ deserialize: (r) => r.split(" ") });
F.contextHash = new F({ perNode: !0 });
F.lookAhead = new F({ perNode: !0 });
F.mounted = new F({ perNode: !0 });
class li {
  constructor(t, e, i) {
    this.tree = t, this.overlay = e, this.parser = i;
  }
  /**
  @internal
  */
  static get(t) {
    return t && t.props && t.props[F.mounted.id];
  }
}
const Ko = /* @__PURE__ */ Object.create(null);
class ot {
  /**
  @internal
  */
  constructor(t, e, i, n = 0) {
    this.name = t, this.props = e, this.id = i, this.flags = n;
  }
  /**
  Define a node type.
  */
  static define(t) {
    let e = t.props && t.props.length ? /* @__PURE__ */ Object.create(null) : Ko, i = (t.top ? 1 : 0) | (t.skipped ? 2 : 0) | (t.error ? 4 : 0) | (t.name == null ? 8 : 0), n = new ot(t.name || "", e, t.id, i);
    if (t.props) {
      for (let s of t.props)
        if (Array.isArray(s) || (s = s(n)), s) {
          if (s[0].perNode)
            throw new RangeError("Can't store a per-node prop on a node type");
          e[s[0].id] = s[1];
        }
    }
    return n;
  }
  /**
  Retrieves a node prop for this type. Will return `undefined` if
  the prop isn't present on this node.
  */
  prop(t) {
    return this.props[t.id];
  }
  /**
  True when this is the top node of a grammar.
  */
  get isTop() {
    return (this.flags & 1) > 0;
  }
  /**
  True when this node is produced by a skip rule.
  */
  get isSkipped() {
    return (this.flags & 2) > 0;
  }
  /**
  Indicates whether this is an error node.
  */
  get isError() {
    return (this.flags & 4) > 0;
  }
  /**
  When true, this node type doesn't correspond to a user-declared
  named node, for example because it is used to cache repetition.
  */
  get isAnonymous() {
    return (this.flags & 8) > 0;
  }
  /**
  Returns true when this node's name or one of its
  [groups](#common.NodeProp^group) matches the given string.
  */
  is(t) {
    if (typeof t == "string") {
      if (this.name == t)
        return !0;
      let e = this.prop(F.group);
      return e ? e.indexOf(t) > -1 : !1;
    }
    return this.id == t;
  }
  /**
  Create a function from node types to arbitrary values by
  specifying an object whose property names are node or
  [group](#common.NodeProp^group) names. Often useful with
  [`NodeProp.add`](#common.NodeProp.add). You can put multiple
  names, separated by spaces, in a single property name to map
  multiple node names to a single value.
  */
  static match(t) {
    let e = /* @__PURE__ */ Object.create(null);
    for (let i in t)
      for (let n of i.split(" "))
        e[n] = t[i];
    return (i) => {
      for (let n = i.prop(F.group), s = -1; s < (n ? n.length : 0); s++) {
        let l = e[s < 0 ? i.name : n[s]];
        if (l)
          return l;
      }
    };
  }
}
ot.none = new ot(
  "",
  /* @__PURE__ */ Object.create(null),
  0,
  8
  /* NodeFlag.Anonymous */
);
const ze = /* @__PURE__ */ new WeakMap(), ds = /* @__PURE__ */ new WeakMap();
var J;
(function(r) {
  r[r.ExcludeBuffers = 1] = "ExcludeBuffers", r[r.IncludeAnonymous = 2] = "IncludeAnonymous", r[r.IgnoreMounts = 4] = "IgnoreMounts", r[r.IgnoreOverlays = 8] = "IgnoreOverlays";
})(J || (J = {}));
class Z {
  /**
  Construct a new tree. See also [`Tree.build`](#common.Tree^build).
  */
  constructor(t, e, i, n, s) {
    if (this.type = t, this.children = e, this.positions = i, this.length = n, this.props = null, s && s.length) {
      this.props = /* @__PURE__ */ Object.create(null);
      for (let [l, o] of s)
        this.props[typeof l == "number" ? l : l.id] = o;
    }
  }
  /**
  @internal
  */
  toString() {
    let t = li.get(this);
    if (t && !t.overlay)
      return t.tree.toString();
    let e = "";
    for (let i of this.children) {
      let n = i.toString();
      n && (e && (e += ","), e += n);
    }
    return this.type.name ? (/\W/.test(this.type.name) && !this.type.isError ? JSON.stringify(this.type.name) : this.type.name) + (e.length ? "(" + e + ")" : "") : e;
  }
  /**
  Get a [tree cursor](#common.TreeCursor) positioned at the top of
  the tree. Mode can be used to [control](#common.IterMode) which
  nodes the cursor visits.
  */
  cursor(t = 0) {
    return new nn(this.topNode, t);
  }
  /**
  Get a [tree cursor](#common.TreeCursor) pointing into this tree
  at the given position and side (see
  [`moveTo`](#common.TreeCursor.moveTo).
  */
  cursorAt(t, e = 0, i = 0) {
    let n = ze.get(this) || this.topNode, s = new nn(n);
    return s.moveTo(t, e), ze.set(this, s._tree), s;
  }
  /**
  Get a [syntax node](#common.SyntaxNode) object for the top of the
  tree.
  */
  get topNode() {
    return new lt(this, 0, 0, null);
  }
  /**
  Get the [syntax node](#common.SyntaxNode) at the given position.
  If `side` is -1, this will move into nodes that end at the
  position. If 1, it'll move into nodes that start at the
  position. With 0, it'll only enter nodes that cover the position
  from both sides.
  
  Note that this will not enter
  [overlays](#common.MountedTree.overlay), and you often want
  [`resolveInner`](#common.Tree.resolveInner) instead.
  */
  resolve(t, e = 0) {
    let i = Oe(ze.get(this) || this.topNode, t, e, !1);
    return ze.set(this, i), i;
  }
  /**
  Like [`resolve`](#common.Tree.resolve), but will enter
  [overlaid](#common.MountedTree.overlay) nodes, producing a syntax node
  pointing into the innermost overlaid tree at the given position
  (with parent links going through all parent structure, including
  the host trees).
  */
  resolveInner(t, e = 0) {
    let i = Oe(ds.get(this) || this.topNode, t, e, !0);
    return ds.set(this, i), i;
  }
  /**
  In some situations, it can be useful to iterate through all
  nodes around a position, including those in overlays that don't
  directly cover the position. This method gives you an iterator
  that will produce all nodes, from small to big, around the given
  position.
  */
  resolveStack(t, e = 0) {
    return Yo(this, t, e);
  }
  /**
  Iterate over the tree and its children, calling `enter` for any
  node that touches the `from`/`to` region (if given) before
  running over such a node's children, and `leave` (if given) when
  leaving the node. When `enter` returns `false`, that node will
  not have its children iterated over (or `leave` called).
  */
  iterate(t) {
    let { enter: e, leave: i, from: n = 0, to: s = this.length } = t, l = t.mode || 0, o = (l & J.IncludeAnonymous) > 0;
    for (let h = this.cursor(l | J.IncludeAnonymous); ; ) {
      let a = !1;
      if (h.from <= s && h.to >= n && (!o && h.type.isAnonymous || e(h) !== !1)) {
        if (h.firstChild())
          continue;
        a = !0;
      }
      for (; a && i && (o || !h.type.isAnonymous) && i(h), !h.nextSibling(); ) {
        if (!h.parent())
          return;
        a = !0;
      }
    }
  }
  /**
  Get the value of the given [node prop](#common.NodeProp) for this
  node. Works with both per-node and per-type props.
  */
  prop(t) {
    return t.perNode ? this.props ? this.props[t.id] : void 0 : this.type.prop(t);
  }
  /**
  Returns the node's [per-node props](#common.NodeProp.perNode) in a
  format that can be passed to the [`Tree`](#common.Tree)
  constructor.
  */
  get propValues() {
    let t = [];
    if (this.props)
      for (let e in this.props)
        t.push([+e, this.props[e]]);
    return t;
  }
  /**
  Balance the direct children of this tree, producing a copy of
  which may have children grouped into subtrees with type
  [`NodeType.none`](#common.NodeType^none).
  */
  balance(t = {}) {
    return this.children.length <= 8 ? this : yn(ot.none, this.children, this.positions, 0, this.children.length, 0, this.length, (e, i, n) => new Z(this.type, e, i, n, this.propValues), t.makeTree || ((e, i, n) => new Z(ot.none, e, i, n)));
  }
  /**
  Build a tree from a postfix-ordered buffer of node information,
  or a cursor over such a buffer.
  */
  static build(t) {
    return Go(t);
  }
}
Z.empty = new Z(ot.none, [], [], 0);
class pn {
  constructor(t, e) {
    this.buffer = t, this.index = e;
  }
  get id() {
    return this.buffer[this.index - 4];
  }
  get start() {
    return this.buffer[this.index - 3];
  }
  get end() {
    return this.buffer[this.index - 2];
  }
  get size() {
    return this.buffer[this.index - 1];
  }
  get pos() {
    return this.index;
  }
  next() {
    this.index -= 4;
  }
  fork() {
    return new pn(this.buffer, this.index);
  }
}
class Jt {
  /**
  Create a tree buffer.
  */
  constructor(t, e, i) {
    this.buffer = t, this.length = e, this.set = i;
  }
  /**
  @internal
  */
  get type() {
    return ot.none;
  }
  /**
  @internal
  */
  toString() {
    let t = [];
    for (let e = 0; e < this.buffer.length; )
      t.push(this.childString(e)), e = this.buffer[e + 3];
    return t.join(",");
  }
  /**
  @internal
  */
  childString(t) {
    let e = this.buffer[t], i = this.buffer[t + 3], n = this.set.types[e], s = n.name;
    if (/\W/.test(s) && !n.isError && (s = JSON.stringify(s)), t += 4, i == t)
      return s;
    let l = [];
    for (; t < i; )
      l.push(this.childString(t)), t = this.buffer[t + 3];
    return s + "(" + l.join(",") + ")";
  }
  /**
  @internal
  */
  findChild(t, e, i, n, s) {
    let { buffer: l } = this, o = -1;
    for (let h = t; h != e && !(Ir(s, n, l[h + 1], l[h + 2]) && (o = h, i > 0)); h = l[h + 3])
      ;
    return o;
  }
  /**
  @internal
  */
  slice(t, e, i) {
    let n = this.buffer, s = new Uint16Array(e - t), l = 0;
    for (let o = t, h = 0; o < e; ) {
      s[h++] = n[o++], s[h++] = n[o++] - i;
      let a = s[h++] = n[o++] - i;
      s[h++] = n[o++] - t, l = Math.max(l, a);
    }
    return new Jt(s, l, this.set);
  }
}
function Ir(r, t, e, i) {
  switch (r) {
    case -2:
      return e < t;
    case -1:
      return i >= t && e < t;
    case 0:
      return e < t && i > t;
    case 1:
      return e <= t && i > t;
    case 2:
      return i > t;
    case 4:
      return !0;
  }
}
function Oe(r, t, e, i) {
  for (var n; r.from == r.to || (e < 1 ? r.from >= t : r.from > t) || (e > -1 ? r.to <= t : r.to < t); ) {
    let l = !i && r instanceof lt && r.index < 0 ? null : r.parent;
    if (!l)
      return r;
    r = l;
  }
  let s = i ? 0 : J.IgnoreOverlays;
  if (i)
    for (let l = r, o = l.parent; o; l = o, o = l.parent)
      l instanceof lt && l.index < 0 && ((n = o.enter(t, e, s)) === null || n === void 0 ? void 0 : n.from) != l.from && (r = o);
  for (; ; ) {
    let l = r.enter(t, e, s);
    if (!l)
      return r;
    r = l;
  }
}
class Hr {
  cursor(t = 0) {
    return new nn(this, t);
  }
  getChild(t, e = null, i = null) {
    let n = gs(this, t, e, i);
    return n.length ? n[0] : null;
  }
  getChildren(t, e = null, i = null) {
    return gs(this, t, e, i);
  }
  resolve(t, e = 0) {
    return Oe(this, t, e, !1);
  }
  resolveInner(t, e = 0) {
    return Oe(this, t, e, !0);
  }
  matchContext(t) {
    return en(this, t);
  }
  enterUnfinishedNodesBefore(t) {
    let e = this.childBefore(t), i = this;
    for (; e; ) {
      let n = e.lastChild;
      if (!n || n.to != e.to)
        break;
      n.type.isError && n.from == n.to ? (i = e, e = n.prevSibling) : e = n;
    }
    return i;
  }
  get node() {
    return this;
  }
  get next() {
    return this.parent;
  }
}
class lt extends Hr {
  constructor(t, e, i, n) {
    super(), this._tree = t, this.from = e, this.index = i, this._parent = n;
  }
  get type() {
    return this._tree.type;
  }
  get name() {
    return this._tree.type.name;
  }
  get to() {
    return this.from + this._tree.length;
  }
  nextChild(t, e, i, n, s = 0) {
    for (let l = this; ; ) {
      for (let { children: o, positions: h } = l._tree, a = e > 0 ? o.length : -1; t != a; t += e) {
        let f = o[t], c = h[t] + l.from;
        if (Ir(n, i, c, c + f.length)) {
          if (f instanceof Jt) {
            if (s & J.ExcludeBuffers)
              continue;
            let u = f.findChild(0, f.buffer.length, e, i - c, n);
            if (u > -1)
              return new It(new jo(l, f, t, c), null, u);
          } else if (s & J.IncludeAnonymous || !f.type.isAnonymous || mn(f)) {
            let u;
            if (!(s & J.IgnoreMounts) && (u = li.get(f)) && !u.overlay)
              return new lt(u.tree, c, t, l);
            let d = new lt(f, c, t, l);
            return s & J.IncludeAnonymous || !d.type.isAnonymous ? d : d.nextChild(e < 0 ? f.children.length - 1 : 0, e, i, n);
          }
        }
      }
      if (s & J.IncludeAnonymous || !l.type.isAnonymous || (l.index >= 0 ? t = l.index + e : t = e < 0 ? -1 : l._parent._tree.children.length, l = l._parent, !l))
        return null;
    }
  }
  get firstChild() {
    return this.nextChild(
      0,
      1,
      0,
      4
      /* Side.DontCare */
    );
  }
  get lastChild() {
    return this.nextChild(
      this._tree.children.length - 1,
      -1,
      0,
      4
      /* Side.DontCare */
    );
  }
  childAfter(t) {
    return this.nextChild(
      0,
      1,
      t,
      2
      /* Side.After */
    );
  }
  childBefore(t) {
    return this.nextChild(
      this._tree.children.length - 1,
      -1,
      t,
      -2
      /* Side.Before */
    );
  }
  enter(t, e, i = 0) {
    let n;
    if (!(i & J.IgnoreOverlays) && (n = li.get(this._tree)) && n.overlay) {
      let s = t - this.from;
      for (let { from: l, to: o } of n.overlay)
        if ((e > 0 ? l <= s : l < s) && (e < 0 ? o >= s : o > s))
          return new lt(n.tree, n.overlay[0].from + this.from, -1, this);
    }
    return this.nextChild(0, 1, t, e, i);
  }
  nextSignificantParent() {
    let t = this;
    for (; t.type.isAnonymous && t._parent; )
      t = t._parent;
    return t;
  }
  get parent() {
    return this._parent ? this._parent.nextSignificantParent() : null;
  }
  get nextSibling() {
    return this._parent && this.index >= 0 ? this._parent.nextChild(
      this.index + 1,
      1,
      0,
      4
      /* Side.DontCare */
    ) : null;
  }
  get prevSibling() {
    return this._parent && this.index >= 0 ? this._parent.nextChild(
      this.index - 1,
      -1,
      0,
      4
      /* Side.DontCare */
    ) : null;
  }
  get tree() {
    return this._tree;
  }
  toTree() {
    return this._tree;
  }
  /**
  @internal
  */
  toString() {
    return this._tree.toString();
  }
}
function gs(r, t, e, i) {
  let n = r.cursor(), s = [];
  if (!n.firstChild())
    return s;
  if (e != null) {
    for (; !n.type.is(e); )
      if (!n.nextSibling())
        return s;
  }
  for (; ; ) {
    if (i != null && n.type.is(i))
      return s;
    if (n.type.is(t) && s.push(n.node), !n.nextSibling())
      return i == null ? s : [];
  }
}
function en(r, t, e = t.length - 1) {
  for (let i = r.parent; e >= 0; i = i.parent) {
    if (!i)
      return !1;
    if (!i.type.isAnonymous) {
      if (t[e] && t[e] != i.name)
        return !1;
      e--;
    }
  }
  return !0;
}
class jo {
  constructor(t, e, i, n) {
    this.parent = t, this.buffer = e, this.index = i, this.start = n;
  }
}
class It extends Hr {
  get name() {
    return this.type.name;
  }
  get from() {
    return this.context.start + this.context.buffer.buffer[this.index + 1];
  }
  get to() {
    return this.context.start + this.context.buffer.buffer[this.index + 2];
  }
  constructor(t, e, i) {
    super(), this.context = t, this._parent = e, this.index = i, this.type = t.buffer.set.types[t.buffer.buffer[i]];
  }
  child(t, e, i) {
    let { buffer: n } = this.context, s = n.findChild(this.index + 4, n.buffer[this.index + 3], t, e - this.context.start, i);
    return s < 0 ? null : new It(this.context, this, s);
  }
  get firstChild() {
    return this.child(
      1,
      0,
      4
      /* Side.DontCare */
    );
  }
  get lastChild() {
    return this.child(
      -1,
      0,
      4
      /* Side.DontCare */
    );
  }
  childAfter(t) {
    return this.child(
      1,
      t,
      2
      /* Side.After */
    );
  }
  childBefore(t) {
    return this.child(
      -1,
      t,
      -2
      /* Side.Before */
    );
  }
  enter(t, e, i = 0) {
    if (i & J.ExcludeBuffers)
      return null;
    let { buffer: n } = this.context, s = n.findChild(this.index + 4, n.buffer[this.index + 3], e > 0 ? 1 : -1, t - this.context.start, e);
    return s < 0 ? null : new It(this.context, this, s);
  }
  get parent() {
    return this._parent || this.context.parent.nextSignificantParent();
  }
  externalSibling(t) {
    return this._parent ? null : this.context.parent.nextChild(
      this.context.index + t,
      t,
      0,
      4
      /* Side.DontCare */
    );
  }
  get nextSibling() {
    let { buffer: t } = this.context, e = t.buffer[this.index + 3];
    return e < (this._parent ? t.buffer[this._parent.index + 3] : t.buffer.length) ? new It(this.context, this._parent, e) : this.externalSibling(1);
  }
  get prevSibling() {
    let { buffer: t } = this.context, e = this._parent ? this._parent.index + 4 : 0;
    return this.index == e ? this.externalSibling(-1) : new It(this.context, this._parent, t.findChild(
      e,
      this.index,
      -1,
      0,
      4
      /* Side.DontCare */
    ));
  }
  get tree() {
    return null;
  }
  toTree() {
    let t = [], e = [], { buffer: i } = this.context, n = this.index + 4, s = i.buffer[this.index + 3];
    if (s > n) {
      let l = i.buffer[this.index + 1];
      t.push(i.slice(n, s, l)), e.push(0);
    }
    return new Z(this.type, t, e, this.to - this.from);
  }
  /**
  @internal
  */
  toString() {
    return this.context.buffer.childString(this.index);
  }
}
function Fr(r) {
  if (!r.length)
    return null;
  if (r.length == 1)
    return r[0];
  let t = 0, e = r[0];
  for (let s = 1; s < r.length; s++) {
    let l = r[s];
    (l.from > e.from || l.to < e.to) && (e = l, t = s);
  }
  let i = e instanceof lt && e.index < 0 ? null : e.parent, n = r.slice();
  return i ? n[t] = i : n.splice(t, 1), new $o(n, e);
}
class $o {
  constructor(t, e) {
    this.heads = t, this.node = e;
  }
  get next() {
    return Fr(this.heads);
  }
}
function Yo(r, t, e) {
  let i = r.resolveInner(t, e), n = null;
  for (let s = i instanceof lt ? i : i.context.parent; s; s = s.parent)
    if (s.index < 0) {
      let l = s.parent;
      (n || (n = [i])).push(l.resolve(t, e)), s = l;
    } else {
      let l = li.get(s.tree);
      if (l && l.overlay && l.overlay[0].from <= t && l.overlay[l.overlay.length - 1].to >= t) {
        let o = new lt(l.tree, l.overlay[0].from + s.from, 0, null);
        (n || (n = [i])).push(Oe(o, t, e, !1));
      }
    }
  return n ? Fr(n) : i;
}
class nn {
  /**
  Shorthand for `.type.name`.
  */
  get name() {
    return this.type.name;
  }
  /**
  @internal
  */
  constructor(t, e = 0) {
    if (this.mode = e, this.buffer = null, this.stack = [], this.index = 0, this.bufferNode = null, t instanceof lt)
      this.yieldNode(t);
    else {
      this._tree = t.context.parent, this.buffer = t.context;
      for (let i = t._parent; i; i = i._parent)
        this.stack.unshift(i.index);
      this.bufferNode = t, this.yieldBuf(t.index);
    }
  }
  yieldNode(t) {
    return t ? (this._tree = t, this.type = t.type, this.from = t.from, this.to = t.to, !0) : !1;
  }
  yieldBuf(t, e) {
    this.index = t;
    let { start: i, buffer: n } = this.buffer;
    return this.type = e || n.set.types[n.buffer[t]], this.from = i + n.buffer[t + 1], this.to = i + n.buffer[t + 2], !0;
  }
  yield(t) {
    return t ? t instanceof lt ? (this.buffer = null, this.yieldNode(t)) : (this.buffer = t.context, this.yieldBuf(t.index, t.type)) : !1;
  }
  /**
  @internal
  */
  toString() {
    return this.buffer ? this.buffer.buffer.childString(this.index) : this._tree.toString();
  }
  /**
  @internal
  */
  enterChild(t, e, i) {
    if (!this.buffer)
      return this.yield(this._tree.nextChild(t < 0 ? this._tree._tree.children.length - 1 : 0, t, e, i, this.mode));
    let { buffer: n } = this.buffer, s = n.findChild(this.index + 4, n.buffer[this.index + 3], t, e - this.buffer.start, i);
    return s < 0 ? !1 : (this.stack.push(this.index), this.yieldBuf(s));
  }
  /**
  Move the cursor to this node's first child. When this returns
  false, the node has no child, and the cursor has not been moved.
  */
  firstChild() {
    return this.enterChild(
      1,
      0,
      4
      /* Side.DontCare */
    );
  }
  /**
  Move the cursor to this node's last child.
  */
  lastChild() {
    return this.enterChild(
      -1,
      0,
      4
      /* Side.DontCare */
    );
  }
  /**
  Move the cursor to the first child that ends after `pos`.
  */
  childAfter(t) {
    return this.enterChild(
      1,
      t,
      2
      /* Side.After */
    );
  }
  /**
  Move to the last child that starts before `pos`.
  */
  childBefore(t) {
    return this.enterChild(
      -1,
      t,
      -2
      /* Side.Before */
    );
  }
  /**
  Move the cursor to the child around `pos`. If side is -1 the
  child may end at that position, when 1 it may start there. This
  will also enter [overlaid](#common.MountedTree.overlay)
  [mounted](#common.NodeProp^mounted) trees unless `overlays` is
  set to false.
  */
  enter(t, e, i = this.mode) {
    return this.buffer ? i & J.ExcludeBuffers ? !1 : this.enterChild(1, t, e) : this.yield(this._tree.enter(t, e, i));
  }
  /**
  Move to the node's parent node, if this isn't the top node.
  */
  parent() {
    if (!this.buffer)
      return this.yieldNode(this.mode & J.IncludeAnonymous ? this._tree._parent : this._tree.parent);
    if (this.stack.length)
      return this.yieldBuf(this.stack.pop());
    let t = this.mode & J.IncludeAnonymous ? this.buffer.parent : this.buffer.parent.nextSignificantParent();
    return this.buffer = null, this.yieldNode(t);
  }
  /**
  @internal
  */
  sibling(t) {
    if (!this.buffer)
      return this._tree._parent ? this.yield(this._tree.index < 0 ? null : this._tree._parent.nextChild(this._tree.index + t, t, 0, 4, this.mode)) : !1;
    let { buffer: e } = this.buffer, i = this.stack.length - 1;
    if (t < 0) {
      let n = i < 0 ? 0 : this.stack[i] + 4;
      if (this.index != n)
        return this.yieldBuf(e.findChild(
          n,
          this.index,
          -1,
          0,
          4
          /* Side.DontCare */
        ));
    } else {
      let n = e.buffer[this.index + 3];
      if (n < (i < 0 ? e.buffer.length : e.buffer[this.stack[i] + 3]))
        return this.yieldBuf(n);
    }
    return i < 0 ? this.yield(this.buffer.parent.nextChild(this.buffer.index + t, t, 0, 4, this.mode)) : !1;
  }
  /**
  Move to this node's next sibling, if any.
  */
  nextSibling() {
    return this.sibling(1);
  }
  /**
  Move to this node's previous sibling, if any.
  */
  prevSibling() {
    return this.sibling(-1);
  }
  atLastNode(t) {
    let e, i, { buffer: n } = this;
    if (n) {
      if (t > 0) {
        if (this.index < n.buffer.buffer.length)
          return !1;
      } else
        for (let s = 0; s < this.index; s++)
          if (n.buffer.buffer[s + 3] < this.index)
            return !1;
      ({ index: e, parent: i } = n);
    } else
      ({ index: e, _parent: i } = this._tree);
    for (; i; { index: e, _parent: i } = i)
      if (e > -1)
        for (let s = e + t, l = t < 0 ? -1 : i._tree.children.length; s != l; s += t) {
          let o = i._tree.children[s];
          if (this.mode & J.IncludeAnonymous || o instanceof Jt || !o.type.isAnonymous || mn(o))
            return !1;
        }
    return !0;
  }
  move(t, e) {
    if (e && this.enterChild(
      t,
      0,
      4
      /* Side.DontCare */
    ))
      return !0;
    for (; ; ) {
      if (this.sibling(t))
        return !0;
      if (this.atLastNode(t) || !this.parent())
        return !1;
    }
  }
  /**
  Move to the next node in a
  [pre-order](https://en.wikipedia.org/wiki/Tree_traversal#Pre-order,_NLR)
  traversal, going from a node to its first child or, if the
  current node is empty or `enter` is false, its next sibling or
  the next sibling of the first parent node that has one.
  */
  next(t = !0) {
    return this.move(1, t);
  }
  /**
  Move to the next node in a last-to-first pre-order traveral. A
  node is followed by its last child or, if it has none, its
  previous sibling or the previous sibling of the first parent
  node that has one.
  */
  prev(t = !0) {
    return this.move(-1, t);
  }
  /**
  Move the cursor to the innermost node that covers `pos`. If
  `side` is -1, it will enter nodes that end at `pos`. If it is 1,
  it will enter nodes that start at `pos`.
  */
  moveTo(t, e = 0) {
    for (; (this.from == this.to || (e < 1 ? this.from >= t : this.from > t) || (e > -1 ? this.to <= t : this.to < t)) && this.parent(); )
      ;
    for (; this.enterChild(1, t, e); )
      ;
    return this;
  }
  /**
  Get a [syntax node](#common.SyntaxNode) at the cursor's current
  position.
  */
  get node() {
    if (!this.buffer)
      return this._tree;
    let t = this.bufferNode, e = null, i = 0;
    if (t && t.context == this.buffer)
      t:
        for (let n = this.index, s = this.stack.length; s >= 0; ) {
          for (let l = t; l; l = l._parent)
            if (l.index == n) {
              if (n == this.index)
                return l;
              e = l, i = s + 1;
              break t;
            }
          n = this.stack[--s];
        }
    for (let n = i; n < this.stack.length; n++)
      e = new It(this.buffer, e, this.stack[n]);
    return this.bufferNode = new It(this.buffer, e, this.index);
  }
  /**
  Get the [tree](#common.Tree) that represents the current node, if
  any. Will return null when the node is in a [tree
  buffer](#common.TreeBuffer).
  */
  get tree() {
    return this.buffer ? null : this._tree._tree;
  }
  /**
  Iterate over the current node and all its descendants, calling
  `enter` when entering a node and `leave`, if given, when leaving
  one. When `enter` returns `false`, any children of that node are
  skipped, and `leave` isn't called for it.
  */
  iterate(t, e) {
    for (let i = 0; ; ) {
      let n = !1;
      if (this.type.isAnonymous || t(this) !== !1) {
        if (this.firstChild()) {
          i++;
          continue;
        }
        this.type.isAnonymous || (n = !0);
      }
      for (; n && e && e(this), n = this.type.isAnonymous, !this.nextSibling(); ) {
        if (!i)
          return;
        this.parent(), i--, n = !0;
      }
    }
  }
  /**
  Test whether the current node matches a given context—a sequence
  of direct parent node names. Empty strings in the context array
  are treated as wildcards.
  */
  matchContext(t) {
    if (!this.buffer)
      return en(this.node, t);
    let { buffer: e } = this.buffer, { types: i } = e.set;
    for (let n = t.length - 1, s = this.stack.length - 1; n >= 0; s--) {
      if (s < 0)
        return en(this.node, t, n);
      let l = i[e.buffer[this.stack[s]]];
      if (!l.isAnonymous) {
        if (t[n] && t[n] != l.name)
          return !1;
        n--;
      }
    }
    return !0;
  }
}
function mn(r) {
  return r.children.some((t) => t instanceof Jt || !t.type.isAnonymous || mn(t));
}
function Go(r) {
  var t;
  let { buffer: e, nodeSet: i, maxBufferLength: n = zo, reused: s = [], minRepeatType: l = i.types.length } = r, o = Array.isArray(e) ? new pn(e, e.length) : e, h = i.types, a = 0, f = 0;
  function c(b, A, v, P, $) {
    let { id: B, start: D, end: V, size: et } = o, ht = f;
    for (; et < 0; )
      if (o.next(), et == -1) {
        let Bt = s[B];
        v.push(Bt), P.push(D - b);
        return;
      } else if (et == -3) {
        a = B;
        return;
      } else if (et == -4) {
        f = B;
        return;
      } else
        throw new RangeError(`Unrecognized record size: ${et}`);
    let Ut = h[B], Pt, bt, wn = D - b;
    if (V - D <= n && (bt = m(o.pos - A, $))) {
      let Bt = new Uint16Array(bt.size - bt.skip), at = o.pos - bt.size, xt = Bt.length;
      for (; o.pos > at; )
        xt = p(bt.start, Bt, xt);
      Pt = new Jt(Bt, V - bt.start, i), wn = bt.start - b;
    } else {
      let Bt = o.pos - et;
      o.next();
      let at = [], xt = [], Vt = B >= l ? B : -1, Xt = 0, Be = V;
      for (; o.pos > Bt; )
        Vt >= 0 && o.id == Vt && o.size >= 0 ? (o.end <= Be - n && (d(at, xt, D, Xt, o.end, Be, Vt, ht), Xt = at.length, Be = o.end), o.next()) : c(D, Bt, at, xt, Vt);
      if (Vt >= 0 && Xt > 0 && Xt < at.length && d(at, xt, D, Xt, D, Be, Vt, ht), at.reverse(), xt.reverse(), Vt > -1 && Xt > 0) {
        let bn = u(Ut);
        Pt = yn(Ut, at, xt, 0, at.length, 0, V - D, bn, bn);
      } else
        Pt = g(Ut, at, xt, V - D, ht - V);
    }
    v.push(Pt), P.push(wn);
  }
  function u(b) {
    return (A, v, P) => {
      let $ = 0, B = A.length - 1, D, V;
      if (B >= 0 && (D = A[B]) instanceof Z) {
        if (!B && D.type == b && D.length == P)
          return D;
        (V = D.prop(F.lookAhead)) && ($ = v[B] + D.length + V);
      }
      return g(b, A, v, P, $);
    };
  }
  function d(b, A, v, P, $, B, D, V) {
    let et = [], ht = [];
    for (; b.length > P; )
      et.push(b.pop()), ht.push(A.pop() + v - $);
    b.push(g(i.types[D], et, ht, B - $, V - B)), A.push($ - v);
  }
  function g(b, A, v, P, $ = 0, B) {
    if (a) {
      let D = [F.contextHash, a];
      B = B ? [D].concat(B) : [D];
    }
    if ($ > 25) {
      let D = [F.lookAhead, $];
      B = B ? [D].concat(B) : [D];
    }
    return new Z(b, A, v, P, B);
  }
  function m(b, A) {
    let v = o.fork(), P = 0, $ = 0, B = 0, D = v.end - n, V = { size: 0, start: 0, skip: 0 };
    t:
      for (let et = v.pos - b; v.pos > et; ) {
        let ht = v.size;
        if (v.id == A && ht >= 0) {
          V.size = P, V.start = $, V.skip = B, B += 4, P += 4, v.next();
          continue;
        }
        let Ut = v.pos - ht;
        if (ht < 0 || Ut < et || v.start < D)
          break;
        let Pt = v.id >= l ? 4 : 0, bt = v.start;
        for (v.next(); v.pos > Ut; ) {
          if (v.size < 0)
            if (v.size == -3)
              Pt += 4;
            else
              break t;
          else
            v.id >= l && (Pt += 4);
          v.next();
        }
        $ = bt, P += ht, B += Pt;
      }
    return (A < 0 || P == b) && (V.size = P, V.start = $, V.skip = B), V.size > 4 ? V : void 0;
  }
  function p(b, A, v) {
    let { id: P, start: $, end: B, size: D } = o;
    if (o.next(), D >= 0 && P < l) {
      let V = v;
      if (D > 4) {
        let et = o.pos - (D - 4);
        for (; o.pos > et; )
          v = p(b, A, v);
      }
      A[--v] = V, A[--v] = B - b, A[--v] = $ - b, A[--v] = P;
    } else
      D == -3 ? a = P : D == -4 && (f = P);
    return v;
  }
  let y = [], S = [];
  for (; o.pos > 0; )
    c(r.start || 0, r.bufferStart || 0, y, S, -1);
  let C = (t = r.length) !== null && t !== void 0 ? t : y.length ? S[0] + y[0].length : 0;
  return new Z(h[r.topID], y.reverse(), S.reverse(), C);
}
const ps = /* @__PURE__ */ new WeakMap();
function _e(r, t) {
  if (!r.isAnonymous || t instanceof Jt || t.type != r)
    return 1;
  let e = ps.get(t);
  if (e == null) {
    e = 1;
    for (let i of t.children) {
      if (i.type != r || !(i instanceof Z)) {
        e = 1;
        break;
      }
      e += _e(r, i);
    }
    ps.set(t, e);
  }
  return e;
}
function yn(r, t, e, i, n, s, l, o, h) {
  let a = 0;
  for (let g = i; g < n; g++)
    a += _e(r, t[g]);
  let f = Math.ceil(
    a * 1.5 / 8
    /* Balance.BranchFactor */
  ), c = [], u = [];
  function d(g, m, p, y, S) {
    for (let C = p; C < y; ) {
      let b = C, A = m[C], v = _e(r, g[C]);
      for (C++; C < y; C++) {
        let P = _e(r, g[C]);
        if (v + P >= f)
          break;
        v += P;
      }
      if (C == b + 1) {
        if (v > f) {
          let P = g[b];
          d(P.children, P.positions, 0, P.children.length, m[b] + S);
          continue;
        }
        c.push(g[b]);
      } else {
        let P = m[C - 1] + g[C - 1].length - A;
        c.push(yn(r, g, m, b, C, A, P, null, h));
      }
      u.push(A + S - s);
    }
  }
  return d(t, e, i, n, 0), (o || h)(c, u, l);
}
class Yt {
  /**
  Construct a tree fragment. You'll usually want to use
  [`addTree`](#common.TreeFragment^addTree) and
  [`applyChanges`](#common.TreeFragment^applyChanges) instead of
  calling this directly.
  */
  constructor(t, e, i, n, s = !1, l = !1) {
    this.from = t, this.to = e, this.tree = i, this.offset = n, this.open = (s ? 1 : 0) | (l ? 2 : 0);
  }
  /**
  Whether the start of the fragment represents the start of a
  parse, or the end of a change. (In the second case, it may not
  be safe to reuse some nodes at the start, depending on the
  parsing algorithm.)
  */
  get openStart() {
    return (this.open & 1) > 0;
  }
  /**
  Whether the end of the fragment represents the end of a
  full-document parse, or the start of a change.
  */
  get openEnd() {
    return (this.open & 2) > 0;
  }
  /**
  Create a set of fragments from a freshly parsed tree, or update
  an existing set of fragments by replacing the ones that overlap
  with a tree with content from the new tree. When `partial` is
  true, the parse is treated as incomplete, and the resulting
  fragment has [`openEnd`](#common.TreeFragment.openEnd) set to
  true.
  */
  static addTree(t, e = [], i = !1) {
    let n = [new Yt(0, t.length, t, 0, !1, i)];
    for (let s of e)
      s.to > t.length && n.push(s);
    return n;
  }
  /**
  Apply a set of edits to an array of fragments, removing or
  splitting fragments as necessary to remove edited ranges, and
  adjusting offsets for fragments that moved.
  */
  static applyChanges(t, e, i = 128) {
    if (!e.length)
      return t;
    let n = [], s = 1, l = t.length ? t[0] : null;
    for (let o = 0, h = 0, a = 0; ; o++) {
      let f = o < e.length ? e[o] : null, c = f ? f.fromA : 1e9;
      if (c - h >= i)
        for (; l && l.from < c; ) {
          let u = l;
          if (h >= u.from || c <= u.to || a) {
            let d = Math.max(u.from, h) - a, g = Math.min(u.to, c) - a;
            u = d >= g ? null : new Yt(d, g, u.tree, u.offset + a, o > 0, !!f);
          }
          if (u && n.push(u), l.to > c)
            break;
          l = s < t.length ? t[s++] : null;
        }
      if (!f)
        break;
      h = f.toA, a = f.toA - f.toB;
    }
    return n;
  }
}
class Jo {
  /**
  Start a parse, returning a [partial parse](#common.PartialParse)
  object. [`fragments`](#common.TreeFragment) can be passed in to
  make the parse incremental.
  
  By default, the entire input is parsed. You can pass `ranges`,
  which should be a sorted array of non-empty, non-overlapping
  ranges, to parse only those ranges. The tree returned in that
  case will start at `ranges[0].from`.
  */
  startParse(t, e, i) {
    return typeof t == "string" && (t = new Uo(t)), i = i ? i.length ? i.map((n) => new ki(n.from, n.to)) : [new ki(0, 0)] : [new ki(0, t.length)], this.createParse(t, e || [], i);
  }
  /**
  Run a full parse, returning the resulting tree.
  */
  parse(t, e, i) {
    let n = this.startParse(t, e, i);
    for (; ; ) {
      let s = n.advance();
      if (s)
        return s;
    }
  }
}
class Uo {
  constructor(t) {
    this.string = t;
  }
  get length() {
    return this.string.length;
  }
  chunk(t) {
    return this.string.slice(t);
  }
  get lineChunks() {
    return !1;
  }
  read(t, e) {
    return this.string.slice(t, e);
  }
}
new F({ perNode: !0 });
let Xo = 0;
class pt {
  /**
  @internal
  */
  constructor(t, e, i) {
    this.set = t, this.base = e, this.modified = i, this.id = Xo++;
  }
  /**
  Define a new tag. If `parent` is given, the tag is treated as a
  sub-tag of that parent, and
  [highlighters](#highlight.tagHighlighter) that don't mention
  this tag will try to fall back to the parent tag (or grandparent
  tag, etc).
  */
  static define(t) {
    if (t != null && t.base)
      throw new Error("Can not derive from a modified tag");
    let e = new pt([], null, []);
    if (e.set.push(e), t)
      for (let i of t.set)
        e.set.push(i);
    return e;
  }
  /**
  Define a tag _modifier_, which is a function that, given a tag,
  will return a tag that is a subtag of the original. Applying the
  same modifier to a twice tag will return the same value (`m1(t1)
  == m1(t1)`) and applying multiple modifiers will, regardless or
  order, produce the same tag (`m1(m2(t1)) == m2(m1(t1))`).
  
  When multiple modifiers are applied to a given base tag, each
  smaller set of modifiers is registered as a parent, so that for
  example `m1(m2(m3(t1)))` is a subtype of `m1(m2(t1))`,
  `m1(m3(t1)`, and so on.
  */
  static defineModifier() {
    let t = new oi();
    return (e) => e.modified.indexOf(t) > -1 ? e : oi.get(e.base || e, e.modified.concat(t).sort((i, n) => i.id - n.id));
  }
}
let _o = 0;
class oi {
  constructor() {
    this.instances = [], this.id = _o++;
  }
  static get(t, e) {
    if (!e.length)
      return t;
    let i = e[0].instances.find((o) => o.base == t && Zo(e, o.modified));
    if (i)
      return i;
    let n = [], s = new pt(n, t, e);
    for (let o of e)
      o.instances.push(s);
    let l = Qo(e);
    for (let o of t.set)
      if (!o.modified.length)
        for (let h of l)
          n.push(oi.get(o, h));
    return s;
  }
}
function Zo(r, t) {
  return r.length == t.length && r.every((e, i) => e == t[i]);
}
function Qo(r) {
  let t = [[]];
  for (let e = 0; e < r.length; e++)
    for (let i = 0, n = t.length; i < n; i++)
      t.push(t[i].concat(r[e]));
  return t.sort((e, i) => i.length - e.length);
}
function th(r) {
  let t = /* @__PURE__ */ Object.create(null);
  for (let e in r) {
    let i = r[e];
    Array.isArray(i) || (i = [i]);
    for (let n of e.split(" "))
      if (n) {
        let s = [], l = 2, o = n;
        for (let c = 0; ; ) {
          if (o == "..." && c > 0 && c + 3 == n.length) {
            l = 1;
            break;
          }
          let u = /^"(?:[^"\\]|\\.)*?"|[^\/!]+/.exec(o);
          if (!u)
            throw new RangeError("Invalid path: " + n);
          if (s.push(u[0] == "*" ? "" : u[0][0] == '"' ? JSON.parse(u[0]) : u[0]), c += u[0].length, c == n.length)
            break;
          let d = n[c++];
          if (c == n.length && d == "!") {
            l = 0;
            break;
          }
          if (d != "/")
            throw new RangeError("Invalid path: " + n);
          o = n.slice(c);
        }
        let h = s.length - 1, a = s[h];
        if (!a)
          throw new RangeError("Invalid path: " + n);
        let f = new sn(i, l, h > 0 ? s.slice(0, h) : null);
        t[a] = f.sort(t[a]);
      }
  }
  return eh.add(t);
}
const eh = new F();
class sn {
  constructor(t, e, i, n) {
    this.tags = t, this.mode = e, this.context = i, this.next = n;
  }
  get opaque() {
    return this.mode == 0;
  }
  get inherit() {
    return this.mode == 1;
  }
  sort(t) {
    return !t || t.depth < this.depth ? (this.next = t, this) : (t.next = this.sort(t.next), t);
  }
  get depth() {
    return this.context ? this.context.length : 0;
  }
}
sn.empty = new sn([], 2, null);
function ih(r, t) {
  let e = /* @__PURE__ */ Object.create(null);
  for (let s of r)
    if (!Array.isArray(s.tag))
      e[s.tag.id] = s.class;
    else
      for (let l of s.tag)
        e[l.id] = s.class;
  let { scope: i, all: n = null } = t || {};
  return {
    style: (s) => {
      let l = n;
      for (let o of s)
        for (let h of o.set) {
          let a = e[h.id];
          if (a) {
            l = l ? l + " " + a : a;
            break;
          }
        }
      return l;
    },
    scope: i
  };
}
const w = pt.define, qe = w(), Rt = w(), ms = w(Rt), ys = w(Rt), Et = w(), Ke = w(Et), vi = w(Et), gt = w(), Wt = w(gt), ut = w(), dt = w(), rn = w(), ue = w(rn), je = w(), M = {
  /**
  A comment.
  */
  comment: qe,
  /**
  A line [comment](#highlight.tags.comment).
  */
  lineComment: w(qe),
  /**
  A block [comment](#highlight.tags.comment).
  */
  blockComment: w(qe),
  /**
  A documentation [comment](#highlight.tags.comment).
  */
  docComment: w(qe),
  /**
  Any kind of identifier.
  */
  name: Rt,
  /**
  The [name](#highlight.tags.name) of a variable.
  */
  variableName: w(Rt),
  /**
  A type [name](#highlight.tags.name).
  */
  typeName: ms,
  /**
  A tag name (subtag of [`typeName`](#highlight.tags.typeName)).
  */
  tagName: w(ms),
  /**
  A property or field [name](#highlight.tags.name).
  */
  propertyName: ys,
  /**
  An attribute name (subtag of [`propertyName`](#highlight.tags.propertyName)).
  */
  attributeName: w(ys),
  /**
  The [name](#highlight.tags.name) of a class.
  */
  className: w(Rt),
  /**
  A label [name](#highlight.tags.name).
  */
  labelName: w(Rt),
  /**
  A namespace [name](#highlight.tags.name).
  */
  namespace: w(Rt),
  /**
  The [name](#highlight.tags.name) of a macro.
  */
  macroName: w(Rt),
  /**
  A literal value.
  */
  literal: Et,
  /**
  A string [literal](#highlight.tags.literal).
  */
  string: Ke,
  /**
  A documentation [string](#highlight.tags.string).
  */
  docString: w(Ke),
  /**
  A character literal (subtag of [string](#highlight.tags.string)).
  */
  character: w(Ke),
  /**
  An attribute value (subtag of [string](#highlight.tags.string)).
  */
  attributeValue: w(Ke),
  /**
  A number [literal](#highlight.tags.literal).
  */
  number: vi,
  /**
  An integer [number](#highlight.tags.number) literal.
  */
  integer: w(vi),
  /**
  A floating-point [number](#highlight.tags.number) literal.
  */
  float: w(vi),
  /**
  A boolean [literal](#highlight.tags.literal).
  */
  bool: w(Et),
  /**
  Regular expression [literal](#highlight.tags.literal).
  */
  regexp: w(Et),
  /**
  An escape [literal](#highlight.tags.literal), for example a
  backslash escape in a string.
  */
  escape: w(Et),
  /**
  A color [literal](#highlight.tags.literal).
  */
  color: w(Et),
  /**
  A URL [literal](#highlight.tags.literal).
  */
  url: w(Et),
  /**
  A language keyword.
  */
  keyword: ut,
  /**
  The [keyword](#highlight.tags.keyword) for the self or this
  object.
  */
  self: w(ut),
  /**
  The [keyword](#highlight.tags.keyword) for null.
  */
  null: w(ut),
  /**
  A [keyword](#highlight.tags.keyword) denoting some atomic value.
  */
  atom: w(ut),
  /**
  A [keyword](#highlight.tags.keyword) that represents a unit.
  */
  unit: w(ut),
  /**
  A modifier [keyword](#highlight.tags.keyword).
  */
  modifier: w(ut),
  /**
  A [keyword](#highlight.tags.keyword) that acts as an operator.
  */
  operatorKeyword: w(ut),
  /**
  A control-flow related [keyword](#highlight.tags.keyword).
  */
  controlKeyword: w(ut),
  /**
  A [keyword](#highlight.tags.keyword) that defines something.
  */
  definitionKeyword: w(ut),
  /**
  A [keyword](#highlight.tags.keyword) related to defining or
  interfacing with modules.
  */
  moduleKeyword: w(ut),
  /**
  An operator.
  */
  operator: dt,
  /**
  An [operator](#highlight.tags.operator) that dereferences something.
  */
  derefOperator: w(dt),
  /**
  Arithmetic-related [operator](#highlight.tags.operator).
  */
  arithmeticOperator: w(dt),
  /**
  Logical [operator](#highlight.tags.operator).
  */
  logicOperator: w(dt),
  /**
  Bit [operator](#highlight.tags.operator).
  */
  bitwiseOperator: w(dt),
  /**
  Comparison [operator](#highlight.tags.operator).
  */
  compareOperator: w(dt),
  /**
  [Operator](#highlight.tags.operator) that updates its operand.
  */
  updateOperator: w(dt),
  /**
  [Operator](#highlight.tags.operator) that defines something.
  */
  definitionOperator: w(dt),
  /**
  Type-related [operator](#highlight.tags.operator).
  */
  typeOperator: w(dt),
  /**
  Control-flow [operator](#highlight.tags.operator).
  */
  controlOperator: w(dt),
  /**
  Program or markup punctuation.
  */
  punctuation: rn,
  /**
  [Punctuation](#highlight.tags.punctuation) that separates
  things.
  */
  separator: w(rn),
  /**
  Bracket-style [punctuation](#highlight.tags.punctuation).
  */
  bracket: ue,
  /**
  Angle [brackets](#highlight.tags.bracket) (usually `<` and `>`
  tokens).
  */
  angleBracket: w(ue),
  /**
  Square [brackets](#highlight.tags.bracket) (usually `[` and `]`
  tokens).
  */
  squareBracket: w(ue),
  /**
  Parentheses (usually `(` and `)` tokens). Subtag of
  [bracket](#highlight.tags.bracket).
  */
  paren: w(ue),
  /**
  Braces (usually `{` and `}` tokens). Subtag of
  [bracket](#highlight.tags.bracket).
  */
  brace: w(ue),
  /**
  Content, for example plain text in XML or markup documents.
  */
  content: gt,
  /**
  [Content](#highlight.tags.content) that represents a heading.
  */
  heading: Wt,
  /**
  A level 1 [heading](#highlight.tags.heading).
  */
  heading1: w(Wt),
  /**
  A level 2 [heading](#highlight.tags.heading).
  */
  heading2: w(Wt),
  /**
  A level 3 [heading](#highlight.tags.heading).
  */
  heading3: w(Wt),
  /**
  A level 4 [heading](#highlight.tags.heading).
  */
  heading4: w(Wt),
  /**
  A level 5 [heading](#highlight.tags.heading).
  */
  heading5: w(Wt),
  /**
  A level 6 [heading](#highlight.tags.heading).
  */
  heading6: w(Wt),
  /**
  A prose separator (such as a horizontal rule).
  */
  contentSeparator: w(gt),
  /**
  [Content](#highlight.tags.content) that represents a list.
  */
  list: w(gt),
  /**
  [Content](#highlight.tags.content) that represents a quote.
  */
  quote: w(gt),
  /**
  [Content](#highlight.tags.content) that is emphasized.
  */
  emphasis: w(gt),
  /**
  [Content](#highlight.tags.content) that is styled strong.
  */
  strong: w(gt),
  /**
  [Content](#highlight.tags.content) that is part of a link.
  */
  link: w(gt),
  /**
  [Content](#highlight.tags.content) that is styled as code or
  monospace.
  */
  monospace: w(gt),
  /**
  [Content](#highlight.tags.content) that has a strike-through
  style.
  */
  strikethrough: w(gt),
  /**
  Inserted text in a change-tracking format.
  */
  inserted: w(),
  /**
  Deleted text.
  */
  deleted: w(),
  /**
  Changed text.
  */
  changed: w(),
  /**
  An invalid or unsyntactic element.
  */
  invalid: w(),
  /**
  Metadata or meta-instruction.
  */
  meta: je,
  /**
  [Metadata](#highlight.tags.meta) that applies to the entire
  document.
  */
  documentMeta: w(je),
  /**
  [Metadata](#highlight.tags.meta) that annotates or adds
  attributes to a given syntactic element.
  */
  annotation: w(je),
  /**
  Processing instruction or preprocessor directive. Subtag of
  [meta](#highlight.tags.meta).
  */
  processingInstruction: w(je),
  /**
  [Modifier](#highlight.Tag^defineModifier) that indicates that a
  given element is being defined. Expected to be used with the
  various [name](#highlight.tags.name) tags.
  */
  definition: pt.defineModifier(),
  /**
  [Modifier](#highlight.Tag^defineModifier) that indicates that
  something is constant. Mostly expected to be used with
  [variable names](#highlight.tags.variableName).
  */
  constant: pt.defineModifier(),
  /**
  [Modifier](#highlight.Tag^defineModifier) used to indicate that
  a [variable](#highlight.tags.variableName) or [property
  name](#highlight.tags.propertyName) is being called or defined
  as a function.
  */
  function: pt.defineModifier(),
  /**
  [Modifier](#highlight.Tag^defineModifier) that can be applied to
  [names](#highlight.tags.name) to indicate that they belong to
  the language's standard environment.
  */
  standard: pt.defineModifier(),
  /**
  [Modifier](#highlight.Tag^defineModifier) that indicates a given
  [names](#highlight.tags.name) is local to some scope.
  */
  local: pt.defineModifier(),
  /**
  A generic variant [modifier](#highlight.Tag^defineModifier) that
  can be used to tag language-specific alternative variants of
  some common tag. It is recommended for themes to define special
  forms of at least the [string](#highlight.tags.string) and
  [variable name](#highlight.tags.variableName) tags, since those
  come up a lot.
  */
  special: pt.defineModifier()
};
ih([
  { tag: M.link, class: "tok-link" },
  { tag: M.heading, class: "tok-heading" },
  { tag: M.emphasis, class: "tok-emphasis" },
  { tag: M.strong, class: "tok-strong" },
  { tag: M.keyword, class: "tok-keyword" },
  { tag: M.atom, class: "tok-atom" },
  { tag: M.bool, class: "tok-bool" },
  { tag: M.url, class: "tok-url" },
  { tag: M.labelName, class: "tok-labelName" },
  { tag: M.inserted, class: "tok-inserted" },
  { tag: M.deleted, class: "tok-deleted" },
  { tag: M.literal, class: "tok-literal" },
  { tag: M.string, class: "tok-string" },
  { tag: M.number, class: "tok-number" },
  { tag: [M.regexp, M.escape, M.special(M.string)], class: "tok-string2" },
  { tag: M.variableName, class: "tok-variableName" },
  { tag: M.local(M.variableName), class: "tok-variableName tok-local" },
  { tag: M.definition(M.variableName), class: "tok-variableName tok-definition" },
  { tag: M.special(M.variableName), class: "tok-variableName2" },
  { tag: M.definition(M.propertyName), class: "tok-propertyName tok-definition" },
  { tag: M.typeName, class: "tok-typeName" },
  { tag: M.namespace, class: "tok-namespace" },
  { tag: M.className, class: "tok-className" },
  { tag: M.macroName, class: "tok-macroName" },
  { tag: M.propertyName, class: "tok-propertyName" },
  { tag: M.operator, class: "tok-operator" },
  { tag: M.comment, class: "tok-comment" },
  { tag: M.meta, class: "tok-meta" },
  { tag: M.invalid, class: "tok-invalid" },
  { tag: M.punctuation, class: "tok-punctuation" }
]);
var Si;
const $e = /* @__PURE__ */ new F(), nh = /* @__PURE__ */ new F();
class St {
  /**
  Construct a language object. If you need to invoke this
  directly, first define a data facet with
  [`defineLanguageFacet`](https://codemirror.net/6/docs/ref/#language.defineLanguageFacet), and then
  configure your parser to [attach](https://codemirror.net/6/docs/ref/#language.languageDataProp) it
  to the language's outer syntax node.
  */
  constructor(t, e, i = [], n = "") {
    this.data = t, this.name = n, E.prototype.hasOwnProperty("tree") || Object.defineProperty(E.prototype, "tree", { get() {
      return le(this);
    } }), this.parser = e, this.extension = [
      he.of(this),
      E.languageData.of((s, l, o) => {
        let h = ws(s, l, o), a = h.type.prop($e);
        if (!a)
          return [];
        let f = s.facet(a), c = h.type.prop(nh);
        if (c) {
          let u = h.resolve(l - h.from, o);
          for (let d of c)
            if (d.test(u, s)) {
              let g = s.facet(d.facet);
              return d.type == "replace" ? g : g.concat(f);
            }
        }
        return f;
      })
    ].concat(i);
  }
  /**
  Query whether this language is active at the given position.
  */
  isActiveAt(t, e, i = -1) {
    return ws(t, e, i).type.prop($e) == this.data;
  }
  /**
  Find the document regions that were parsed using this language.
  The returned regions will _include_ any nested languages rooted
  in this language, when those exist.
  */
  findRegions(t) {
    let e = t.facet(he);
    if ((e == null ? void 0 : e.data) == this.data)
      return [{ from: 0, to: t.doc.length }];
    if (!e || !e.allowsNesting)
      return [];
    let i = [], n = (s, l) => {
      if (s.prop($e) == this.data) {
        i.push({ from: l, to: l + s.length });
        return;
      }
      let o = s.prop(F.mounted);
      if (o) {
        if (o.tree.prop($e) == this.data) {
          if (o.overlay)
            for (let h of o.overlay)
              i.push({ from: h.from + l, to: h.to + l });
          else
            i.push({ from: l, to: l + s.length });
          return;
        } else if (o.overlay) {
          let h = i.length;
          if (n(o.tree, o.overlay[0].from + l), i.length > h)
            return;
        }
      }
      for (let h = 0; h < s.children.length; h++) {
        let a = s.children[h];
        a instanceof Z && n(a, s.positions[h] + l);
      }
    };
    return n(le(t), 0), i;
  }
  /**
  Indicates whether this language allows nested languages. The
  default implementation returns true.
  */
  get allowsNesting() {
    return !0;
  }
}
St.setState = /* @__PURE__ */ q.define();
function ws(r, t, e) {
  let i = r.facet(he), n = le(r).topNode;
  if (!i || i.allowsNesting)
    for (let s = n; s; s = s.enter(t, e, J.ExcludeBuffers))
      s.type.isTop && (n = s);
  return n;
}
function le(r) {
  let t = r.field(St.state, !1);
  return t ? t.tree : Z.empty;
}
class sh {
  /**
  Create an input object for the given document.
  */
  constructor(t) {
    this.doc = t, this.cursorPos = 0, this.string = "", this.cursor = t.iter();
  }
  get length() {
    return this.doc.length;
  }
  syncTo(t) {
    return this.string = this.cursor.next(t - this.cursorPos).value, this.cursorPos = t + this.string.length, this.cursorPos - this.string.length;
  }
  chunk(t) {
    return this.syncTo(t), this.string;
  }
  get lineChunks() {
    return !0;
  }
  read(t, e) {
    let i = this.cursorPos - this.string.length;
    return t < i || e >= this.cursorPos ? this.doc.sliceString(t, e) : this.string.slice(t - i, e - i);
  }
}
let de = null;
class hi {
  constructor(t, e, i = [], n, s, l, o, h) {
    this.parser = t, this.state = e, this.fragments = i, this.tree = n, this.treeLen = s, this.viewport = l, this.skipped = o, this.scheduleOn = h, this.parse = null, this.tempSkipped = [];
  }
  /**
  @internal
  */
  static create(t, e, i) {
    return new hi(t, e, [], Z.empty, 0, i, [], null);
  }
  startParse() {
    return this.parser.startParse(new sh(this.state.doc), this.fragments);
  }
  /**
  @internal
  */
  work(t, e) {
    return e != null && e >= this.state.doc.length && (e = void 0), this.tree != Z.empty && this.isDone(e ?? this.state.doc.length) ? (this.takeTree(), !0) : this.withContext(() => {
      var i;
      if (typeof t == "number") {
        let n = Date.now() + t;
        t = () => Date.now() > n;
      }
      for (this.parse || (this.parse = this.startParse()), e != null && (this.parse.stoppedAt == null || this.parse.stoppedAt > e) && e < this.state.doc.length && this.parse.stopAt(e); ; ) {
        let n = this.parse.advance();
        if (n)
          if (this.fragments = this.withoutTempSkipped(Yt.addTree(n, this.fragments, this.parse.stoppedAt != null)), this.treeLen = (i = this.parse.stoppedAt) !== null && i !== void 0 ? i : this.state.doc.length, this.tree = n, this.parse = null, this.treeLen < (e ?? this.state.doc.length))
            this.parse = this.startParse();
          else
            return !0;
        if (t())
          return !1;
      }
    });
  }
  /**
  @internal
  */
  takeTree() {
    let t, e;
    this.parse && (t = this.parse.parsedPos) >= this.treeLen && ((this.parse.stoppedAt == null || this.parse.stoppedAt > t) && this.parse.stopAt(t), this.withContext(() => {
      for (; !(e = this.parse.advance()); )
        ;
    }), this.treeLen = t, this.tree = e, this.fragments = this.withoutTempSkipped(Yt.addTree(this.tree, this.fragments, !0)), this.parse = null);
  }
  withContext(t) {
    let e = de;
    de = this;
    try {
      return t();
    } finally {
      de = e;
    }
  }
  withoutTempSkipped(t) {
    for (let e; e = this.tempSkipped.pop(); )
      t = bs(t, e.from, e.to);
    return t;
  }
  /**
  @internal
  */
  changes(t, e) {
    let { fragments: i, tree: n, treeLen: s, viewport: l, skipped: o } = this;
    if (this.takeTree(), !t.empty) {
      let h = [];
      if (t.iterChangedRanges((a, f, c, u) => h.push({ fromA: a, toA: f, fromB: c, toB: u })), i = Yt.applyChanges(i, h), n = Z.empty, s = 0, l = { from: t.mapPos(l.from, -1), to: t.mapPos(l.to, 1) }, this.skipped.length) {
        o = [];
        for (let a of this.skipped) {
          let f = t.mapPos(a.from, 1), c = t.mapPos(a.to, -1);
          f < c && o.push({ from: f, to: c });
        }
      }
    }
    return new hi(this.parser, e, i, n, s, l, o, this.scheduleOn);
  }
  /**
  @internal
  */
  updateViewport(t) {
    if (this.viewport.from == t.from && this.viewport.to == t.to)
      return !1;
    this.viewport = t;
    let e = this.skipped.length;
    for (let i = 0; i < this.skipped.length; i++) {
      let { from: n, to: s } = this.skipped[i];
      n < t.to && s > t.from && (this.fragments = bs(this.fragments, n, s), this.skipped.splice(i--, 1));
    }
    return this.skipped.length >= e ? !1 : (this.reset(), !0);
  }
  /**
  @internal
  */
  reset() {
    this.parse && (this.takeTree(), this.parse = null);
  }
  /**
  Notify the parse scheduler that the given region was skipped
  because it wasn't in view, and the parse should be restarted
  when it comes into view.
  */
  skipUntilInView(t, e) {
    this.skipped.push({ from: t, to: e });
  }
  /**
  Returns a parser intended to be used as placeholder when
  asynchronously loading a nested parser. It'll skip its input and
  mark it as not-really-parsed, so that the next update will parse
  it again.
  
  When `until` is given, a reparse will be scheduled when that
  promise resolves.
  */
  static getSkippingParser(t) {
    return new class extends Jo {
      createParse(e, i, n) {
        let s = n[0].from, l = n[n.length - 1].to;
        return {
          parsedPos: s,
          advance() {
            let h = de;
            if (h) {
              for (let a of n)
                h.tempSkipped.push(a);
              t && (h.scheduleOn = h.scheduleOn ? Promise.all([h.scheduleOn, t]) : t);
            }
            return this.parsedPos = l, new Z(ot.none, [], [], l - s);
          },
          stoppedAt: null,
          stopAt() {
          }
        };
      }
    }();
  }
  /**
  @internal
  */
  isDone(t) {
    t = Math.min(t, this.state.doc.length);
    let e = this.fragments;
    return this.treeLen >= t && e.length && e[0].from == 0 && e[0].to >= t;
  }
  /**
  Get the context for the current parse, or `null` if no editor
  parse is in progress.
  */
  static get() {
    return de;
  }
}
function bs(r, t, e) {
  return Yt.applyChanges(r, [{ fromA: t, toA: e, fromB: t, toB: e }]);
}
class oe {
  constructor(t) {
    this.context = t, this.tree = t.tree;
  }
  apply(t) {
    if (!t.docChanged && this.tree == this.context.tree)
      return this;
    let e = this.context.changes(t.changes, t.state), i = this.context.treeLen == t.startState.doc.length ? void 0 : Math.max(t.changes.mapPos(this.context.treeLen), e.viewport.to);
    return e.work(20, i) || e.takeTree(), new oe(e);
  }
  static init(t) {
    let e = Math.min(3e3, t.doc.length), i = hi.create(t.facet(he).parser, t, { from: 0, to: e });
    return i.work(20, e) || i.takeTree(), new oe(i);
  }
}
St.state = /* @__PURE__ */ Ft.define({
  create: oe.init,
  update(r, t) {
    for (let e of t.effects)
      if (e.is(St.setState))
        return e.value;
    return t.startState.facet(he) != t.state.facet(he) ? oe.init(t.state) : r.apply(t);
  }
});
let Vr = (r) => {
  let t = setTimeout(
    () => r(),
    500
    /* Work.MaxPause */
  );
  return () => clearTimeout(t);
};
typeof requestIdleCallback < "u" && (Vr = (r) => {
  let t = -1, e = setTimeout(
    () => {
      t = requestIdleCallback(r, {
        timeout: 500 - 100
        /* Work.MinPause */
      });
    },
    100
    /* Work.MinPause */
  );
  return () => t < 0 ? clearTimeout(e) : cancelIdleCallback(t);
});
const Ai = typeof navigator < "u" && (!((Si = navigator.scheduling) === null || Si === void 0) && Si.isInputPending) ? () => navigator.scheduling.isInputPending() : null, rh = /* @__PURE__ */ Tt.fromClass(class {
  constructor(t) {
    this.view = t, this.working = null, this.workScheduled = 0, this.chunkEnd = -1, this.chunkBudget = -1, this.work = this.work.bind(this), this.scheduleWork();
  }
  update(t) {
    let e = this.view.state.field(St.state).context;
    (e.updateViewport(t.view.viewport) || this.view.viewport.to > e.treeLen) && this.scheduleWork(), (t.docChanged || t.selectionSet) && (this.view.hasFocus && (this.chunkBudget += 50), this.scheduleWork()), this.checkAsyncSchedule(e);
  }
  scheduleWork() {
    if (this.working)
      return;
    let { state: t } = this.view, e = t.field(St.state);
    (e.tree != e.context.tree || !e.context.isDone(t.doc.length)) && (this.working = Vr(this.work));
  }
  work(t) {
    this.working = null;
    let e = Date.now();
    if (this.chunkEnd < e && (this.chunkEnd < 0 || this.view.hasFocus) && (this.chunkEnd = e + 3e4, this.chunkBudget = 3e3), this.chunkBudget <= 0)
      return;
    let { state: i, viewport: { to: n } } = this.view, s = i.field(St.state);
    if (s.tree == s.context.tree && s.context.isDone(
      n + 1e5
      /* Work.MaxParseAhead */
    ))
      return;
    let l = Date.now() + Math.min(this.chunkBudget, 100, t && !Ai ? Math.max(25, t.timeRemaining() - 5) : 1e9), o = s.context.treeLen < n && i.doc.length > n + 1e3, h = s.context.work(() => Ai && Ai() || Date.now() > l, n + (o ? 0 : 1e5));
    this.chunkBudget -= Date.now() - e, (h || this.chunkBudget <= 0) && (s.context.takeTree(), this.view.dispatch({ effects: St.setState.of(new oe(s.context)) })), this.chunkBudget > 0 && !(h && !o) && this.scheduleWork(), this.checkAsyncSchedule(s.context);
  }
  checkAsyncSchedule(t) {
    t.scheduleOn && (this.workScheduled++, t.scheduleOn.then(() => this.scheduleWork()).catch((e) => $t(this.view.state, e)).then(() => this.workScheduled--), t.scheduleOn = null);
  }
  destroy() {
    this.working && this.working();
  }
  isWorking() {
    return !!(this.working || this.workScheduled > 0);
  }
}, {
  eventHandlers: { focus() {
    this.scheduleWork();
  } }
}), he = /* @__PURE__ */ O.define({
  combine(r) {
    return r.length ? r[0] : null;
  },
  enables: (r) => [
    St.state,
    rh,
    T.contentAttributes.compute([r], (t) => {
      let e = t.facet(r);
      return e && e.name ? { "data-language": e.name } : {};
    })
  ]
});
function Wr(r, t) {
  let e = t.mapPos(r.from, 1), i = t.mapPos(r.to, -1);
  return e >= i ? void 0 : { from: e, to: i };
}
const lh = /* @__PURE__ */ q.define({ map: Wr }), zr = /* @__PURE__ */ q.define({ map: Wr }), qr = /* @__PURE__ */ Ft.define({
  create() {
    return H.none;
  },
  update(r, t) {
    r = r.map(t.changes);
    for (let e of t.effects)
      if (e.is(lh) && !hh(r, e.value.from, e.value.to)) {
        let { preparePlaceholder: i } = t.state.facet(Kr), n = i ? H.replace({ widget: new fh(i(t.state, e.value)) }) : ks;
        r = r.update({ add: [n.range(e.value.from, e.value.to)] });
      } else
        e.is(zr) && (r = r.update({
          filter: (i, n) => e.value.from != i || e.value.to != n,
          filterFrom: e.value.from,
          filterTo: e.value.to
        }));
    if (t.selection) {
      let e = !1, { head: i } = t.selection.main;
      r.between(i, i, (n, s) => {
        n < i && s > i && (e = !0);
      }), e && (r = r.update({
        filterFrom: i,
        filterTo: i,
        filter: (n, s) => s <= i || n >= i
      }));
    }
    return r;
  },
  provide: (r) => T.decorations.from(r),
  toJSON(r, t) {
    let e = [];
    return r.between(0, t.doc.length, (i, n) => {
      e.push(i, n);
    }), e;
  },
  fromJSON(r) {
    if (!Array.isArray(r) || r.length % 2)
      throw new RangeError("Invalid JSON for fold state");
    let t = [];
    for (let e = 0; e < r.length; ) {
      let i = r[e++], n = r[e++];
      if (typeof i != "number" || typeof n != "number")
        throw new RangeError("Invalid JSON for fold state");
      t.push(ks.range(i, n));
    }
    return H.set(t, !0);
  }
});
function xs(r) {
  return r.field(qr, !1) || W.empty;
}
function oh(r, t, e) {
  var i;
  let n = null;
  return (i = r.field(qr, !1)) === null || i === void 0 || i.between(t, e, (s, l) => {
    (!n || n.from > s) && (n = { from: s, to: l });
  }), n;
}
function hh(r, t, e) {
  let i = !1;
  return r.between(t, t, (n, s) => {
    n == t && s == e && (i = !0);
  }), i;
}
const ah = {
  placeholderDOM: null,
  preparePlaceholder: null,
  placeholderText: "…"
}, Kr = /* @__PURE__ */ O.define({
  combine(r) {
    return ol(r, ah);
  }
});
function jr(r, t) {
  let { state: e } = r, i = e.facet(Kr), n = (l) => {
    let o = r.lineBlockAt(r.posAtDOM(l.target)), h = oh(r.state, o.from, o.to);
    h && r.dispatch({ effects: zr.of(h) }), l.preventDefault();
  };
  if (i.placeholderDOM)
    return i.placeholderDOM(r, n, t);
  let s = document.createElement("span");
  return s.textContent = i.placeholderText, s.setAttribute("aria-label", e.phrase("folded code")), s.title = e.phrase("unfold"), s.className = "cm-foldPlaceholder", s.onclick = n, s;
}
const ks = /* @__PURE__ */ H.replace({ widget: /* @__PURE__ */ new class extends De {
  toDOM(r) {
    return jr(r, null);
  }
}() });
class fh extends De {
  constructor(t) {
    super(), this.value = t;
  }
  eq(t) {
    return this.value == t.value;
  }
  toDOM(t) {
    return jr(t, this.value);
  }
}
const ch = /* @__PURE__ */ Object.create(null), vs = [ot.none], Ss = [], uh = /* @__PURE__ */ Object.create(null);
for (let [r, t] of [
  ["variable", "variableName"],
  ["variable-2", "variableName.special"],
  ["string-2", "string.special"],
  ["def", "variableName.definition"],
  ["tag", "tagName"],
  ["attribute", "attributeName"],
  ["type", "typeName"],
  ["builtin", "variableName.standard"],
  ["qualifier", "modifier"],
  ["error", "invalid"],
  ["header", "heading"],
  ["property", "propertyName"]
])
  uh[r] = /* @__PURE__ */ dh(ch, t);
function Ci(r, t) {
  Ss.indexOf(r) > -1 || (Ss.push(r), console.warn(t));
}
function dh(r, t) {
  let e = null;
  for (let s of t.split(".")) {
    let l = r[s] || M[s];
    l ? typeof l == "function" ? e ? e = l(e) : Ci(s, `Modifier ${s} used at start of tag`) : e ? Ci(s, `Tag ${s} used as modifier`) : e = l : Ci(s, `Unknown highlighting tag ${s}`);
  }
  if (!e)
    return 0;
  let i = t.replace(/ /g, "_"), n = ot.define({
    id: vs.length,
    name: i,
    props: [th({ [i]: e })]
  });
  return vs.push(n), n.id;
}
const Sh = (r) => [
  ph,
  r != null && r.styles ? T.theme(r.styles) : gh
], gh = T.theme({
  ".cm-bold": {
    fontWeight: "bold"
  }
}), ph = Tt.fromClass(
  class {
    constructor(t) {
      this.decorations = As(t);
    }
    update(t) {
      (t.docChanged || t.viewportChanged || t.selectionSet) && (this.decorations = As(t.view));
    }
  },
  {
    decorations: (r) => r.decorations
  }
), As = (r) => {
  const t = [], e = r.state.selection.ranges;
  for (const { from: i, to: n } of r.visibleRanges)
    le(r.state).iterate({
      enter: ({ type: s, from: l, to: o }) => {
        if (s.name !== "StrongEmphasis")
          return;
        const h = r.lineBlockAt(l);
        e.some(
          ({ from: f, to: c }) => f <= h.to && c >= h.from
        ) || (t.push(H.mark({ class: "cm-bold" }).range(l, o)), t.push(H.replace({}).range(l, l + 2)), t.push(H.replace({}).range(o - 2, o)));
      },
      from: i,
      to: n
    });
  return H.set(t, !0);
}, Ch = (r) => [
  yh,
  r != null && r.styles ? T.theme(r.styles) : mh
], mh = T.theme({
  ".cm-heading": { fontWeight: "bold" },
  ".cm-heading-1": { fontSize: "2.00em" },
  ".cm-heading-2": { fontSize: "1.50em" },
  ".cm-heading-3": { fontSize: "1.17em" },
  ".cm-heading-4": { fontSize: "1.00em" },
  ".cm-heading-5": { fontSize: "0.83rem" },
  ".cm-heading-6": { fontSize: "0.67rem" }
}), yh = Tt.fromClass(
  class {
    constructor(t) {
      this.decorations = Cs(t);
    }
    update(t) {
      (t.docChanged || t.viewportChanged || t.selectionSet) && (this.decorations = Cs(t.view));
    }
  },
  {
    decorations: (r) => r.decorations
  }
), Cs = (r) => {
  const t = [], e = r.state.selection.ranges;
  for (const { from: i, to: n } of r.visibleRanges)
    le(r.state).iterate({
      enter: ({ type: s, name: l, from: o, to: h }) => {
        const a = r.lineBlockAt(o);
        if (!e.some(
          ({ from: c, to: u }) => c <= a.to && u >= a.from
        )) {
          if (s.name.includes("Heading")) {
            const c = parseInt(/[1-6]$/.exec(l)[0]), u = H.line({
              class: ["cm-heading", wh(c)].join(" ")
            });
            t.push(u.range(r.state.doc.lineAt(o).from));
          } else if (s.name === "HeaderMark" && /[#]/.test(r.state.sliceDoc(o, h))) {
            const c = H.replace({});
            t.push(c.range(o, h + 1));
          }
        }
      },
      from: i,
      to: n
    });
  return H.set(t, !0);
}, wh = (r) => {
  switch (r) {
    case 1:
      return "cm-heading-1";
    case 2:
      return "cm-heading-2";
    case 3:
      return "cm-heading-3";
    case 4:
      return "cm-heading-4";
    case 5:
      return "cm-heading-5";
    case 6:
      return "cm-heading-6";
    default:
      return "";
  }
}, Oh = (r) => [
  xh,
  r != null && r.styles ? T.theme(r.styles) : bh
], bh = T.theme({
  ".cm-block": {
    margin: "0 auto"
  },
  ".cm-block-begin": {
    borderRadius: "2px 2px 0 0"
  },
  ".cm-block-end": {
    borderRadius: "0 0 2px 2px"
  }
}), xh = Tt.fromClass(
  class {
    constructor(t) {
      this.decorations = Ms(t);
    }
    update(t) {
      (t.docChanged || t.viewportChanged || t.selectionSet) && (this.decorations = Ms(t.view));
    }
  },
  {
    decorations: (r) => r.decorations
  }
), Ms = (r) => {
  const t = [];
  for (const { from: e, to: i } of r.visibleRanges)
    le(r.state).iterate({
      enter: ({ type: n, from: s, to: l }) => {
        if (!["FencedCode", "CodeBlock"].includes(n.name))
          return;
        let o = r.viewportLineBlocks.filter((a) => a.from <= l && s <= a.to);
        const h = xs(r.state).iter();
        for (; xs(r.state).iter().value; )
          o = o.filter((a) => !(h.from <= a.to && a.from <= h.to)), h.next();
        o.forEach((a, f) => {
          const c = H.line({ class: ["cm-block", f === 0 ? "cm-block-begin" : a.to === l ? "cm-block-end" : ""].join(" ") });
          t.push(c.range(a.from));
        });
      },
      from: e,
      to: i
    });
  return H.set(t, !0);
};
export {
  Oh as block,
  Sh as bold,
  Ch as heading
};
